@@ -21,12 +21,6 @@
 #include <GG/WndEvent.h>
 #include <boost/signals2/signal.hpp>
 
-
-namespace boost { namespace archive {
-    class xml_oarchive;
-    class xml_iarchive;
-} }
-
 namespace GG {
 
 class Cursor;
@@ -37,7 +31,7 @@ class Timer;
 struct GUIImpl;
 
 template <typename T>
-std::shared_ptr<T> LockAndResetIfExpired(std::weak_ptr<T>& ptr) {
+std::shared_ptr<T> LockAndResetIfExpired(std::weak_ptr<T>& ptr) noexcept {
     auto locked = ptr.lock();
     if (!locked)
         ptr.reset();
@@ -95,7 +89,7 @@ std::shared_ptr<T> LockAndResetIfExpired(std::weak_ptr<T>& ptr) {
     individual Wnd.
 
     <p>A note about "button-down-repeat".  When you click on the down-button
-    on a scroll-bar, you probably expect the the button's action (scrolling
+    on a scroll-bar, you probably expect the button's action (scrolling
     down one increment) to repeat when you hold down the button, much like the
     way kestrokes are repeated when you hold down a keyboard key.  This is in
     fact what happens, and it is accomplished by having the scroll and its
@@ -178,9 +172,9 @@ public:
     bool                        MouseLRSwapped() const;             ///< returns true if the left and right mouse button press events are set to be swapped before event handling. This is to facilitate left-handed mouse users semi-automatically.
     virtual std::string         ClipboardText() const;              ///< returns text stored in a clipboard
 
-    /** Returns the (begin, end) indices of the code points or char indices
-      * of the word-tokens in the given string. */
+    /** Returns the (begin, end) code point indices of the of the word-tokens in the given string. */
     std::vector<std::pair<CPSize, CPSize>>   FindWords(std::string_view str) const;
+    /** Returns the (begin, end) string indices of the of the word-tokens in the given string. */
     std::vector<std::pair<StrSize, StrSize>> FindWordsStringIndices(std::string_view str) const;
     std::vector<std::string_view>            FindWordsStringViews(std::string_view str) const;
 
@@ -223,7 +217,7 @@ public:
     virtual void    HandleSystemEvents() = 0;
 
     /** Event handler for GG events. */
-    void            HandleGGEvent(EventType event, Key key, std::uint32_t key_code_point,
+    void            HandleGGEvent(EventType event, Key key, uint32_t key_code_point,
                                   Flags<ModKey> mod_keys, Pt pos, Pt rel,
                                   std::string text = std::string());
 
@@ -241,7 +235,8 @@ public:
     /** Adds \p wnd onto the modal windows "stack".  Modal windows are owned by the GUI as a
         top-level window. */
     void            RegisterModal(std::shared_ptr<Wnd> wnd);
-    void            RunModal(std::shared_ptr<Wnd> wnd, bool& done);
+    void            RunModal(const bool& done);
+    void            RunModal(std::shared_ptr<Wnd> wnd);
     void            Remove(const std::shared_ptr<Wnd>& wnd);               ///< removes \a wnd from the z-list.  Removing a null pointer or removing the same window multiple times is a no-op.
     void            MoveUp(const std::shared_ptr<Wnd>& wnd);               ///< moves \a wnd to the top of the z-list
     void            MoveDown(const std::shared_ptr<Wnd>& wnd);             ///< moves \a wnd to the bottom of the z-list
@@ -294,24 +289,24 @@ public:
 
     /** Returns a shared_ptr to the desired font, supporting all printable
         ASCII characters. */
-    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts);
+    std::shared_ptr<Font> GetFont(std::string_view font_filename, unsigned int pts);
 
     /** Returns a shared_ptr to the desired font, supporting all printable
         ASCII characters, from the in-memory contents \a file_contents. */
-    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+    std::shared_ptr<Font> GetFont(std::string_view font_filename, unsigned int pts,
                                   const std::vector<uint8_t>& file_contents);
 
     /** Returns a shared_ptr to the desired font, supporting all the
         characters in the UnicodeCharsets in the range [first, last). */
     template <typename CharSetIter>
-    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+    std::shared_ptr<Font> GetFont(std::string_view font_filename, unsigned int pts,
                                   CharSetIter first, CharSetIter last);
 
     /** Returns a shared_ptr to the desired font, supporting all the
         characters in the UnicodeCharsets in the range [first, last), from the
         in-memory contents \a file_contents. */
     template <typename CharSetIter>
-    std::shared_ptr<Font> GetFont(const std::string& font_filename, unsigned int pts,
+    std::shared_ptr<Font> GetFont(std::string_view font_filename, unsigned int pts,
                                   const std::vector<uint8_t>& file_contents,
                                   CharSetIter first, CharSetIter last);
 
@@ -320,7 +315,7 @@ public:
 
     /** Removes the desired font from the managed pool; since shared_ptr's are
         used, the font may be deleted much later. */
-    void                       FreeFont(const std::string& font_filename, unsigned int pts);
+    void FreeFont(std::string_view font_filename, unsigned int pts);
 
     /** Adds an already-constructed texture to the managed pool \warning
         calling code <b>must not</b> delete \a texture; the texture pool will
@@ -328,11 +323,8 @@ public:
     std::shared_ptr<Texture> StoreTexture(Texture* texture, const std::string& texture_name);
 
     /** Adds an already-constructed texture to the managed pool. */
-    std::shared_ptr<Texture> StoreTexture(const std::shared_ptr<Texture> &texture, const std::string& texture_name);
-
-    /** Loads the requested texture from file \a name; mipmap textures are
-      * generated if \a mipmap is true. */
-    std::shared_ptr<Texture> GetTexture(const std::string& name, bool mipmap = false);
+    std::shared_ptr<Texture> StoreTexture(const std::shared_ptr<Texture>& texture,
+                                          const std::string& texture_name);
 
     /** Loads the requested texture from file \a name; mipmap textures are
       * generated if \a mipmap is true. */
@@ -340,11 +332,7 @@ public:
 
     /** Removes the desired texture from the managed pool; since shared_ptr's
       * are used, the texture may be deleted much later. */
-    void                       FreeTexture(const std::string& name);
-
-    /** Removes the desired texture from the managed pool; since shared_ptr's
-      * are used, the texture may be deleted much later. */
-    void                       FreeTexture(const boost::filesystem::path& path);
+    void FreeTexture(const boost::filesystem::path& path);
 
     /** Sets the currently-installed style factory. */
     void SetStyleFactory(const std::shared_ptr<StyleFactory>& factory);
@@ -370,7 +358,7 @@ public:
     bool SetPrevFocusWndInCycle();                          ///< sets the focus Wnd to the next INTERACTIVE Wnd in a cycle determined by Wnd parent-child relationships
     bool SetNextFocusWndInCycle();                          ///< sets the focus Wnd to the next in the cycle.
 
-    static GUI*  GetGUI();                  ///< allows any GG code access to GUI framework by calling GUI::GetGUI()
+    static GUI*  GetGUI() noexcept;                         ///< allows any GG code access to GUI framework by calling GUI::GetGUI()
 
     /** If \p wnd is visible recursively call PreRenderWindow() on all \p wnd's children and then
         call \p wnd->PreRender().  The order guarantees that when wnd->PreRender() is called all
@@ -466,12 +454,12 @@ bool GUI::OrCombiner::operator()(InIt first, InIt last) const
 }
 
 template <typename CharSetIter>
-std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
+std::shared_ptr<Font> GUI::GetFont(std::string_view font_filename, unsigned int pts,
                                    CharSetIter first, CharSetIter last)
 { return GetFontManager().GetFont(font_filename, pts, first, last); }
 
 template <typename CharSetIter>
-std::shared_ptr<Font> GUI::GetFont(const std::string& font_filename, unsigned int pts,
+std::shared_ptr<Font> GUI::GetFont(std::string_view font_filename, unsigned int pts,
                                    const std::vector<uint8_t>& file_contents,
                                    CharSetIter first, CharSetIter last)
 { return GetFontManager().GetFont(font_filename, pts, file_contents, first, last); }
