@@ -23,12 +23,11 @@ class TextBlock : public GG::BlockControl
 {
 public:
     //! Create a new TextBlock.
-    TextBlock(X x, Y y, X w, const std::string& str, const std::shared_ptr<Font>& font, Clr color,
+    TextBlock(X x, Y y, X w, std::string str, std::shared_ptr<Font> font, Clr color,
               Flags<TextFormat> format, Flags<WndFlag> flags);
 
     void CompleteConstruction() override;
-    void Render() override
-    {};
+    void Render() override {}
 
     //! Calculate the size based on the width it should take.
     Pt SetMaxWidth(X width) override;
