@@ -21,6 +21,36 @@
 using namespace GG;
 
 namespace {
+enum class TestEnum : int8_t { INVALID = -1, VALID = 0, MAX = 1, OUT_OF_RANGE, END, ZERO = 0, ONE, TWO, THREE };
+constexpr auto* test_enum_text = "INVALID = -1, VALID = 0, MAX = 1, OUT_OF_RANGE, END";
+
+constexpr auto split_result = GG::EnumMap<TestEnum>::Split(test_enum_text, ',');
+static_assert(split_result.first == "INVALID = -1");
+static_assert(split_result.second == " VALID = 0, MAX = 1, OUT_OF_RANGE, END");
+
+constexpr auto trimmed_result = GG::EnumMap<TestEnum>::Trim(split_result.first);
+static_assert(trimmed_result == "INVALID = -1");
+
+constexpr auto split_apply_result = GG::EnumMap<TestEnum>::SplitApply(
+    test_enum_text, GG::EnumMap<TestEnum>::Trim, ',');
+static_assert(split_apply_result.first == 5);
+static_assert(split_apply_result.second[0] == "INVALID = -1");
+static_assert(split_apply_result.second[1] == "VALID = 0");
+static_assert(split_apply_result.second[2] == "MAX = 1");
+static_assert(split_apply_result.second[3] == "OUT_OF_RANGE");
+static_assert(split_apply_result.second[4] == "END");
+
+constexpr GG::EnumMap<TestEnum> cmap(test_enum_text);
+static_assert(cmap["MAX"] == TestEnum::MAX);
+static_assert(cmap["OUT_OF_RANGE"] == TestEnum::OUT_OF_RANGE);
+
+
+constexpr auto em = GG::CGetEnumMap<GG::WndRegion>();
+constexpr auto qq = em[GG::WndRegion::WR_TOPLEFT];
+constexpr auto rr = GG::to_string(GG::WndRegion::WR_TOPLEFT);
+static_assert(rr == "WR_TOPLEFT");
+static_assert(rr == qq);
+
 
 namespace mi = boost::multi_index;
 
@@ -28,34 +58,34 @@ struct GridLayoutWnd
 {
     GridLayoutWnd() {}
 
-    GridLayoutWnd(std::shared_ptr<Wnd>& wnd_, const Pt& ul_, const Pt& lr_) : wnd(wnd_), ul(ul_), lr(lr_) {}
+    GridLayoutWnd(std::shared_ptr<Wnd>& wnd_, Pt ul_, Pt lr_) : wnd(wnd_), ul(ul_), lr(lr_) {}
     std::shared_ptr<Wnd> wnd;
     Pt ul;
     Pt lr;
 };
 struct IsLeft
 {
-    bool operator()(const Pt& lhs, const Pt& rhs) const {return lhs.x < rhs.x;}
-    bool operator()(X x, const Pt& pt) const            {return x < pt.x;}
-    bool operator()(const Pt& pt, X x) const            {return pt.x < x;}
+    bool operator()(Pt lhs, Pt rhs) const {return lhs.x < rhs.x;}
+    bool operator()(X x, Pt pt) const     {return x < pt.x;}
+    bool operator()(Pt pt, X x) const     {return pt.x < x;}
 };
 struct IsTop
 {
-    bool operator()(const Pt& lhs, const Pt& rhs) const {return lhs.y < rhs.y;}
-    bool operator()(Y y, const Pt& pt) const            {return y < pt.y;}
-    bool operator()(const Pt& pt, Y y) const            {return pt.y < y;}
+    bool operator()(Pt lhs, Pt rhs) const {return lhs.y < rhs.y;}
+    bool operator()(Y y, Pt pt) const     {return y < pt.y;}
+    bool operator()(Pt pt, Y y) const     {return pt.y < y;}
 };
 struct IsRight
 {
-    bool operator()(const Pt& lhs, const Pt& rhs) const {return rhs.x < lhs.x;}
-    bool operator()(X x, const Pt& pt) const            {return pt.x < x;}
-    bool operator()(const Pt& pt, X x) const            {return x < pt.x;}
+    bool operator()(Pt lhs, Pt rhs) const {return rhs.x < lhs.x;}
+    bool operator()(X x, Pt pt) const     {return pt.x < x;}
+    bool operator()(Pt pt, X x) const     {return x < pt.x;}
 };
 struct IsBottom
 {
-    bool operator()(const Pt& lhs, const Pt& rhs) const {return rhs.y < lhs.y;}
-    bool operator()(Y y, const Pt& pt) const            {return pt.y < y;}
-    bool operator()(const Pt& pt, Y y) const            {return y < pt.y;}
+    bool operator()(Pt lhs, Pt rhs) const {return rhs.y < lhs.y;}
+    bool operator()(Y y, Pt pt) const     {return pt.y < y;}
+    bool operator()(Pt pt, Y y) const     {return y < pt.y;}
 };
 struct Pointer {};
 struct LayoutLeft {};
@@ -165,28 +195,28 @@ bool Wnd::PreRenderRequired() const
     if (m_needs_prerender)
         return true;
 
-    auto&& layout = GetLayout();
+    auto layout = GetLayout();
 
     return (layout && layout->m_needs_prerender);
 }
 
 void Wnd::DropsAcceptable(DropsAcceptableIter first, DropsAcceptableIter last,
-                          const Pt& pt, Flags<ModKey> mod_keys) const
+                          Pt pt, Flags<ModKey> mod_keys) const
 {
     // default reject all drops. derived classes can override to accept drops
     for (auto& it = first; it != last; ++it)
         it->second = false;
 }
 
-Pt Wnd::UpperLeft() const
+Pt Wnd::UpperLeft() const noexcept
 {
     Pt retval = m_upperleft;
-    if (auto&& parent = Parent())
+    if (auto parent = Parent())
         retval += parent->ClientUpperLeft();
     return retval;
 }
 
-Pt Wnd::LowerRight() const
+Pt Wnd::LowerRight() const noexcept
 {
     Pt retval = m_lowerright;
     if (auto&& parent = Parent())
@@ -196,41 +226,14 @@ Pt Wnd::LowerRight() const
 
 Pt Wnd::MinUsableSize() const
 {
-    auto&& layout = GetLayout();
+    auto layout = GetLayout();
     return layout ? layout->MinUsableSize() : Size();
 }
 
-Pt Wnd::ClientUpperLeft() const
-{ return UpperLeft(); }
-
-Pt Wnd::ClientLowerRight() const
-{ return LowerRight(); }
-
-Pt Wnd::ClientSize() const
-{ return ClientLowerRight() - ClientUpperLeft(); }
-
-X Wnd::ClientWidth() const
-{ return ClientLowerRight().x - ClientUpperLeft().x; }
-
-Y Wnd::ClientHeight() const
-{ return ClientLowerRight().y - ClientUpperLeft().y; }
-
-Pt Wnd::ScreenToWindow(const Pt& pt) const
-{ return pt - UpperLeft(); }
-
-Pt Wnd::ScreenToClient(const Pt& pt) const
-{ return pt - ClientUpperLeft(); }
-
-bool Wnd::InWindow(const Pt& pt) const
-{ return pt >= UpperLeft() && pt < LowerRight(); }
-
-bool Wnd::InClient(const Pt& pt) const
-{ return pt >= ClientUpperLeft() && pt < ClientLowerRight(); }
-
-std::shared_ptr<Wnd> Wnd::Parent() const
+std::shared_ptr<Wnd> Wnd::Parent() const noexcept
 { return LockAndResetIfExpired(m_parent); }
 
-bool Wnd::IsAncestorOf(const std::shared_ptr<Wnd>& wnd) const
+bool Wnd::IsAncestorOf(const std::shared_ptr<Wnd>& wnd) const noexcept
 {
     // Is this Wnd one of wnd's (direct or indirect) parents?
     if (!wnd)
@@ -244,7 +247,7 @@ bool Wnd::IsAncestorOf(const std::shared_ptr<Wnd>& wnd) const
     return false;
 }
 
-std::shared_ptr<Wnd> Wnd::RootParent() const
+std::shared_ptr<Wnd> Wnd::RootParent() const noexcept
 {
     auto parent{Parent()};
     auto gparent{parent ? parent->Parent() : nullptr};
@@ -255,16 +258,16 @@ std::shared_ptr<Wnd> Wnd::RootParent() const
     return parent;
 }
 
-std::shared_ptr<Layout> Wnd::GetLayout() const
+std::shared_ptr<Layout> Wnd::GetLayout() const noexcept
 { return LockAndResetIfExpired(m_layout); }
 
-Layout* Wnd::ContainingLayout() const
+Layout* Wnd::ContainingLayout() const noexcept
 { return LockAndResetIfExpired(m_containing_layout).get(); }
 
 const std::shared_ptr<StyleFactory>& Wnd::GetStyleFactory() const
 { return m_style_factory ? m_style_factory : GUI::GetGUI()->GetStyleFactory(); }
 
-WndRegion Wnd::WindowRegion(const Pt& pt) const
+WndRegion Wnd::WindowRegion(Pt pt) const
 {
     static constexpr int LEFT = 0;
     static constexpr int MIDDLE = 1;
@@ -299,8 +302,8 @@ void Wnd::ClampRectWithMinAndMaxSize(Pt& ul, Pt& lr) const
 {
     Pt min_sz = MinSize();
     Pt max_sz = MaxSize();
-    auto&& layout = GetLayout();
-    if (layout) {
+
+    if (auto layout = GetLayout()) {
         Pt layout_min_sz = layout->MinSize() + (Size() - ClientSize());
         min_sz.x = std::max(min_sz.x, layout_min_sz.x);
         min_sz.y = std::max(min_sz.y, layout_min_sz.y);
@@ -329,7 +332,7 @@ void Wnd::ClampRectWithMinAndMaxSize(Pt& ul, Pt& lr) const
     }
 }
 
-void Wnd::AcceptDrops(const Pt& pt, std::vector<std::shared_ptr<Wnd>> wnds, Flags<ModKey> mod_keys)
+void Wnd::AcceptDrops(Pt pt, std::vector<std::shared_ptr<Wnd>> wnds, Flags<ModKey> mod_keys)
 {
     if (!Interactive() && Parent())
         ForwardEventToParent();
@@ -356,25 +359,25 @@ void Wnd::Show()
         child->Show();
 }
 
-void Wnd::MoveTo(const Pt& pt)
+void Wnd::MoveTo(Pt pt)
 { SizeMove(pt, pt + Size()); }
 
-void Wnd::OffsetMove(const Pt& pt)
+void Wnd::OffsetMove(Pt pt)
 { SizeMove(m_upperleft + pt, m_lowerright + pt); }
 
-void Wnd::SizeMove(const Pt& ul_, const Pt& lr_)
+void Wnd::SizeMove(Pt ul_, Pt lr_)
 {
-    Pt ul = ul_, lr = lr_;
-    Pt original_sz = Size();
-    bool resized = (original_sz != (lr - ul));
+    auto ul = ul_, lr = lr_;
+    const auto original_sz = Size();
+    const bool resized = (original_sz != (lr - ul));
     if (resized)
         ClampRectWithMinAndMaxSize(ul, lr);
 
     m_upperleft = ul;
     m_lowerright = lr;
     if (resized) {
-        bool size_changed = Size() != original_sz;
-        auto&& layout = GetLayout();
+        const bool size_changed = Size() != original_sz;
+        auto layout = GetLayout();
         if (layout && size_changed)
             layout->Resize(ClientSize());
         if (size_changed && !dynamic_cast<Layout*>(this))
@@ -383,23 +386,23 @@ void Wnd::SizeMove(const Pt& ul_, const Pt& lr_)
     }
 }
 
-void Wnd::Resize(const Pt& sz)
+void Wnd::Resize(Pt sz)
 { SizeMove(m_upperleft, m_upperleft + sz); }
 
-void Wnd::SetMinSize(const Pt& sz)
+void Wnd::SetMinSize(Pt sz)
 {
-    bool min_size_changed = m_min_size != sz;
+    const bool min_size_changed = m_min_size != sz;
     m_min_size = sz;
-    if (Width() < m_min_size.x || Height() < m_min_size.y)
+    if (Width() < m_min_size.x || Height() < m_min_size.y) {
         Resize(Pt(std::max(Width(), m_min_size.x), std::max(Height(), m_min_size.y)));
     // The previous Resize() will call ChildSizeOrMinSizeChanged() itself if needed
-    else if (min_size_changed && !dynamic_cast<Layout*>(this)) {
+    } else if (min_size_changed && !dynamic_cast<Layout*>(this)) {
         if (auto&& containing_layout = LockAndResetIfExpired(m_containing_layout))
             containing_layout->ChildSizeOrMinSizeChanged();
     }
 }
 
-void Wnd::SetMaxSize(const Pt& sz)
+void Wnd::SetMaxSize(Pt sz)
 {
     m_max_size = sz;
     if (m_max_size.x < Width() || m_max_size.y < Height())
@@ -517,7 +520,7 @@ void Wnd::InstallEventFilter(std::shared_ptr<Wnd> wnd)
         return;
     RemoveEventFilter(wnd);
 
-    m_filters.emplace_back(std::move(wnd));
+    m_filters.push_back(wnd);
     wnd->m_filtering.insert(shared_from_this());
 }
 
@@ -525,8 +528,8 @@ void Wnd::RemoveEventFilter(const std::shared_ptr<Wnd>& wnd)
 {
     if (!wnd)
         return;
-    const auto& it = std::find_if(m_filters.begin(), m_filters.end(),
-                                  [&wnd](const std::weak_ptr<Wnd>& x){ return x.lock() == wnd; });
+    const auto it = std::find_if(m_filters.begin(), m_filters.end(),
+                                 [&wnd](const std::weak_ptr<Wnd>& x){ return x.lock() == wnd; });
     if (it != m_filters.end())
         m_filters.erase(it);
     wnd->m_filtering.erase(shared_from_this());
@@ -541,7 +544,7 @@ void Wnd::HorizontalLayout()
     Pt client_sz = ClientSize();
     for (auto& child : m_children) {
         Pt wnd_ul = child->RelativeUpperLeft(), wnd_lr = child->RelativeLowerRight();
-        if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
+        if (wnd_ul.x < X0 || wnd_ul.y < Y0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             continue;
         wnds.push_back(child);
     }
@@ -567,7 +570,7 @@ void Wnd::VerticalLayout()
     Pt client_sz = ClientSize();
     for (auto& child : m_children) {
         Pt wnd_ul = child->RelativeUpperLeft(), wnd_lr = child->RelativeLowerRight();
-        if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
+        if (wnd_ul.x < X0 || wnd_ul.y < Y0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             continue;
         wnds.push_back(child);
     }
@@ -596,7 +599,7 @@ void Wnd::GridLayout()
     for (auto it = m_children.begin(); it != m_children.end(); ++it) {
         auto& wnd = *it;
         Pt wnd_ul = wnd->RelativeUpperLeft(), wnd_lr = wnd->RelativeLowerRight();
-        if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
+        if (wnd_ul.x < X0 || wnd_ul.y < Y0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             continue;
 
         auto it2 = it;
@@ -616,7 +619,7 @@ void Wnd::GridLayout()
          it != grid_layout.get<LayoutLeft>().end(); ++it)
     {
         Pt ul = it->ul;
-        for (X x = ul.x - 1; x >= 0; --x) {
+        for (X x = ul.x - 1; x >= X0; --x) {
             if (grid_layout.get<LayoutRight>().count(x + 1, IsRight())) {
                 break;
             } else if (grid_layout.get<LayoutLeft>().count(x, IsLeft())) {
@@ -646,7 +649,7 @@ void Wnd::GridLayout()
     // align tops of windows
     for (TopIter it = grid_layout.get<LayoutTop>().begin(); it != grid_layout.get<LayoutTop>().end(); ++it) {
         Pt ul = it->ul;
-        for (Y y = ul.y - 1; y >= 0; --y) {
+        for (Y y = ul.y - Y1; y >= Y0; --y) {
             if (grid_layout.get<LayoutBottom>().count(y + 1, IsBottom())) {
                 break;
             } else if (grid_layout.get<LayoutTop>().count(y, IsTop())) {
@@ -693,13 +696,17 @@ void Wnd::GridLayout()
     // populate this new layout with the child windows, based on their placements in the pixel-grid layout
     for (const GridLayoutWnd& layout_wnd : grid_layout.get<Pointer>()) {
         auto& wnd = layout_wnd.wnd;
-        Pt ul = layout_wnd.ul;
-        Pt lr = layout_wnd.lr;
-        int left = std::distance(unique_lefts.begin(), unique_lefts.find(ul.x));
-        int top = std::distance(unique_tops.begin(), unique_tops.find(ul.y));
-        int right = std::distance(unique_lefts.begin(), unique_lefts.lower_bound(lr.x));
-        int bottom = std::distance(unique_tops.begin(), unique_tops.lower_bound(lr.y));
-        layout->Add(wnd, top, left, bottom - top, right - left);
+        const auto ul = layout_wnd.ul;
+        const auto lr = layout_wnd.lr;
+        const auto left = std::distance(unique_lefts.begin(), unique_lefts.find(ul.x));
+        const auto top = std::distance(unique_tops.begin(), unique_tops.find(ul.y));
+        const auto right = std::distance(unique_lefts.begin(), unique_lefts.lower_bound(lr.x));
+        const auto bottom = std::distance(unique_tops.begin(), unique_tops.lower_bound(lr.y));
+        const auto height_sz = static_cast<size_t>((bottom >= top) ? (bottom - top) : 0u);
+        const auto width_sz = static_cast<size_t>((right >= left) ? (right - left) : 0u);
+        const auto top_sz = top > 0 ? static_cast<size_t>(top) : 0u;
+        const auto left_sz = left > 0 ? static_cast<size_t>(left) : 0u;
+        layout->Add(wnd, top_sz, left_sz, height_sz, width_sz);
     }
 }
 
@@ -709,12 +716,12 @@ void Wnd::SetLayout(const std::shared_ptr<Layout>& layout)
     if (layout == mm_layout || layout == LockAndResetIfExpired(m_containing_layout))
         throw BadLayout("Wnd::SetLayout() : Attempted to set a Wnd's layout to be its current layout or the layout that contains the Wnd");
     RemoveLayout();
-    auto children = m_children;
+    const auto children{m_children};
     DetachChildren();
     Pt client_sz = ClientSize();
     for (auto& wnd : children) {
         Pt wnd_ul = wnd->RelativeUpperLeft(), wnd_lr = wnd->RelativeLowerRight();
-        if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
+        if (wnd_ul.x < X0 || wnd_ul.y < Y0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             AttachChild(wnd);
     }
     AttachChild(layout);
@@ -724,16 +731,16 @@ void Wnd::SetLayout(const std::shared_ptr<Layout>& layout)
 
 void Wnd::SetLayout(std::shared_ptr<Layout>&& layout)
 {
-    auto&& mm_layout = GetLayout();
+    auto mm_layout = GetLayout();
     if (layout == mm_layout || layout == LockAndResetIfExpired(m_containing_layout))
         throw BadLayout("Wnd::SetLayout() : Attempted to set a Wnd's layout to be its current layout or the layout that contains the Wnd");
     RemoveLayout();
-    auto children = m_children;
+    const auto children{m_children};
     DetachChildren();
-    Pt client_sz = ClientSize();
+    const Pt client_sz = ClientSize();
     for (auto& wnd : children) {
         Pt wnd_ul = wnd->RelativeUpperLeft(), wnd_lr = wnd->RelativeLowerRight();
-        if (wnd_ul.x < 0 || wnd_ul.y < 0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
+        if (wnd_ul.x < X0 || wnd_ul.y < Y0 || client_sz.x < wnd_lr.x || client_sz.y < wnd_lr.y)
             AttachChild(wnd);
     }
     AttachChild(layout);
@@ -783,25 +790,18 @@ void Wnd::PreRender()
         layout->PreRender();
 }
 
-void Wnd::RequirePreRender()
-{ m_needs_prerender = true; }
-
-void Wnd::Render() {}
-
 bool Wnd::Run()
 {
-    bool retval = false;
-    auto parent{Parent()};
-    if (!parent && m_flags & MODAL) {
+    if ((m_flags & MODAL) && !Parent()) {
         GUI* gui = GUI::GetGUI();
         gui->RegisterModal(shared_from_this());
         ModalInit();
-        m_done = false;
-        gui->RunModal(shared_from_this(), m_done);
+        m_modal_done.store(false);
+        gui->RunModal(shared_from_this());
         gui->Remove(shared_from_this());
-        retval = true;
+        return true;
     }
-    return retval;
+    return false;
 }
 
 void Wnd::SetBrowseModeTime(unsigned int time, std::size_t mode)
@@ -846,10 +846,10 @@ Wnd::DragDropRenderingState Wnd::GetDragDropRenderingState() const
     return retval;
 }
 
-void Wnd::LButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::LButtonDown(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
+void Wnd::LDrag(Pt pt, Pt move, Flags<ModKey> mod_keys)
 {
     if (Dragable())
         OffsetMove(move);
@@ -857,68 +857,68 @@ void Wnd::LDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
         ForwardEventToParent();
 }
 
-void Wnd::LButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::LButtonUp(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::LClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::LClick(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::LDoubleClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::LDoubleClick(Pt pt, Flags<ModKey> mod_keys)
 { LClick(pt, mod_keys); }
 
-void Wnd::MButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MButtonDown(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
+void Wnd::MDrag(Pt pt, Pt move, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MButtonUp(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MClick(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MDoubleClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MDoubleClick(Pt pt, Flags<ModKey> mod_keys)
 { MClick(pt, mod_keys); }
 
-void Wnd::RButtonDown(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::RButtonDown(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::RDrag(const Pt& pt, const Pt& move, Flags<ModKey> mod_keys)
+void Wnd::RDrag(Pt pt, Pt move, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::RButtonUp(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::RButtonUp(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::RClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::RClick(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::RDoubleClick(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::RDoubleClick(Pt pt, Flags<ModKey> mod_keys)
 { RClick(pt, mod_keys); }
 
-void Wnd::MouseEnter(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MouseEnter(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MouseHere(const Pt& pt, Flags<ModKey> mod_keys)
+void Wnd::MouseHere(Pt pt, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
 void Wnd::MouseLeave()
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::MouseWheel(const Pt& pt, int move, Flags<ModKey> mod_keys)
+void Wnd::MouseWheel(Pt pt, int move, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::DragDropEnter(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
+void Wnd::DragDropEnter(Pt pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::DragDropHere(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
+void Wnd::DragDropHere(Pt pt, std::map<const Wnd*, bool>& drop_wnds_acceptable, Flags<ModKey> mod_keys)
 {
     if (!Interactive())
         ForwardEventToParent();
     this->DropsAcceptable(drop_wnds_acceptable.begin(), drop_wnds_acceptable.end(), pt, mod_keys);
 }
 
-void Wnd::CheckDrops(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_acceptable,
+void Wnd::CheckDrops(Pt pt, std::map<const Wnd*, bool>& drop_wnds_acceptable,
                      Flags<ModKey> mod_keys)
 {
     if (!Interactive())
@@ -929,10 +929,10 @@ void Wnd::CheckDrops(const Pt& pt, std::map<const Wnd*, bool>& drop_wnds_accepta
 void Wnd::DragDropLeave()
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Wnd::KeyPress(Key key, uint32_t key_code_point, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
-void Wnd::KeyRelease(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void Wnd::KeyRelease(Key key, uint32_t key_code_point, Flags<ModKey> mod_keys)
 { if (!Interactive()) ForwardEventToParent(); }
 
 void Wnd::TextInput(const std::string&)
@@ -1094,6 +1094,8 @@ void Wnd::BeginClippingImpl(ChildClippingMode mode)
     case ChildClippingMode::ClipToWindow:
         BeginScissorClipping(UpperLeft(), LowerRight());
         break;
+    case Wnd::ChildClippingMode::ClipToAncestorClient:
+        break;
     }
 }
 
@@ -1108,6 +1110,8 @@ void Wnd::EndClippingImpl(ChildClippingMode mode)
     case ChildClippingMode::ClipToClientAndWindowSeparately:
         EndScissorClipping();
         break;
+    case Wnd::ChildClippingMode::ClipToAncestorClient:
+        break;
     }
 }
 
