@@ -7,9 +7,8 @@
 //! Some Rights Reserved.  See COPYING file or https://www.gnu.org/licenses/lgpl-2.1.txt
 //! SPDX-License-Identifier: LGPL-2.1-or-later
 
-#include <boost/cast.hpp>
 #include <boost/algorithm/string/predicate.hpp>
-#include <boost/filesystem/operations.hpp>
+#include <boost/filesystem.hpp>
 #include <boost/format.hpp>
 // boost::spirit::classic pulls in windows.h which in turn defines macro
 // versions of min and max.  Defining NOMINMAX disables the creation of those
@@ -45,75 +44,92 @@ using namespace boost::spirit::classic;
 // these functors are used by the if_p, while_p, and for_p parsers in UpdateList()
 struct LeadingWildcard
 {
-    LeadingWildcard(const std::string& str) : m_value(!str.empty() && *str.begin() == '*') {}
-    bool operator()() const {return m_value;}
-    bool m_value;
+    constexpr LeadingWildcard(std::string_view str) noexcept :
+        m_value(!str.empty() && str.front() == '*')
+    {}
+    constexpr bool operator()() const noexcept { return m_value; }
+    const bool m_value;
 };
 struct TrailingWildcard
 {
-    TrailingWildcard(const std::string& str) : m_value(!str.empty() && *str.rbegin() == '*') {}
-    bool operator()() const {return m_value;}
-    bool m_value;
+    constexpr TrailingWildcard(std::string_view str) noexcept :
+        m_value(!str.empty() && *str.rbegin() == '*')
+    {}
+    constexpr bool operator()() const noexcept { return m_value; }
+    const bool m_value;
 };
 
 struct Index
 {
-    Index(int i = 0) : m_initial_value(i) {}
-    void operator()() const {value = m_initial_value;}
-    int m_initial_value;
-    static int value;
+    constexpr Index(std::size_t i = 0) noexcept :
+        m_initial_value(i)
+    {}
+    constexpr void operator()() const noexcept { value = m_initial_value; }
+    const std::size_t m_initial_value;
+    static std::size_t value;
 };
-int Index::value;
+std::size_t Index::value{};
+
 struct IndexLess
 {
-    IndexLess(int val) : m_value(val) {}
-    bool operator()() const {return Index::value <  m_value;}
-    int m_value;
+    constexpr IndexLess(std::size_t val) noexcept :
+        m_value(val)
+    {}
+    bool operator()() const noexcept { return Index::value < m_value; }
+    const std::size_t m_value;
 };
 struct IndexIncr
 {
-    void operator()() const {++Index::value;}
+    void operator()() const noexcept { ++Index::value; }
 };
 
 struct FrontStringBegin
 {
-    FrontStringBegin(const std::shared_ptr<std::vector<std::string>>& strings) :
-        m_strings(strings)
+    FrontStringBegin(std::shared_ptr<std::vector<std::string>> strings) noexcept :
+        m_strings(std::move(strings))
     {}
 
-    const char* operator()() const {return m_strings->front().c_str();}
+    const char* operator()() const noexcept
+    { return m_strings && !m_strings->empty() ? m_strings->front().c_str() : ""; }
 
-    std::shared_ptr<std::vector<std::string>> m_strings;
+    const std::shared_ptr<std::vector<std::string>> m_strings;
 };
 struct FrontStringEnd
 {
-    FrontStringEnd(const std::shared_ptr<std::vector<std::string>>& strings) :
-        m_strings(strings)
+    FrontStringEnd(std::shared_ptr<std::vector<std::string>> strings) noexcept :
+        m_strings(std::move(strings))
     {}
 
-    const char* operator()() const {return m_strings->front().c_str() + m_strings->front().size();}
+    const char* operator()() const noexcept
+    { return m_strings && !m_strings->empty() ? std::next(m_strings->front().c_str(), m_strings->front().size()) : ""; }
 
-    std::shared_ptr<std::vector<std::string>> m_strings;
+    const std::shared_ptr<std::vector<std::string>> m_strings;
 };
 struct IndexedStringBegin
 {
-    IndexedStringBegin(const std::shared_ptr<std::vector<std::string>>& strings) :
-        m_strings(strings)
+    IndexedStringBegin(std::shared_ptr<std::vector<std::string>> strings) noexcept :
+        m_strings(std::move(strings))
     {}
 
-    const char* operator()() const {return (*m_strings)[Index::value].c_str();}
+    const char* operator()() const noexcept
+    { return m_strings && m_strings->size() > Index::value ? (*m_strings)[Index::value].c_str() : ""; }
 
-    std::shared_ptr<std::vector<std::string>> m_strings;
+    const std::shared_ptr<std::vector<std::string>> m_strings;
 };
 struct IndexedStringEnd
 {
-    IndexedStringEnd(const std::shared_ptr<std::vector<std::string>>& strings) :
-        m_strings(strings)
+    IndexedStringEnd(std::shared_ptr<std::vector<std::string>> strings) noexcept :
+        m_strings(std::move(strings))
     {}
 
-    const char* operator()() const {return (*m_strings)[Index::value].c_str() + (*m_strings)[Index::value].size();}
+    const char* operator()() const noexcept {
+        if (!m_strings)
+            return "";
+        const auto& s{*m_strings};
+        return (s.size() > Index::value) ? std::next(s[Index::value].c_str(), s[Index::value].size()) : "";
+    }
 
-    std::shared_ptr<std::vector<std::string>> m_strings;
+    const std::shared_ptr<std::vector<std::string>> m_strings;
 };
 
 bool WindowsRoot(const std::string& root_name)
@@ -150,13 +166,11 @@ FileDlg::FileDlg(const std::string& directory, const std::string& filename, bool
     m_init_directory(directory),
     m_init_filename(filename)
 {
-    const auto& style = GetStyleFactory();
-
     if (m_save)
         multi = false;
 
     // finally, we can create the listbox with the files in it, sized to fill the available space
-    m_files_list = style->NewListBox(m_border_color);
+    m_files_list = GetStyleFactory()->NewListBox(m_border_color);
     m_files_list->SetStyle(LIST_NOSORT | (multi ? LIST_NONE : LIST_SINGLESEL));
 }
 
@@ -206,6 +220,7 @@ void FileDlg::CompleteConstruction()
 
     namespace ph = boost::placeholders;
 
+    // TODO: store scoped connections in member vector?
     m_ok_button->LeftClickedSignal.connect(boost::bind(&FileDlg::OkClicked, this));
     m_cancel_button->LeftClickedSignal.connect(boost::bind(&FileDlg::CancelClicked, this));
     m_files_list->SelRowsChangedSignal.connect(boost::bind(&FileDlg::FileSetChanged, this, ph::_1));
@@ -215,19 +230,10 @@ void FileDlg::CompleteConstruction()
 
     if (!m_init_filename.empty()) {
         fs::path filename_path = fs::system_complete(fs::path(m_init_filename));
-        m_files_edit->SetText(filename_path.leaf().string());
+        m_files_edit->SetText(filename_path.filename().string());
     }
 }
 
-std::set<std::string> FileDlg::Result() const
-{ return m_result; }
-
-bool FileDlg::SelectDirectories() const
-{ return m_select_directories; }
-
-bool FileDlg::AppendMissingSaveExtension() const
-{ return m_append_missing_save_extension; }
-
 void FileDlg::Render()
 {
     FlatRectangle(UpperLeft(), LowerRight(), m_color, m_border_color, 1);
@@ -240,7 +246,7 @@ void FileDlg::Render()
     }
 }
 
-void FileDlg::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod_keys)
+void FileDlg::KeyPress(Key key, uint32_t key_code_point, Flags<ModKey> mod_keys)
 {
     if (key == Key::GGK_RETURN || key == Key::GGK_KP_ENTER)
         OkHandler(false);
@@ -268,9 +274,6 @@ void FileDlg::SetFileFilters(const std::vector<std::pair<std::string, std::strin
     UpdateList();
 }
 
-const fs::path& FileDlg::WorkingDirectory()
-{ return s_working_dir; }
-
 const boost::filesystem::path FileDlg::StringToPath(const std::string& str) {
 #if defined(_WIN32)
     // convert UTF-8 path string to UTF-16
@@ -338,7 +341,7 @@ void FileDlg::OkHandler(bool double_click)
             OpenDirectory();
         } else if (files.size() == 1) {
             results_valid = true;
-            std::string save_file = *files.begin();
+            std::string save_file = files.front();
             if (m_append_missing_save_extension &&
                 m_file_filters.size() == 1 &&
                 std::count(m_file_filters[0].second.begin(), m_file_filters[0].second.end(), '*') == 1 &&
@@ -423,25 +426,45 @@ void FileDlg::OkHandler(bool double_click)
         }
     }
     if (results_valid)
-        m_done = true;
+        m_modal_done.store(true);
 }
 
 void FileDlg::CancelClicked()
 {
-    m_done = true;
+    m_modal_done.store(true);
     m_result.clear();
 }
 
-void FileDlg::FileSetChanged(const ListBox::SelectionSet& files)
+void FileDlg::FileSetChanged(const ListBox::SelectionSet& file_rows)
 {
+    using sel_t = std::decay_t<decltype(*file_rows.begin())>;
+    static_assert(std::is_same_v<sel_t, ListBox::iterator>);
+
+    static constexpr auto to_filename = [](ListBox::iterator file_row_it) -> std::string_view {
+        const auto& sh_ptr = *file_row_it;
+        if (!sh_ptr || sh_ptr->empty())
+            return {};
+        const auto* ctrl = sh_ptr->at(0);
+        const auto* tc = dynamic_cast<const TextControl*>(ctrl);
+        return tc ? tc->Text() : std::string_view{};
+    };
+
+
     std::string all_files;
+    all_files.reserve(file_rows.size() * 50); // guesstimate
     bool dir_selected = false;
-    for (const auto& file : files) {
-        std::string filename = !(**file).empty() ? boost::polymorphic_downcast<TextControl*>((**file).at(0))->Text() : "";
-        if (filename[0] != '[') {
+
+    for (const auto& file_row_it : file_rows) {
+        auto filename = to_filename(file_row_it);
+
+        if (filename.empty()) {
+            continue;
+
+        } else if (filename.front() != '[') { // not a directory
             if (!all_files.empty())
                 all_files += " ";
             all_files += filename;
+
         } else {
             if (m_select_directories) {
                 if (!all_files.empty())
@@ -458,7 +481,7 @@ void FileDlg::FileSetChanged(const ListBox::SelectionSet& files)
         m_ok_button->SetText(m_open_str);
 }
 
-void FileDlg::FileDoubleClicked(DropDownList::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys)
+void FileDlg::FileDoubleClicked(DropDownList::iterator it, GG::Pt pt, Flags<ModKey> modkeys)
 {
     m_files_list->DeselectAll();
     m_files_list->SelectRow(it);
@@ -479,7 +502,7 @@ void FileDlg::SetWorkingDirectory(const fs::path& p)
 {
     m_files_edit->Clear();
     FilesEditChanged(m_files_edit->Text());
-    s_working_dir = p;
+    s_working_dir = fs::canonical(p);
     UpdateDirectoryText();
     UpdateList();
 }
@@ -527,7 +550,7 @@ void FileDlg::UpdateList()
             if (non_wildcards->empty()) {
                 file_filters[i] = *anychar_p;
             } else {
-                file_filters[i] = 
+                file_filters[i] =
                     if_p (LeadingWildcard(filter_specs[i])) [
                         *(wildcard - f_str_p(FrontStringBegin(non_wildcards), FrontStringEnd(non_wildcards)))
                         >> f_str_p(FrontStringBegin(non_wildcards), FrontStringEnd(non_wildcards))
@@ -548,7 +571,7 @@ void FileDlg::UpdateList()
     if (!m_in_win32_drive_selection) {
         // parent directory selector
         if ((s_working_dir.string() != s_working_dir.root_path().string() &&
-             !s_working_dir.branch_path().string().empty()) ||
+             !s_working_dir.parent_path().string().empty()) ||
             Win32Paths())
         {
             auto row = Wnd::Create<ListBox::Row>();
@@ -581,15 +604,15 @@ void FileDlg::UpdateList()
 #if defined(_WIN32)
                     // convert UTF-16 path to UTF-8 for display
                     boost::filesystem::path::string_type file_name_native = it->path().filename().native();
-                    std::string temp;
-                    utf8::utf16to8(file_name_native.begin(), file_name_native.end(), std::back_inserter(temp));
-                    std::string row_text = "[" + temp + "]";
+                    std::string row_text{"["};
+                    row_text.reserve(file_name_native.size()*2 + 3);
+                    utf8::utf16to8(file_name_native.begin(), file_name_native.end(), std::back_inserter(row_text));
+                    row_text += ']';
 #else
                     std::string row_text = "[" + it->path().filename().string() + "]";
 #endif
-                    row->push_back(GetStyleFactory()->NewTextControl(row_text, m_font, m_text_color,
-                                                                     FORMAT_NOWRAP));
-                    sorted_rows.emplace(row_text, std::move(row));
+                    row->push_back(GetStyleFactory()->NewTextControl(row_text, m_font, m_text_color, FORMAT_NOWRAP));
+                    sorted_rows.emplace(std::move(row_text), std::move(row));
                 }
             } catch (const fs::filesystem_error&) {
             }
@@ -597,12 +620,12 @@ void FileDlg::UpdateList()
 
         std::vector<std::shared_ptr<ListBox::Row>> rows;
         rows.reserve(sorted_rows.size());
-        for (const auto& row : sorted_rows)
-            rows.push_back(row.second);
+        for (auto& row : sorted_rows)
+            rows.push_back(std::move(row.second));
+        sorted_rows.clear();
         m_files_list->Insert(std::move(rows));
 
         if (!m_select_directories) {
-            sorted_rows.clear();
             for (fs::directory_iterator it(s_working_dir); it != end_it; ++it) {
                 try {
                     if (fs::exists(*it) && !fs::is_directory(*it) && it->path().filename().native()[0] != '.') {
@@ -679,25 +702,34 @@ void FileDlg::OpenDirectory()
     if (sels.empty())
         return;
 
-    std::string directory;
-    directory = !(***sels.begin()).empty() ? boost::polymorphic_downcast<TextControl*>((***sels.begin()).at(0))->Text() : "";
+    const auto wrapped_directory_sv = [sel{(*sels.begin())->get()}]() -> std::string_view {
+        if (!sel || sel->empty())
+            return {};
+        try {
+            const auto* tc = dynamic_cast<TextControl*>(sel->at(0));
+            return tc ? std::string_view{tc->Text()} : std::string_view{};
+        } catch (...) {
+            return std::string_view{};
+        }
+    }();
 
-    if (directory.size() < 2 || directory[0] != '[')
+    if (wrapped_directory_sv.size() < 2 || wrapped_directory_sv.front() != '[' || wrapped_directory_sv.back() != ']')
         return;
 
-    directory = directory.substr(1, directory.size() - 2); // strip off '[' and ']'
+    const auto directory_sv = wrapped_directory_sv.substr(1, wrapped_directory_sv.size() - 2u);
 
-    if (directory == ".") {
+    if (directory_sv == ".") {
         // remain in current directory
         UpdateList();
+        return;
 
-    } else if (directory == "..") {
+    } else if (directory_sv == "..") {
         // move to parent directory of current directory
         if (s_working_dir.string() != s_working_dir.root_path().string() &&
-            !s_working_dir.branch_path().string().empty())
+            !s_working_dir.parent_path().string().empty())
         {
             // move to new directory
-            SetWorkingDirectory(s_working_dir.branch_path());
+            SetWorkingDirectory(s_working_dir.parent_path());
 
         } else {
             // switch to drive selection mode
@@ -708,41 +740,38 @@ void FileDlg::OpenDirectory()
             DoLayout();
             UpdateList();
         }
+        return;
 
     } else {
         // move to contained directory, which may be a drive selection...
         if (!m_in_win32_drive_selection) {
-
 #if defined(_WIN32)
             // convert UTF-8 file name to UTF-16
             boost::filesystem::path::string_type directory_native;
-            utf8::utf8to16(directory.begin(), directory.end(), std::back_inserter(directory_native));
+            utf8::utf8to16(directory_sv.begin(), directory_sv.end(), std::back_inserter(directory_native));
             SetWorkingDirectory(s_working_dir / fs::path(directory_native));
 #else
-            SetWorkingDirectory(s_working_dir / fs::path(directory));
+            SetWorkingDirectory(s_working_dir / fs::path(std::string{directory_sv}));
 #endif
+
         } else {
             m_in_win32_drive_selection = false;
             try {
-                SetWorkingDirectory(fs::path(directory + "\\"));
+                SetWorkingDirectory(fs::path(std::string{directory_sv} + "\\"));
             } catch (const fs::filesystem_error& e) {
-                if (e.code() == boost::system::errc::io_error) {
-                    m_in_win32_drive_selection = true;
-                    m_files_edit->Clear();
-                    FilesEditChanged(m_files_edit->Text());
-                    m_curr_dir_text->SetText("");
-                    DoLayout();
-                    UpdateList();
-                    auto dlg =
-                        GetStyleFactory()->NewThreeButtonDlg(X(175), Y(75),
-                                                             style->Translate("Device is not ready."),
-                                                             m_font, m_color,
-                                                             m_border_color, m_color,
-                                                             m_text_color, 1);
-                    dlg->Run();
-                } else {
+                if (e.code() != boost::system::errc::io_error)
                     throw;
-                }
+
+                m_in_win32_drive_selection = true;
+                m_files_edit->Clear();
+                FilesEditChanged(m_files_edit->Text());
+                m_curr_dir_text->SetText("");
+                DoLayout();
+                UpdateList();
+                auto dlg = GetStyleFactory()->NewThreeButtonDlg(
+                    X(175), Y(75), style->Translate("Device is not ready."),
+                    m_font, m_color, m_border_color, m_color, m_text_color, 1);
+                dlg->Run();
             }
         }
     }
