@@ -95,19 +95,19 @@ void BuildingsPanel::Update() {
     }
     int system_id = planet->SystemID();
 
-    const int indicator_size = static_cast<int>(Value(Width() * 1.0 / m_columns));
+    const int indicator_size = static_cast<int>(Width() / static_cast<float>(m_columns));
 
-    int this_client_empire_id = GGHumanClientApp::GetApp()->EmpireID();
-    const auto& this_client_known_destroyed_objects = GetUniverse().EmpireKnownDestroyedObjectIDs(this_client_empire_id);
-    const auto& this_client_stale_object_info = GetUniverse().EmpireStaleKnowledgeObjectIDs(this_client_empire_id);
-    const ScriptingContext context{GetUniverse(), Empires(), GetGalaxySetupData(), GetSpeciesManager(), GetSupplyManager()};
+    const int this_client_empire_id = GGHumanClientApp::GetApp()->EmpireID();
+    const ScriptingContext context;
+    const auto& this_client_known_destroyed_objects = context.ContextUniverse().EmpireKnownDestroyedObjectIDs(this_client_empire_id);
+    const auto& this_client_stale_object_info = context.ContextUniverse().EmpireStaleKnowledgeObjectIDs(this_client_empire_id);
 
     // get existing / finished buildings and use them to create building indicators
     for (int object_id : planet->BuildingIDs()) {
         // skip known destroyed and stale info objects
-        if (this_client_known_destroyed_objects.count(object_id))
+        if (this_client_known_destroyed_objects.contains(object_id))
             continue;
-        if (this_client_stale_object_info.count(object_id))
+        if (this_client_stale_object_info.contains(object_id))
             continue;
 
         auto building = Objects().get<Building>(object_id);
@@ -170,14 +170,14 @@ void BuildingsPanel::ExpandCollapseButtonPressed()
 { ExpandCollapse(!s_expanded_map[m_planet_id]); }
 
 void BuildingsPanel::DoLayout() {
-    auto old_size = Size();
+    const auto old_size = Size();
     AccordionPanel::DoLayout();
 
     int row = 0;
     int column = 0;
     static constexpr int padding = 5; // space around and between adjacent indicators
     const GG::X effective_width = Width() - padding * (m_columns + 1);  // padding on either side and between
-    const int indicator_size = static_cast<int>(Value(effective_width * 1.0 / m_columns));
+    const int indicator_size = static_cast<int>(Value(effective_width) / m_columns);
     GG::Y height;
 
     // update size of panel and position and visibility of widgets
@@ -220,7 +220,7 @@ void BuildingsPanel::DoLayout() {
     }
 
     if (m_building_indicators.empty()) {
-        height = GG::Y(0);  // hide if empty
+        height = GG::Y0;  // hide if empty
         DetachChild(m_expand_button.get());
     } else {
         AttachChild(m_expand_button);
@@ -307,7 +307,9 @@ void BuildingIndicator::PreRender() {
 void BuildingIndicator::Refresh() {
     SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
 
-    auto building = Objects().get<Building>(m_building_id);
+    const ScriptingContext context;
+
+    auto building = context.ContextObjects().get<Building>(m_building_id);
     if (!building)
         return;
 
@@ -325,9 +327,9 @@ void BuildingIndicator::Refresh() {
 
         // Scanlines for not currently-visible objects?
         if (GetOptionsDB().Get<bool>("ui.map.scanlines.shown")) {
-            int empire_id = GGHumanClientApp::GetApp()->EmpireID();
+            const int empire_id = GGHumanClientApp::GetApp()->EmpireID();
             if (empire_id != ALL_EMPIRES &&
-                GetUniverse().GetObjectVisibilityByEmpire(m_building_id, empire_id) < Visibility::VIS_BASIC_VISIBILITY)
+                context.ContextUniverse().GetObjectVisibilityByEmpire(m_building_id, empire_id) < Visibility::VIS_BASIC_VISIBILITY)
             { AttachChild(m_scanlines); }
         }
 
@@ -353,19 +355,17 @@ void BuildingIndicator::Refresh() {
     DoLayout();
 }
 
-void BuildingIndicator::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-    GG::Pt old_size = Size();
-
+void BuildingIndicator::SizeMove(GG::Pt ul, GG::Pt lr) {
+    const auto old_size = Size();
     GG::Wnd::SizeMove(ul, lr);
-
     if (old_size != Size())
         DoLayout();
 }
 
-void BuildingIndicator::MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys)
+void BuildingIndicator::MouseWheel(GG::Pt pt, int move, GG::Flags<GG::ModKey> mod_keys)
 { ForwardEventToParent(); }
 
-void BuildingIndicator::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+void BuildingIndicator::RClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys) {
     // verify that this indicator represents an existing building, and not a
     // queued production item, and that the owner of the building is this
     // client's player's empire
@@ -435,13 +435,13 @@ void BuildingIndicator::DoLayout() {
     GG::Pt child_lr = Size() - GG::Pt(GG::X1, GG::Y1);   // extra pixel prevents graphic from overflowing border box
 
     if (m_graphic)
-        m_graphic->SizeMove(GG::Pt(GG::X0, GG::Y0), child_lr);
+        m_graphic->SizeMove(GG::Pt0, child_lr);
 
     if (m_scanlines)
-        m_scanlines->SizeMove(GG::Pt(GG::X0, GG::Y0), child_lr);
+        m_scanlines->SizeMove(GG::Pt0, child_lr);
 
     if (m_scrap_indicator)
-        m_scrap_indicator->SizeMove(GG::Pt(GG::X0, GG::Y0), child_lr);
+        m_scrap_indicator->SizeMove(GG::Pt0, child_lr);
 
     GG::Y bar_top = Height() * 4 / 5;
     if (m_progress_bar)
