@@ -7,14 +7,14 @@
 
 
 /// Display a log of combat events with expandable log sections with more details of a combat event
-class CombatLogWnd : public GG::Wnd {
+class CombatLogWnd final : public GG::Wnd {
 public:
     CombatLogWnd(GG::X w, GG::Y h);
 
     ~CombatLogWnd();
 
-    GG::Pt ClientUpperLeft() const override;
-    GG::Pt ClientLowerRight() const override;
+    GG::Pt ClientUpperLeft() const noexcept override;
+    GG::Pt ClientLowerRight() const noexcept override;
     GG::Pt MinUsableSize() const override;
 
     void PreRender() override;
