@@ -7,14 +7,14 @@
 
 
 /// Shows a report on a combat
-class CombatReportWnd : public CUIWnd {
+class CombatReportWnd final : public CUIWnd {
 public:
     CombatReportWnd(std::string_view config_name = "");
     void CompleteConstruction() override;
     virtual ~CombatReportWnd(); // Must have explicit destructor since Impl is incomplete here
 
     void CloseClicked() override;
-    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+    void SizeMove(GG::Pt ul, GG::Pt lr) override;
     void SetLog(int log_id);
 
 private:
