@@ -3,12 +3,12 @@
 
 #include "CUIWnd.h"
 
-class InGameMenu : public CUIWnd {
+class InGameMenu final : public CUIWnd {
 public:
     InGameMenu();
     void CompleteConstruction() override;
 
-    void KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
+    void KeyPress(GG::Key key, uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) override;
     void DoLayout();
 
 protected:
