@@ -144,7 +144,7 @@ namespace {
 
         // first add only use those ordered templates actually in the current set of sitrep templates
         for (std::string& templ : OrderedSitrepTemplateStrings()) {
-            if (template_set.count(templ) &&
+            if (template_set.contains(templ) &&
                 !std::count(retval.begin(), retval.end(), templ))
             { retval.push_back(std::move(templ)); }
         }
@@ -184,31 +184,20 @@ namespace {
         GetOptionsDB().Commit();
     }
 
-    class ColorEmpire : public LinkDecorator {
-    public:
-        std::string Decorate(const std::string& target, const std::string& content) const override {
-            GG::Clr color = ClientUI::DefaultLinkColor();
-            int id = CastStringToInt(target);
-            Empire* empire = GetEmpire(id);
-            if (empire)
-                color = empire->Color();
-            return GG::RgbaTag(color) + content + "</rgba>";
-        }
-    };
-
     //////////////////////////////////
     // SitRepLinkText
     //////////////////////////////////
-    class SitRepLinkText : public LinkText {
+    class SitRepLinkText final : public LinkText {
     public:
-        SitRepLinkText(GG::X x, GG::Y y, GG::X w, const std::string& str, const std::shared_ptr<GG::Font>& font,
+        SitRepLinkText(GG::X x, GG::Y y, GG::X w, std::string str, std::shared_ptr<GG::Font> font,
                        GG::Flags<GG::TextFormat> format = GG::FORMAT_NONE, GG::Clr color = GG::CLR_BLACK) :
-            LinkText(x, y, w, str, font, format, color) {}
+            LinkText(x, y, w, std::move(str), std::move(font), format, color)
+        {}
 
-        mutable boost::signals2::signal<void(const GG::Pt&, GG::Flags<GG::ModKey>)> RightClickedSignal;
+        mutable boost::signals2::signal<void(GG::Pt, GG::Flags<GG::ModKey>)> RightClickedSignal;
 
     protected:
-        void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) override {
+        void RClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys) override {
             if (GetLinkUnderPt(pt) != -1) {
                 LinkText::RClick(pt, mod_keys);
             } else {
@@ -232,12 +221,11 @@ namespace {
 
             SetChildClippingMode(ChildClippingMode::ClipToClient);
 
-            int icon_dim = GetIconSize();
+            const int icon_dim = GetIconSize();
             std::string icon_texture = (m_sitrep_entry.GetIcon().empty() ?
                 "/icons/sitrep/generic.png" : m_sitrep_entry.GetIcon());
-            std::shared_ptr<GG::Texture> icon = ClientUI::GetTexture(
-                ClientUI::ArtDir() / icon_texture, true);
-            m_icon = GG::Wnd::Create<GG::StaticGraphic>(icon, GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+            std::shared_ptr<GG::Texture> icon = ClientUI::GetTexture(ClientUI::ArtDir() / icon_texture, true);
+            m_icon = GG::Wnd::Create<GG::StaticGraphic>(std::move(icon), GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
             AttachChild(m_icon);
             m_icon->Resize(GG::Pt(GG::X(icon_dim), GG::Y(icon_dim)));
 
@@ -245,12 +233,14 @@ namespace {
             GG::X icon_left(spacer.x);
             GG::X text_left(icon_left + GG::X(icon_dim) + sitrep_spacing);
             GG::X text_width(ClientWidth() - text_left - spacer.x);
+            const ScriptingContext context;
+
             m_link_text = GG::Wnd::Create<SitRepLinkText>(
-                GG::X0, GG::Y0, text_width, m_sitrep_entry.GetText() + " ", ClientUI::GetFont(),
+                GG::X0, GG::Y0, text_width, m_sitrep_entry.GetText(context) + " ", ClientUI::GetFont(),
                 GG::FORMAT_LEFT | GG::FORMAT_VCENTER | GG::FORMAT_WORDBREAK, ClientUI::TextColor());
-            m_link_text->SetDecorator(VarText::EMPIRE_ID_TAG, new ColorEmpire());
-            m_link_text->SetDecorator(TextLinker::BROWSE_PATH_TAG, new PathTypeDecorator());
-            m_link_text->SetDecorator(VarText::FOCS_VALUE_TAG, new ValueRefDecorator());
+            m_link_text->SetDecorator(VarText::EMPIRE_ID_TAG, TextLinker::DecoratorType::ColorByEmpire);
+            m_link_text->SetDecorator(TextLinker::BROWSE_PATH_TAG, TextLinker::DecoratorType::PathType);
+            m_link_text->SetDecorator(VarText::FOCS_VALUE_TAG, TextLinker::DecoratorType::ValueRef);
             AttachChild(m_link_text);
 
             namespace ph = boost::placeholders;
@@ -271,21 +261,21 @@ namespace {
                               background_clr, ClientUI::WndOuterBorderColor(), 1u);
         }
 
-        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
+        void SizeMove(GG::Pt ul, GG::Pt lr) override {
             if (ul != ClientUpperLeft() || (lr.x - ul.x) != Width())
                 DoLayout(ul, lr.x - ul.x);
         }
 
         const SitRepEntry& GetSitRepEntry() const noexcept { return m_sitrep_entry; }
 
-        mutable boost::signals2::signal<void(const GG::Pt&, GG::Flags<GG::ModKey>)> RightClickedSignal;
+        mutable boost::signals2::signal<void(GG::Pt, GG::Flags<GG::ModKey>)> RightClickedSignal;
 
     protected:
-        void RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod) override
+        void RClick(GG::Pt pt, GG::Flags<GG::ModKey> mod) override
         { RightClickedSignal(pt, mod); }
 
     private:
-        void DoLayout(const GG::Pt& ul, const GG::X& width) {
+        void DoLayout(GG::Pt ul, GG::X width) {
             // Resize the text
             GG::Pt spacer = GG::Pt(GG::X(sitrep_edge_to_content_spacing), GG::Y(sitrep_edge_to_content_spacing));
             GG::X icon_left(ul.x + spacer.x);
@@ -352,7 +342,7 @@ namespace {
             GG::ListBox::Row::Resize(m_panel->Size() + border);
         }
 
-        void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override {
+        void SizeMove(GG::Pt ul, GG::Pt lr) override {
             if (!m_panel || (Size() != (lr - ul)))
                 RequirePreRender();
             GG::ListBox::Row::SizeMove(ul, lr);
@@ -422,30 +412,28 @@ void SitRepPanel::CompleteConstruction() {
 }
 
 void SitRepPanel::DoLayout() {
-    GG::X BUTTON_WIDTH(ClientUI::Pts()*4);
+    GG::X BUTTON_WIDTH{ClientUI::Pts()*4};
     GG::Y BUTTON_HEIGHT = m_last_turn_button->MinUsableSize().y;
-    int PAD(3);
+    static constexpr GG::X SITREP_PANEL_PAD{3};
 
     GG::Pt button_ul(ClientWidth() - GG::X(INNER_BORDER_ANGLE_OFFSET) - BUTTON_WIDTH,
                      ClientHeight() - BUTTON_HEIGHT);
 
     m_last_turn_button->SizeMove(button_ul, button_ul + GG::Pt(BUTTON_WIDTH, BUTTON_HEIGHT));
-    button_ul -= GG::Pt(BUTTON_WIDTH + GG::X(PAD), GG::Y0);
+    button_ul -= GG::Pt(BUTTON_WIDTH + SITREP_PANEL_PAD, GG::Y0);
     m_next_turn_button->SizeMove(button_ul, button_ul + GG::Pt(BUTTON_WIDTH, BUTTON_HEIGHT));
-    button_ul -= GG::Pt(BUTTON_WIDTH + GG::X(PAD), GG::Y0);
+    button_ul -= GG::Pt(BUTTON_WIDTH + SITREP_PANEL_PAD, GG::Y0);
     m_prev_turn_button->SizeMove(button_ul, button_ul + GG::Pt(BUTTON_WIDTH, BUTTON_HEIGHT));
-    button_ul -= GG::Pt(BUTTON_WIDTH + GG::X(PAD), GG::Y0);
+    button_ul -= GG::Pt(BUTTON_WIDTH + SITREP_PANEL_PAD, GG::Y0);
 
-    m_sitreps_lb->SizeMove(GG::Pt(GG::X0, GG::Y0), GG::Pt(ClientWidth() - 1, button_ul.y));
+    m_sitreps_lb->SizeMove(GG::Pt0, GG::Pt(ClientWidth() - 1, button_ul.y));
 
     m_filter_button->SizeMove(GG::Pt(GG::X0, button_ul.y), GG::Pt(BUTTON_WIDTH*2, button_ul.y + BUTTON_HEIGHT));
 
     SetMinSize(GG::Pt(6*BUTTON_WIDTH, 6*BUTTON_HEIGHT));
 }
 
-void SitRepPanel::KeyPress(GG::Key key, std::uint32_t key_code_point,
-                           GG::Flags<GG::ModKey> mod_keys)
-{
+void SitRepPanel::KeyPress(GG::Key key, uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
     switch (key) {
     case GG::Key::GGK_RETURN:
     case GG::Key::GGK_KP_ENTER:
@@ -458,10 +446,9 @@ void SitRepPanel::KeyPress(GG::Key key, std::uint32_t key_code_point,
     }
 }
 
-void SitRepPanel::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
-    GG::Pt old_size = GG::Wnd::Size();
-    std::size_t first_visible_queue_row = std::distance(m_sitreps_lb->begin(),
-                                                        m_sitreps_lb->FirstRowShown());
+void SitRepPanel::SizeMove(GG::Pt ul, GG::Pt lr) {
+    const GG::Pt old_size = GG::Wnd::Size();
+    const std::size_t first_visible_queue_row = std::distance(m_sitreps_lb->begin(), m_sitreps_lb->FirstRowShown());
 
     CUIWnd::SizeMove(ul, lr);
 
@@ -504,22 +491,24 @@ namespace {
     /** Return true iff the \a sitrep is not hidden, validates and is not snoozed. */
     bool IsSitRepInvalid(const SitRepEntry& sitrep, const std::set<std::string>& hidden_templates) {
         auto& label = sitrep.GetLabelString().empty() ? sitrep.GetTemplateString() : sitrep.GetLabelString();
-        if (hidden_templates.count(label))
+        if (hidden_templates.contains(label))
             return true;
 
+        const ScriptingContext context;
+
         // Validation is time consuming because all variables are substituted.
         // Having ui.map.sitrep.invalid.shown off / disabled will hide sitreps that do not
         // validate. Having it on will skip the validation check.
-        if (!GetOptionsDB().Get<bool>("ui.map.sitrep.invalid.shown") && !sitrep.Validate())
+        if (!GetOptionsDB().Get<bool>("ui.map.sitrep.invalid.shown") && !sitrep.Validate(context))
             return true;
 
         // Check for snoozing.
-        if (permanently_snoozed_sitreps.count(sitrep.GetText()))
+        if (permanently_snoozed_sitreps.contains(sitrep.GetText(context)))
             return true;
 
         auto sitrep_set_it = snoozed_sitreps.find(sitrep.GetTurn());
         if (sitrep_set_it != snoozed_sitreps.end()
-            && sitrep_set_it->second.count(sitrep.GetText()))
+            && sitrep_set_it->second.contains(sitrep.GetText(context)))
         { return true; }
 
         return false;
@@ -577,7 +566,8 @@ void SitRepPanel::NextClicked() {
 void SitRepPanel::LastClicked() {
     auto turns = GetUnvalidatedSitRepsSortedByTurn(GGHumanClientApp::GetApp()->EmpireID());
     // search backwards from current turn for a non-empty sitrep turn
-    ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, CurrentTurn() + 1, false, m_hidden_sitrep_templates));
+    ShowSitRepsForTurn(GetNextNonEmptySitrepsTurn(turns, GGHumanClientApp::GetApp()->CurrentTurn() + 1,
+                                                  false, m_hidden_sitrep_templates));
 }
 
 void SitRepPanel::FilterClicked() {
@@ -597,7 +587,7 @@ void SitRepPanel::FilterClicked() {
     for (const std::string& templ : all_templates) {
         menu_index_templates[index] = templ;
         bool check = true;
-        if (m_hidden_sitrep_templates.count(templ)) {
+        if (m_hidden_sitrep_templates.contains(templ)) {
             check = false;
             all_checked = false;
         }
@@ -644,7 +634,7 @@ void SitRepPanel::FilterClicked() {
     Update();
 }
 
-void SitRepPanel::IgnoreSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
+void SitRepPanel::IgnoreSitRep(GG::ListBox::iterator it, GG::Pt pt, GG::Flags<GG::ModKey> mod) {
     SitRepRow* sitrep_row = dynamic_cast<SitRepRow*>(it->get());
     if (!sitrep_row)
         return;
@@ -652,12 +642,14 @@ void SitRepPanel::IgnoreSitRep(GG::ListBox::iterator it, const GG::Pt& pt, const
     const SitRepEntry& sitrep = sitrep_row->GetSitRepEntry();
     if (sitrep.GetTurn() <= 0)
         return;
-    snoozed_sitreps[sitrep.GetTurn()].insert(sitrep.GetText());
+
+    const ScriptingContext context;
+    snoozed_sitreps[sitrep.GetTurn()].insert(sitrep.GetText(context));
 
     Update();
 }
 
-void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& mod) {
+void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, GG::Pt pt, GG::Flags<GG::ModKey> mod) {
     GG::MenuItem menu_contents, submenu_ignore, submenu_block, separator_item;
     std::string sitrep_text, sitrep_template;
     std::string entry_margin("  ");
@@ -667,9 +659,11 @@ void SitRepPanel::DismissalMenu(GG::ListBox::iterator it, const GG::Pt& pt, cons
     if (it != m_sitreps_lb->end()) 
         sitrep_row = dynamic_cast<SitRepRow*>(it->get());
     submenu_ignore.label = entry_margin + UserString("SITREP_IGNORE_MENU");
+
     if (sitrep_row) {
+        const ScriptingContext context;
         const SitRepEntry& sitrep_entry = sitrep_row->GetSitRepEntry();
-        sitrep_text = sitrep_entry.GetText();
+        sitrep_text = sitrep_entry.GetText(context);
         start_turn = sitrep_entry.GetTurn();
         if (start_turn > 0) {
             auto snooze5_action = [&sitrep_text, start_turn]() { SnoozeSitRepForNTurns(sitrep_text, start_turn, 5); };
@@ -863,7 +857,7 @@ void SitRepPanel::SetHiddenSitRepTemplates(const std::set<std::string>& template
 
 int SitRepPanel::NumVisibleSitrepsThisTurn() const {
     auto turns = GetUnvalidatedSitRepsSortedByTurn(GGHumanClientApp::GetApp()->EmpireID());
-    auto& this_turn_sitreps = turns[CurrentTurn()];
+    auto& this_turn_sitreps = turns[GGHumanClientApp::GetApp()->CurrentTurn()];
     auto is_valid = [this](const auto* s) { return s && !IsSitRepInvalid(*s, m_hidden_sitrep_templates); };
 
     return std::count_if(this_turn_sitreps.begin(), this_turn_sitreps.end(), is_valid);
