@@ -16,6 +16,7 @@
 #include "../util/i18n.h"
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
+#include "../util/ranges.h"
 #include "../Empire/Empire.h"
 
 #include <GG/StaticGraphic.h>
@@ -35,7 +36,10 @@ namespace {
     }
 
     /// Adds color tags to name_o according to the empires in owner_empire_ids
-    std::string ColorNameByOwners(const std::string& name_o, std::set<int>& owner_empire_ids, const EmpireManager& empires) {
+    std::string ColorNameByOwners(const std::string& name_o,
+                                  std::set<int>& owner_empire_ids,
+                                  const EmpireManager& empires)
+    {
         if (owner_empire_ids.size() < 1) {
             return name_o;
 
@@ -107,7 +111,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size,
         return;
 
     const auto& known_destroyed_object_ids = universe.EmpireKnownDestroyedObjectIDs(client_empire_id);
-    if (known_destroyed_object_ids.count(system_id))
+    if (known_destroyed_object_ids.contains(system_id))
         return;
 
 
@@ -126,7 +130,7 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size,
     for (auto& planet : system_planets) {
         int planet_id = planet->ID();
 
-        if (known_destroyed_object_ids.count(planet_id))
+        if (known_destroyed_object_ids.contains(planet_id))
             continue;
 
         // is planet a capital?
@@ -141,28 +145,22 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size,
 
         // is planet a homeworld? (for any species)
         if (!homeworld) {
-            for (const auto& entry : species_manager.GetSpeciesHomeworldsMap()) {
-                const auto& homeworld_ids = entry.second;
-                if (homeworld_ids.count(planet_id)) {
-                    homeworld = true;
-                    break;
-                }
-            }
+            const auto is_homeworld = [planet_id](const auto& hw_ids) { return hw_ids.contains(planet_id); };
+            homeworld = range_any_of(species_manager.GetSpeciesHomeworldsMap() | range_values,
+                                     is_homeworld);
         }
 
         // does planet contain a shipyard?
         if (!has_shipyard) {
-            for (auto& building : objects.find<const Building>(planet->BuildingIDs())) {
-                int building_id = building->ID();
+            const auto not_destroyed = [&known_destroyed_object_ids](const auto id)
+            { return !known_destroyed_object_ids.contains(id); };
 
-                if (known_destroyed_object_ids.count(building_id))
-                    continue;
+            const auto get_building = [&objects](const auto id) { return objects.getRaw<const Building>(id); };
 
-                if (building->HasTag(TAG_SHIPYARD, context)) {
-                    has_shipyard = true;
-                    break;
-                }
-            }
+            static constexpr auto is_shipyard = [](const Building* b) { return b && b->HasTag(TAG_SHIPYARD); };
+
+            has_shipyard = range_any_of(planet->BuildingIDs() | range_filter(not_destroyed) | range_transform(get_building),
+                                        is_shipyard);
         }
 
         // is planet populated by neutral species
@@ -191,16 +189,16 @@ OwnerColoredSystemName::OwnerColoredSystemName(int system_id, int font_size,
         wrapped_system_name = "<i>" + wrapped_system_name + "</i>";
     if (has_shipyard)
         wrapped_system_name = "<u>" + wrapped_system_name + "</u>";
-    std::shared_ptr<GG::Font> font;
-    if (capital)
-        font = ClientUI::GetBoldFont(font_size);
-    else
-        font = ClientUI::GetFont(font_size);
+    const auto font = capital ? ClientUI::GetBoldFont(font_size) : ClientUI::GetFont(font_size);
 
     GG::Clr text_color = ClientUI::SystemNameTextColor();
     if (has_player_planet) {
-        if (owner_empire_ids.size() == 1)
-            text_color = GetEmpire(*owner_empire_ids.begin())->Color();
+        if (owner_empire_ids.size() == 1) {
+            if (const auto* owner_empire = GetEmpire(*owner_empire_ids.begin()))
+                text_color = owner_empire->Color();
+            else
+                DebugLogger() << "OwnerColoredSystemName couldn't get empire with id: " << *owner_empire_ids.begin();
+        }
     } else if (has_neutrals) {
         text_color = ClientUI::TextColor();
     }
@@ -223,14 +221,14 @@ void OwnerColoredSystemName::CompleteConstruction() {
     }
     AttachChild(m_text);
     GG::Pt text_size(m_text->TextLowerRight() - m_text->TextUpperLeft());
-    m_text->SizeMove(GG::Pt(GG::X0, GG::Y0), text_size);
+    m_text->SizeMove(GG::Pt0, text_size);
     Resize(text_size);
 }
 
 void OwnerColoredSystemName::Render()
 {}
 
-void OwnerColoredSystemName::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+void OwnerColoredSystemName::SizeMove(GG::Pt ul, GG::Pt lr) {
     GG::Control::SizeMove(ul, lr);
 
     // Center text
@@ -433,7 +431,7 @@ GG::Pt SystemIcon::NthFleetButtonUpperLeft(unsigned int button_number, bool movi
 int SystemIcon::EnclosingCircleDiameter() const
 { return static_cast<const int>(Value(Width()) * GetOptionsDB().Get<double>("ui.map.system.circle.size")) + 1; }
 
-void SystemIcon::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+void SystemIcon::SizeMove(GG::Pt ul, GG::Pt lr) {
     Wnd::SizeMove(ul, lr);
 
     const bool USE_TINY_GRAPHICS = Value(Width()) < GetOptionsDB().Get<int>("ui.map.system.icon.tiny.threshold");
@@ -485,7 +483,7 @@ void SystemIcon::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     }
 
     const bool USE_TINY_MOUSEOVER_INDICATOR = m_tiny_mouseover_indicator &&
-                                              (Value(Width()) < m_tiny_mouseover_indicator->Width());
+                                              (Width() < m_tiny_mouseover_indicator->Width());
 
     // normal mouseover indicator - attach / detach / show / hide done by MouseEnter and MouseLeave
     if (m_mouseover_indicator && !USE_TINY_MOUSEOVER_INDICATOR) {
@@ -556,21 +554,21 @@ void SystemIcon::RenderOverlay(double zoom_factor) {
     m_overlay_texture->OrthoBlit(overlay_ul, overlay_lr);
 }
 
-void SystemIcon::LClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+void SystemIcon::LClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) LeftClickedSignal(m_system_id); }
 
-void SystemIcon::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+void SystemIcon::RClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) RightClickedSignal(m_system_id, mod_keys); }
 
-void SystemIcon::LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+void SystemIcon::LDoubleClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) LeftDoubleClickedSignal(m_system_id); }
 
-void SystemIcon::RDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys)
+void SystemIcon::RDoubleClick(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys)
 { if (!Disabled()) RightDoubleClickedSignal(m_system_id); }
 
-void SystemIcon::MouseEnter(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
+void SystemIcon::MouseEnter(GG::Pt pt, GG::Flags<GG::ModKey> mod_keys) {
     const bool USE_TINY_MOUSEOVER_INDICATOR = m_tiny_mouseover_indicator &&
-                                              (Value(Width()) < m_tiny_mouseover_indicator->Width());
+                                              (Width() < m_tiny_mouseover_indicator->Width());
     // indicate mouseover
     if (m_mouseover_indicator && !USE_TINY_MOUSEOVER_INDICATOR) {
         int client_empire_id = GGHumanClientApp::GetApp()->EmpireID();
@@ -631,7 +629,7 @@ void SystemIcon::MouseLeave() {
     MouseLeavingSignal(m_system_id);
 }
 
-void SystemIcon::MouseWheel(const GG::Pt& pt, int move, GG::Flags<GG::ModKey> mod_keys)
+void SystemIcon::MouseWheel(GG::Pt pt, int move, GG::Flags<GG::ModKey> mod_keys)
 { ForwardEventToParent(); }
 
 void SystemIcon::SetSelected(bool selected) {
@@ -720,7 +718,7 @@ void SystemIcon::PositionSystemName(int pts) {
     name.second->MoveTo(GG::Pt(name_left, name_top));
 }
 
-bool SystemIcon::InWindow(const GG::Pt& pt) const {
+bool SystemIcon::InWindow(GG::Pt pt) const {
     // find if cursor is within required distance of centre of icon
     const int RADIUS = EnclosingCircleDiameter() / 2;
     const int RADIUS2 = RADIUS*RADIUS;
