@@ -29,8 +29,19 @@
 
 #include <thread>
 #include <queue>
+#include <utility>
+#if !defined(__cpp_lib_integer_comparison_functions)
+namespace std {
+    inline auto cmp_less_equal(auto&& lhs, auto&& rhs) { return lhs <= rhs; }
+    inline auto cmp_not_equal(auto&& lhs, auto&& rhs) { return lhs != rhs; }
+}
+#endif
+
+
+static_assert(BOOST_VERSION >= 106600); // when udp::resolver::resolve appears
 
 using boost::asio::ip::tcp;
+using boost::asio::ip::udp;
 using namespace Networking;
 
 namespace {
@@ -45,50 +56,45 @@ namespace {
         ServerDiscoverer(boost::asio::io_context& io_context) :
             m_io_context(&io_context),
             m_timer(io_context),
-            m_socket(io_context),
-            m_recv_buf(),
-            m_receive_successful(false),
-            m_server_name()
+            m_socket(io_context)
         {}
 
-        const ServerList& Servers() const
-        { return m_servers; }
+        const ServerList& Servers() const noexcept { return m_servers; }
 
         void DiscoverServers() {
             using namespace boost::asio::ip;
+            using namespace boost::asio;
             udp::resolver resolver(*m_io_context);
-            udp::resolver::query query(udp::v4(), "255.255.255.255",
-                                       std::to_string(Networking::DiscoveryPort()),
-                                       resolver_query_base::address_configured |
-                                       resolver_query_base::numeric_service);
-            udp::resolver::iterator end_it;
-            for (auto it = resolver.resolve(query); it != end_it; ++it) {
-                udp::endpoint receiver_endpoint = *it;
+            const auto results = resolver.resolve(udp::v4(), "255.255.255.255",
+                                                  std::to_string(Networking::DiscoveryPort()),
+                                                  resolver_query_base::address_configured |
+                                                  resolver_query_base::numeric_service);
 
+            for (const auto& entry : results) {
+                udp::endpoint receiver_endpoint = entry.endpoint();
                 m_socket.close();
                 m_socket.open(udp::v4());
-                m_socket.set_option(boost::asio::socket_base::broadcast(true));
+                m_socket.set_option(socket_base::broadcast(true));
 
-                m_socket.send_to(boost::asio::buffer(DISCOVERY_QUESTION),
+                m_socket.send_to(buffer(DISCOVERY_QUESTION),
                                  receiver_endpoint);
 
                 m_socket.async_receive_from(
-                    boost::asio::buffer(m_recv_buf),
+                    buffer(m_recv_buf),
                     m_sender_endpoint,
                     boost::bind(&ServerDiscoverer::HandleReceive,
                                 this,
-                                boost::asio::placeholders::error,
-                                boost::asio::placeholders::bytes_transferred));
+                                placeholders::error,
+                                placeholders::bytes_transferred));
 
                 m_timer.expires_after(std::chrono::seconds(2));
                 m_timer.async_wait(boost::bind(&ServerDiscoverer::CloseSocket, this));
                 m_io_context->run();
-                m_io_context->reset();
+                m_io_context->restart();
                 if (m_receive_successful) {
-                    boost::asio::ip::address address = m_server_name == "localhost" ?
-                        boost::asio::ip::address::from_string("127.0.0.1") :
-                        m_sender_endpoint.address();
-                    m_servers.push_back({address, m_server_name});
+                    auto address = (m_server_name == "localhost") ?
+                        make_address("127.0.0.1") : m_sender_endpoint.address();
+                    m_servers.emplace_back(address, m_server_name);
                 }
                 m_receive_successful = false;
                 m_server_name.clear();
@@ -120,24 +126,21 @@ namespace {
             }
         }
 
-        void CloseSocket()
-        { m_socket.close(); }
+        void CloseSocket() { m_socket.close(); }
 
-        boost::asio::io_context*            m_io_context;
+        boost::asio::io_context* const      m_io_context;
         boost::asio::high_resolution_timer  m_timer;
-        boost::asio::ip::udp::socket        m_socket;
+        udp::socket             m_socket;
 
-        std::array<char, 1024>              m_recv_buf = {};
+        std::array<char, 1024>  m_recv_buf = {};
 
-        boost::asio::ip::udp::endpoint      m_sender_endpoint;
-        bool                                m_receive_successful = false;
-        std::string                         m_server_name;
-        ServerList                          m_servers;
+        udp::endpoint           m_sender_endpoint;
+        std::string             m_server_name;
+        ServerList              m_servers;
+        bool                    m_receive_successful = false;
     };
 }
 
-
-
 class ClientNetworking::Impl {
 public:
     /** The type of list returned by a call to DiscoverLANServers(). */
@@ -161,7 +164,7 @@ public:
     int HostPlayerID() const noexcept { return m_host_player_id; }
 
     /** Returns whether the indicated player ID is the host. */
-    bool PlayerIsHost(int player_id) const;
+    bool PlayerIsHost(int player_id) const noexcept;
 
     /** Checks if the client has some authorization \a role. */
     bool HasAuthRole(Networking::RoleType role) const;
@@ -177,17 +180,16 @@ public:
         attempts will be made until \a timeout seconds has elapsed. If \p
         expect_timeout is true, timeout is not reported as an error. */
     bool ConnectToServer(const ClientNetworking* const self,
-                         const std::string& ip_address,
-                         const std::chrono::milliseconds& timeout = std::chrono::seconds(10),
+                         std::string ip_address,
+                         std::chrono::milliseconds timeout = std::chrono::seconds(10),
                          bool expect_timeout = false);
 
     /** Connects to the server on the client's host.  On failure, repeated
         attempts will be made until \a timeout seconds has elapsed. If \p
         expect_timeout is true, timeout is not reported as an error.*/
-    bool ConnectToLocalHostServer(
-        const ClientNetworking* const self,
-        const std::chrono::milliseconds& timeout = std::chrono::seconds(10),
-        bool expect_timeout = false);
+    bool ConnectToLocalHostServer(const ClientNetworking* const self,
+                                  std::chrono::milliseconds timeout = std::chrono::seconds(10),
+                                  bool expect_timeout = false);
 
     /** Sends \a message to the server. This function actually just enqueues
         the message for sending and returns immediately. */
@@ -200,23 +202,32 @@ public:
         Remove the message from the incoming message queue. */
     boost::optional<Message> GetMessage();
 
-    /** Disconnects the client from the server. */
     void DisconnectFromServer();
 
-    /** Sets player ID for this client. */
     void SetPlayerID(int player_id);
 
-    /** Sets Host player ID. */
-    void SetHostPlayerID(int host_player_id);
+    void SetHostPlayerID(int host_player_id) noexcept { m_host_player_id = host_player_id; }
 
-    /** Get authorization roles access. */
-    Networking::AuthRoles& AuthorizationRoles();
+    Networking::AuthRoles& AuthorizationRoles() noexcept { return m_roles; }
 
 private:
     void HandleException(const boost::system::system_error& error);
-    void HandleConnection(const boost::system::error_code& error,
-                          tcp::resolver::iterator endpoint_it);
-    void HandleResolve(const boost::system::error_code& error, tcp::resolver::iterator results);
+
+    using results_type = tcp::resolver::results_type;
+    using results_iterator = results_type::iterator;
+
+    void HandleConnection(const boost::system::error_code& error, const auto& results)
+    { HandleConnection(error, results.begin(), results.end()); }
+
+    void HandleConnection(const boost::system::error_code& error, results_iterator endpoint_it,
+                          const results_iterator end_of_endpoints_it);
+
+    void HandleResolve(const boost::system::error_code& error, const auto& results)
+    { HandleResolve(error, results.begin(), results.end()); }
+
+    void HandleResolve(const boost::system::error_code& error, const results_iterator results_it,
+                       const results_iterator end_of_endpoints_it);
+
     void HandleDeadlineTimeout(const boost::system::error_code& error);
 
     void NetworkingThread(const std::shared_ptr<const ClientNetworking> self);
@@ -229,20 +240,19 @@ private:
     void AsyncWriteMessage();
     void SendMessageImpl(Message message);
     void DisconnectFromServerImpl();
-    bool _IsConnected() const;  // Non-thread-safe: Return true iff the client is full duplex connected to the server.
     bool CloseSocketIfNotConnected();  // Close the socket iff the client is not fully duplex connected to the server.
     void LaunchNetworkThread(const ClientNetworking* const self);
 
-    int                             m_player_id = Networking::INVALID_PLAYER_ID;
-    int                             m_host_player_id = Networking::INVALID_PLAYER_ID;
-    Networking::AuthRoles           m_roles;
+    int                   m_player_id = Networking::INVALID_PLAYER_ID;
+    int                   m_host_player_id = Networking::INVALID_PLAYER_ID;
+    Networking::AuthRoles m_roles;
 
     boost::asio::io_context            m_io_context;
-    boost::asio::ip::tcp::socket       m_socket;
+    tcp::socket                        m_socket;
     boost::asio::high_resolution_timer m_deadline_timer;
     boost::asio::high_resolution_timer m_reconnect_timer;
-    tcp::resolver::iterator            m_resolver_results;
-    bool                               m_deadline_has_expired{ false };
+    results_iterator                   m_resolver_results_it;
+    bool                               m_deadline_has_expired = false;
 
     // m_mutex guards m_incoming_message, m_rx_connected and m_tx_connected which are written by
     // the networking thread and read by the main thread to check incoming messages and connection
@@ -276,16 +286,12 @@ ClientNetworking::Impl::Impl() :
 
 bool ClientNetworking::Impl::IsConnected() const {
     std::scoped_lock lock(m_mutex);
-    return _IsConnected();
-}
-
-bool ClientNetworking::Impl::_IsConnected() const {
     return m_rx_connected && m_tx_connected;
 }
 
 bool ClientNetworking::Impl::CloseSocketIfNotConnected() {
     std::scoped_lock lock(m_mutex);
-    bool do_close = !_IsConnected();
+    const bool do_close = !(m_rx_connected && m_tx_connected);
     if (do_close)
         m_socket.close();
     return do_close;
@@ -301,7 +307,7 @@ bool ClientNetworking::Impl::IsTxConnected() const {
     return m_tx_connected;
 }
 
-bool ClientNetworking::Impl::PlayerIsHost(int player_id) const {
+bool ClientNetworking::Impl::PlayerIsHost(int player_id) const noexcept {
     if (player_id == Networking::INVALID_PLAYER_ID)
         return false;
     return player_id == m_host_player_id;
@@ -311,14 +317,11 @@ bool ClientNetworking::Impl::HasAuthRole(Networking::RoleType role) const
 { return m_roles.HasRole(role); }
 
 ClientNetworking::ServerNames ClientNetworking::Impl::DiscoverLANServerNames() {
-    if (!IsConnected())
-        return ServerNames();
     ServerDiscoverer discoverer(m_io_context);
     discoverer.DiscoverServers();
     ServerNames names;
-    for (const auto& server : discoverer.Servers()) {
-        names.push_back(server.second);
-    }
+    for (const auto& [server_address, server_name] : discoverer.Servers())
+        names.push_back(server_name.empty() ? server_address.to_string() : server_name);
     return names;
 }
 
@@ -343,48 +346,47 @@ void ClientNetworking::Impl::LaunchNetworkThread(const ClientNetworking* const s
     m_socket.set_option(boost::asio::socket_base::keep_alive(true));
 
     DebugLogger(network) << "ConnectToServer() : starting networking thread";
-    boost::thread(boost::bind(&ClientNetworking::Impl::NetworkingThread, this, self->shared_from_this()));
+    boost::thread net_thread(boost::bind(&ClientNetworking::Impl::NetworkingThread, this, self->shared_from_this()));
 }
 
 
-bool ClientNetworking::Impl::ConnectToServer(
-    const ClientNetworking* const self,
-    const std::string& ip_address,
-    const std::chrono::milliseconds& timeout,
-    bool expect_timeout)
+bool ClientNetworking::Impl::ConnectToServer(const ClientNetworking* const self,
+                                             std::string ip_address,
+                                             std::chrono::milliseconds timeout,
+                                             bool expect_timeout)
 {
-    TraceLogger(network) << "ClientNetworking::Impl::ConnectToServer(" << self << ", " << ip_address << ", " << timeout.count() << ", " << expect_timeout << ")";
+    TraceLogger(network) << "ClientNetworking::Impl::ConnectToServer(" << self
+                         << ", " << ip_address << ", " << timeout.count() << ", " << expect_timeout << ")";
     using Clock = std::chrono::high_resolution_clock;
-    Clock::time_point start_time = Clock::now();
+    const Clock::time_point start_time = Clock::now();
 
     using namespace boost::asio::ip;
     tcp::resolver resolver(m_io_context);
-    tcp::resolver::query query(ip_address,
-                               std::to_string(Networking::MessagePort()),
-                               resolver_query_base::numeric_service);
-    
-    // Resolve the query - will try to connect on success.
-    resolver.async_resolve(query, [this](const auto& err, const auto& results) {
-        HandleResolve(err, results); 
-    });
+
+    const auto resolve_handler = [this](const auto& err, tcp::resolver::results_type results)
+    { HandleResolve(err, results); };
+
+    resolver.async_resolve(ip_address, std::to_string(Networking::MessagePort()),
+                           resolver_query_base::numeric_service, resolve_handler);
+
 
     TraceLogger(network) << "ClientNetworking::Impl::ConnectToServer() - Resolving...";
     m_io_context.run_one();
     TraceLogger(network) << "ClientNetworking::Impl::ConnectToServer() - Resolved.";
     // configure the deadline timer to close socket and cancel connection attempts at timeout
     m_deadline_has_expired = false;
-    m_deadline_timer.expires_from_now(timeout);
+    m_deadline_timer.expires_after(timeout);
     m_deadline_timer.async_wait([this](const auto& err) { HandleDeadlineTimeout(err); });
-    
+
     try {
         TraceLogger(network) << "ClientNetworking::Impl::ConnectToServer() - Starting asio event loop";
         m_io_context.run(); // blocks until connection or timeout
-        m_io_context.reset();
+        m_io_context.restart();
 
         if (IsConnected()) {
-            auto connection_time = Clock::now() - start_time;
-            DebugLogger(network) << "Connecting to server took "
-                << std::chrono::duration_cast<std::chrono::milliseconds>(connection_time).count() << " ms.";
+            const auto connection_time = Clock::now() - start_time;
+            const auto time_ms = std::chrono::duration_cast<std::chrono::milliseconds>(connection_time).count();
+            DebugLogger(network) << "Connecting to server took " << time_ms << " ms.";
             LaunchNetworkThread(self);
         }
         else {
@@ -403,17 +405,18 @@ bool ClientNetworking::Impl::ConnectToServer(
     return IsConnected();
 }
 
-bool ClientNetworking::Impl::ConnectToLocalHostServer(
-    const ClientNetworking* const self,
-    const std::chrono::milliseconds& timeout,
-    bool expect_timeout)
+bool ClientNetworking::Impl::ConnectToLocalHostServer(const ClientNetworking* const self,
+                                                      std::chrono::milliseconds timeout,
+                                                      bool expect_timeout)
 {
-    TraceLogger(network) << "ClientNetworking::Impl::ConnectToLocalHostServer(" << self << ", " << timeout.count() << ", " << expect_timeout << ")";
+    TraceLogger(network) << "ClientNetworking::Impl::ConnectToLocalHostServer(" << self
+                         << ", " << timeout.count() << ", " << expect_timeout << ")";
     bool retval = false;
 #if FREEORION_WIN32
     try {
 #endif
-        retval = ConnectToServer(self, "127.0.0.1", timeout, expect_timeout);
+        static const std::string localhost_ip{"127.0.0.1"};
+        retval = ConnectToServer(self, localhost_ip, timeout, expect_timeout);
 #if FREEORION_WIN32
     } catch (const boost::system::system_error& e) {
         if (e.code().value() != WSAEADDRNOTAVAIL)
@@ -433,7 +436,7 @@ void ClientNetworking::Impl::DisconnectFromServer() {
     }
 
     if (is_open)
-        m_io_context.post(boost::bind(&ClientNetworking::Impl::DisconnectFromServerImpl, this));
+        boost::asio::post(m_io_context, boost::bind(&ClientNetworking::Impl::DisconnectFromServerImpl, this));
 }
 
 void ClientNetworking::Impl::SetPlayerID(int player_id) {
@@ -441,19 +444,13 @@ void ClientNetworking::Impl::SetPlayerID(int player_id) {
     m_player_id = player_id;
 }
 
-void ClientNetworking::Impl::SetHostPlayerID(int host_player_id)
-{ m_host_player_id = host_player_id; }
-
-Networking::AuthRoles& ClientNetworking::Impl::AuthorizationRoles()
-{ return m_roles; }
-
 void ClientNetworking::Impl::SendMessage(Message&& message) {
     if (!IsTxConnected()) {
         ErrorLogger(network) << "ClientNetworking::SendMessage can't send message when not transmit connected";
         return;
     }
     TraceLogger(network) << "ClientNetworking::SendMessage() : sending message " << message;
-    m_io_context.post(boost::bind(&ClientNetworking::Impl::SendMessageImpl, this, std::move(message)));
+    boost::asio::post(m_io_context, boost::bind(&ClientNetworking::Impl::SendMessageImpl, this, std::move(message)));
 }
 
 void ClientNetworking::Impl::SendSelfMessage(Message&& message) {
@@ -469,49 +466,60 @@ boost::optional<Message> ClientNetworking::Impl::GetMessage() {
 }
 
 void ClientNetworking::Impl::HandleConnection(const boost::system::error_code& error,
-                                              tcp::resolver::iterator endpoint_it)
+                                              results_iterator endpoint_it,
+                                              const results_iterator end_of_endpoints_it)
 {
+    if (endpoint_it == end_of_endpoints_it)
+        return;
+
     DebugLogger(network) << "ClientNetworking::HandleConnection : " << endpoint_it->host_name();
+
     if (error == boost::asio::error::operation_aborted) {
         DebugLogger(network) << "ClientNetworking::HandleConnection : Operation aborted.";
         return;
-    }
-    else if (error) {
+
+    } else if (error) {
         DebugLogger(network) << "ClientNetworking::HandleConnection : connection error #"
-                             << error.value() <<" \"" << error.message() << "\""
+                             << error.value() << " \"" << error.message() << "\""
                              << "... retrying";
         m_socket.close();
         endpoint_it++;
-        if (endpoint_it == tcp::resolver::iterator())
-        {
-            endpoint_it = m_resolver_results;
-            m_reconnect_timer.expires_from_now(std::chrono::milliseconds(100));
-            m_reconnect_timer.async_wait([this, endpoint_it](const auto& err) {
+
+        if (endpoint_it == end_of_endpoints_it) {
+            endpoint_it = m_resolver_results_it;
+
+            const auto handle_connection_deadlined =
+                [this, endpoint_it, end_of_endpoints_it](const auto& error)
+            {
                 // If the m_deadline_timer has expired, it will try to cancel
                 // this timer and set the m_deadline_has_expired flag.
                 // If expiry of both timers is sufficiently close together
                 // this callback may have already been scheduled and this timer
                 // can no longer be canceled - so need to check the flag here.
-                if (err == boost::asio::error::operation_aborted
-                    || m_deadline_has_expired)
-                {
+                if (error == boost::asio::error::operation_aborted || m_deadline_has_expired) {
                     TraceLogger(network) << "ClientNetworking::Impl::m_reconnect_timer::async_wait - Canceling reconnect attempts due to deadline timeout";
                     return;
-
                 }
+
                 TraceLogger(network) << "ClientNetworking::Impl::m_reconnect_timer::async_wait - Scheduling another connection attempt";
-                m_socket.async_connect(*endpoint_it, [this, endpoint_it](const auto& error) {
-                    HandleConnection(error, endpoint_it);
-                });
-            });
+                const auto handle_connection = [this, endpoint_it, end_of_endpoints_it](const auto& error)
+                { HandleConnection(error, endpoint_it, end_of_endpoints_it); };
+
+                m_socket.async_connect(*endpoint_it, handle_connection);
+            };
+
+            m_reconnect_timer.expires_after(std::chrono::milliseconds(100));
+            m_reconnect_timer.async_wait(handle_connection_deadlined);
+
         } else {
-            m_socket.async_connect(*endpoint_it, [this, endpoint_it](const auto& error) {
-                HandleConnection(error, endpoint_it);
-            });
+            const auto handle_connection = [this, endpoint_it, end_of_endpoints_it](const auto& error)
+            { HandleConnection(error, endpoint_it, end_of_endpoints_it); };
+
+            m_socket.async_connect(*endpoint_it, handle_connection);
         }
     } else {
         m_deadline_timer.cancel();
-        const auto& endpoint = endpoint_it->endpoint();
+        const auto endpoint = endpoint_it->endpoint();
         InfoLogger(network) << "Connected to server at " << endpoint.address() << ":" << endpoint.port();
         std::scoped_lock lock(m_mutex);
         m_rx_connected = true;
@@ -519,8 +527,8 @@ void ClientNetworking::Impl::HandleConnection(const boost::system::error_code& e
     }
 }
 
-void ClientNetworking::Impl::HandleResolve(const boost::system::error_code& error, 
-                                           tcp::resolver::iterator results)
+void ClientNetworking::Impl::HandleResolve(const boost::system::error_code& error,
+                                           const results_iterator results_it, results_iterator results_end_it)
 {
     TraceLogger(network) << "ClientNetworking::Impl::HandleResolve(" << error << ")";
     if (error) {
@@ -529,28 +537,28 @@ void ClientNetworking::Impl::HandleResolve(const boost::system::error_code& erro
         return;
     }
 
-    m_resolver_results = results;
+    m_resolver_results_it = results_it;
 
     DebugLogger(network) << "Attempt to connect to server at one of these addresses:";
-    tcp::resolver::iterator end_it;
-    for (tcp::resolver::iterator it = results; it != end_it; ++it) {
+
+    for (auto it = results_it; it != results_end_it; ++it) {
         DebugLogger(network) << "host_name: " << it->host_name()
-            << "  address: " << it->endpoint().address()
-            << "  port: " << it->endpoint().port();
+                             << "  address: " << it->endpoint().address()
+                             << "  port: " << it->endpoint().port();
     }
 
     m_socket.close();
-    m_socket.async_connect(*results, [this, results](const auto& error) {
-        HandleConnection(error, results); 
-    });
+    const auto handle_connection = [this, results_it, results_end_it](const auto& error)
+    { HandleConnection(error, results_it, results_end_it); };
+    m_socket.async_connect(*results_it, handle_connection);
+
     TraceLogger(network) << "Return from ClientNetworking::Impl::HandleResolve()";
 }
 
 void ClientNetworking::Impl::HandleDeadlineTimeout(const boost::system::error_code& error)
 {
     TraceLogger(network) << "ClientNetworking::Impl::HandleDeadlineTimeout(" << error << ")";
-    if (error == boost::asio::error::operation_aborted)
-    {
+    if (error == boost::asio::error::operation_aborted) {
         // Canceled e.g. due to successfull connection
         DebugLogger(network) << "ConnectToServer() : Deadline timer cancelled.";
         return;
@@ -560,9 +568,7 @@ void ClientNetworking::Impl::HandleDeadlineTimeout(const boost::system::error_co
     m_reconnect_timer.cancel();
     bool did_close_socket = CloseSocketIfNotConnected();
     if (did_close_socket)
-    {
         DebugLogger(network) << "ConnectToServer() : Timeout.";
-    }
     TraceLogger(network) << "Return from ClientNetworking::Impl::HandleDeadlineTimeout()";
 }
 
@@ -570,12 +576,11 @@ void ClientNetworking::Impl::HandleException(const boost::system::system_error&
     if (error.code() == boost::asio::error::eof) {
         DebugLogger(network) << "Client connection disconnected by EOF from server.";
         m_socket.close();
-    }
-    else if (error.code() == boost::asio::error::connection_reset)
+    } else if (error.code() == boost::asio::error::connection_reset) {
         DebugLogger(network) << "Client connection disconnected, due to connection reset from server.";
-    else if (error.code() == boost::asio::error::operation_aborted)
+    } else if (error.code() == boost::asio::error::operation_aborted) {
         DebugLogger(network) << "Client connection closed by client.";
-    else {
+    } else {
         ErrorLogger(network) << "ClientNetworking::NetworkingThread() : Networking thread will be terminated "
                              << "due to unhandled exception error #" << error.code().value() << " \""
                              << error.code().message() << "\"";
@@ -583,7 +588,7 @@ void ClientNetworking::Impl::HandleException(const boost::system::system_error&
 }
 
 void ClientNetworking::Impl::NetworkingThread(const std::shared_ptr<const ClientNetworking> self) {
-    auto protect_from_destruction_in_other_thread = self;
+    const auto protect_from_destruction_in_other_thread{self};
     try {
         if (!m_outgoing_messages.empty())
             AsyncWriteMessage();
@@ -594,7 +599,7 @@ void ClientNetworking::Impl::NetworkingThread(const std::shared_ptr<const Client
     }
     decltype(m_outgoing_messages) empty_queue;
     m_outgoing_messages.swap(empty_queue); // clear queue
-    m_io_context.reset();
+    m_io_context.restart();
     { // Mutex scope
         std::scoped_lock lock(m_mutex);
         m_rx_connected = false;
@@ -604,7 +609,8 @@ void ClientNetworking::Impl::NetworkingThread(const std::shared_ptr<const Client
 }
 
 void ClientNetworking::Impl::HandleMessageBodyRead(const std::shared_ptr<const ClientNetworking>& keep_alive,
-                                                   boost::system::error_code error, std::size_t bytes_transferred)
+                                                   boost::system::error_code error,
+                                                   std::size_t bytes_transferred)
 {
     if (error)
         throw boost::system::system_error(error);
@@ -617,7 +623,8 @@ void ClientNetworking::Impl::HandleMessageBodyRead(const std::shared_ptr<const C
 }
 
 void ClientNetworking::Impl::HandleMessageHeaderRead(const std::shared_ptr<const ClientNetworking>& keep_alive,
-                                                     boost::system::error_code error, std::size_t bytes_transferred)
+                                                     boost::system::error_code error,
+                                                     std::size_t bytes_transferred)
 {
     if (error)
         throw boost::system::system_error(error);
@@ -656,8 +663,8 @@ void ClientNetworking::Impl::HandleMessageWrite(boost::system::error_code error,
         return;
     }
 
-    assert(static_cast<int>(bytes_transferred) <= static_cast<int>(Message::HeaderBufferSize) + m_outgoing_header[Message::Parts::SIZE]);
-    if (static_cast<int>(bytes_transferred) != static_cast<int>(Message::HeaderBufferSize) + m_outgoing_header[Message::Parts::SIZE])
+    assert(std::cmp_less_equal(bytes_transferred, Message::HeaderBufferSize + m_outgoing_header[Message::Parts::SIZE]));
+    if (std::cmp_not_equal(bytes_transferred, Message::HeaderBufferSize + m_outgoing_header[Message::Parts::SIZE]))
         return;
 
     m_outgoing_messages.pop();
@@ -730,7 +737,7 @@ void ClientNetworking::Impl::DisconnectFromServerImpl() {
 
     // Note: m_socket.is_open() may be independently true/false on each of these checks.
     if (m_socket.is_open())
-        m_socket.shutdown(boost::asio::ip::tcp::socket::shutdown_both);
+        m_socket.shutdown(tcp::socket::shutdown_both);
 }
 
 
@@ -753,40 +760,35 @@ bool ClientNetworking::IsRxConnected() const
 bool ClientNetworking::IsTxConnected() const
 { return m_impl->IsTxConnected(); }
 
-int ClientNetworking::PlayerID() const
+int ClientNetworking::PlayerID() const noexcept
 { return m_impl->PlayerID(); }
 
-int ClientNetworking::HostPlayerID() const
+int ClientNetworking::HostPlayerID() const noexcept
 { return m_impl->HostPlayerID(); }
 
-bool ClientNetworking::PlayerIsHost(int player_id) const
+bool ClientNetworking::PlayerIsHost(int player_id) const noexcept
 { return m_impl->PlayerIsHost(player_id); }
 
 bool ClientNetworking::HasAuthRole(Networking::RoleType role) const
 { return m_impl->HasAuthRole(role); }
 
-const std::string& ClientNetworking::Destination() const
+const std::string& ClientNetworking::Destination() const noexcept
 { return m_impl->Destination(); }
 
 ClientNetworking::ServerNames ClientNetworking::DiscoverLANServerNames()
 { return m_impl->DiscoverLANServerNames(); }
 
-bool ClientNetworking::ConnectToServer(
-    const std::string& ip_address,
-    const std::chrono::milliseconds& timeout)
-{ return m_impl->ConnectToServer(this, ip_address, timeout); }
+bool ClientNetworking::ConnectToServer(std::string ip_address,
+                                       std::chrono::milliseconds timeout)
+{ return m_impl->ConnectToServer(this, std::move(ip_address), timeout); }
 
-bool ClientNetworking::ConnectToLocalHostServer(
-    const std::chrono::milliseconds& timeout)
+bool ClientNetworking::ConnectToLocalHostServer(std::chrono::milliseconds timeout)
 { return m_impl->ConnectToLocalHostServer(this, timeout); }
 
-bool ClientNetworking::PingServer(
-    const std::string& ip_address,
-    const std::chrono::milliseconds& timeout)
-{ return m_impl->ConnectToServer(this, ip_address, timeout, true /*expect_timeout*/); }
+bool ClientNetworking::PingServer(std::string ip_address, std::chrono::milliseconds timeout)
+{ return m_impl->ConnectToServer(this, std::move(ip_address), timeout, true /*expect_timeout*/); }
 
-bool ClientNetworking::PingLocalHostServer(
-    const std::chrono::milliseconds& timeout)
+bool ClientNetworking::PingLocalHostServer(std::chrono::milliseconds timeout)
 { return m_impl->ConnectToLocalHostServer(this, timeout, true /*expect_timeout*/); }
 
 void ClientNetworking::DisconnectFromServer()
@@ -795,7 +797,7 @@ void ClientNetworking::DisconnectFromServer()
 void ClientNetworking::SetPlayerID(int player_id)
 { return m_impl->SetPlayerID(player_id); }
 
-void ClientNetworking::SetHostPlayerID(int host_player_id)
+void ClientNetworking::SetHostPlayerID(int host_player_id) noexcept
 { return m_impl->SetHostPlayerID(host_player_id); }
 
 Networking::AuthRoles& ClientNetworking::AuthorizationRoles()
