@@ -1,4 +1,12 @@
-from common.base_prod import TECH_COST_MULTIPLIER
+from focs._effects import (
+    EffectsGroup,
+    IsSource,
+    SetEmpireMeter,
+    Source,
+    Value,
+)
+from focs._tech import *
+from macros.base_prod import TECH_COST_MULTIPLIER
 
 Tech(
     name="CON_ARCH_PSYCH",
@@ -12,7 +20,7 @@ Tech(
     unlock=[Item(type=UnlockPolicy, name="PLC_MODERATION"), Item(type=UnlockPolicy, name="PLC_RACIAL_PURITY")],
     effectsgroups=[
         EffectsGroup(
-            scope=Source,
+            scope=IsSource,
             effects=SetEmpireMeter(empire=Source.Owner, meter="SOCIAL_CATEGORY_NUM_POLICY_SLOTS", value=Value + 1),
         )
     ],
