@@ -1,5 +1,17 @@
-from common.base_prod import TECH_COST_MULTIPLIER
-from common.priorities import TARGET_AFTER_2ND_SCALING_PRIORITY
+from focs._effects import (
+    EffectsGroup,
+    Happiness,
+    NamedReal,
+    OwnedBy,
+    Planet,
+    SetTargetResearch,
+    Source,
+    TargetPopulation,
+    Value,
+)
+from focs._tech import *
+from macros.base_prod import TECH_COST_MULTIPLIER
+from macros.priorities import TARGET_AFTER_2ND_SCALING_PRIORITY
 
 Tech(
     name="LRN_NASCENT_AI",
@@ -12,7 +24,7 @@ Tech(
     prerequisites=["LRN_ALGO_ELEGANCE"],
     effectsgroups=[
         EffectsGroup(
-            scope=ProductionCenter
+            scope=Planet()
             & OwnedBy(empire=Source.Owner)
             & TargetPopulation(low=0.0001)
             & Happiness(low=NamedReal(name="LRN_NASCENT_AI_MIN_STABILITY", value=10)),
