@@ -1,5 +1,18 @@
-from common.base_prod import INDUSTRY_PER_POP, TECH_COST_MULTIPLIER
-from common.priorities import TARGET_AFTER_SCALING_PRIORITY
+from focs._effects import (
+    EffectsGroup,
+    Focus,
+    Happiness,
+    NamedReal,
+    OwnedBy,
+    Planet,
+    SetTargetIndustry,
+    Source,
+    Target,
+    Value,
+)
+from focs._tech import *
+from macros.base_prod import INDUSTRY_PER_POP, TECH_COST_MULTIPLIER
+from macros.priorities import TARGET_AFTER_SCALING_PRIORITY
 
 Tech(
     name="PRO_ROBOTIC_PROD",
@@ -11,7 +24,7 @@ Tech(
     tags=["PEDIA_PRODUCTION_CATEGORY"],
     effectsgroups=[
         EffectsGroup(
-            scope=ProductionCenter
+            scope=Planet()
             & OwnedBy(empire=Source.Owner)
             & Focus(type=["FOCUS_INDUSTRY"])
             & Happiness(low=NamedReal(name="PRO_ROBOTIC_PROD_MIN_STABILITY", value=5)),
