@@ -1,4 +1,16 @@
-from common.base_prod import TECH_COST_MULTIPLIER
+from focs._effects import (
+    EffectsGroup,
+    IsSource,
+    NamedReal,
+    OwnedBy,
+    Planet,
+    SetDetection,
+    SetEmpireMeter,
+    Source,
+    Value,
+)
+from focs._tech import *
+from macros.base_prod import TECH_COST_MULTIPLIER
 
 Tech(
     name="SPY_DETECT_5",
@@ -15,7 +27,7 @@ Tech(
             effects=SetDetection(value=Value + NamedReal(name="SPY_DETECT_5_RANGE", value=150.0)),
         ),
         EffectsGroup(
-            scope=Source,
+            scope=IsSource,
             effects=SetEmpireMeter(empire=Source.Owner, meter="METER_DETECTION_STRENGTH", value=Value + 200),
         ),
     ],
