@@ -10,8 +10,8 @@
 #include "ValueRefPythonParser.h"
 
 condition_wrapper operator&(const condition_wrapper& lhs, const condition_wrapper& rhs) {
-    std::shared_ptr<Condition::ValueTest> lhs_cond = std::dynamic_pointer_cast<Condition::ValueTest>(lhs.condition);
-    std::shared_ptr<Condition::ValueTest> rhs_cond = std::dynamic_pointer_cast<Condition::ValueTest>(rhs.condition);
+    auto lhs_cond = std::dynamic_pointer_cast<const Condition::ValueTest>(lhs.condition);
+    auto rhs_cond = std::dynamic_pointer_cast<const Condition::ValueTest>(rhs.condition);
 
     if (lhs_cond && rhs_cond) {
         const auto lhs_vals = lhs_cond->ValuesDouble();
@@ -72,6 +72,10 @@ condition_wrapper operator&(const value_ref_wrapper<double>& lhs, const value_re
     return lhs.operator condition_wrapper() & rhs.operator condition_wrapper();
 }
 
+condition_wrapper operator&(const value_ref_wrapper<int>& lhs, const condition_wrapper& rhs) {
+    return lhs.operator condition_wrapper() & rhs;
+}
+
 
 condition_wrapper operator|(const condition_wrapper& lhs, const condition_wrapper& rhs) {
     return condition_wrapper(std::make_shared<Condition::Or>(
@@ -147,6 +151,32 @@ namespace {
         return condition_wrapper(std::make_shared<Condition::MeterValue>(m, std::move(low), std::move(high)));
     }
 
+    condition_wrapper insert_sorted_number_of_(const boost::python::tuple& args, const boost::python::dict& kw, Condition::SortingMethod method) {
+        std::unique_ptr<ValueRef::ValueRef<int>> number;
+        auto number_args = boost::python::extract<value_ref_wrapper<int>>(kw["number"]);
+        if (number_args.check()) {
+            number = ValueRef::CloneUnique(number_args().value_ref);
+        } else {
+            number = std::make_unique<ValueRef::Constant<int>>(boost::python::extract<int>(kw["number"])());
+        }
+
+        std::unique_ptr<ValueRef::ValueRef<double>> sortkey;
+        if (kw.has_key("sortkey")) {
+            auto sortkey_args = boost::python::extract<value_ref_wrapper<double>>(kw["sortkey"]);
+            if (sortkey_args.check()) {
+                sortkey = ValueRef::CloneUnique(sortkey_args().value_ref);
+            } else {
+                sortkey = std::make_unique<ValueRef::Constant<double>>(boost::python::extract<double>(kw["sortkey"])());
+            }
+        }
+
+        auto condition = ValueRef::CloneUnique(boost::python::extract<condition_wrapper>(kw["condition"])().condition);
+        return condition_wrapper(std::make_shared<Condition::SortedNumberOf>(std::move(number),
+                                                                             std::move(sortkey),
+                                                                             method,
+                                                                             std::move(condition)));
+    }
+
     condition_wrapper insert_visible_to_empire_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::unique_ptr<ValueRef::ValueRef<int>> empire;
         auto empire_args = boost::python::extract<value_ref_wrapper<int>>(kw["empire"]);
@@ -256,6 +286,20 @@ namespace {
         return condition_wrapper(std::make_shared<Condition::Species>());
     }
 
+    condition_wrapper insert_is_field_(const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::vector<std::unique_ptr<ValueRef::ValueRef<std::string>>> names;
+        boost::python::stl_input_iterator<boost::python::object> it_begin(kw["name"]), it_end;
+        for (auto it = it_begin; it != it_end; ++it) {
+            auto name_arg = boost::python::extract<value_ref_wrapper<std::string>>(*it);
+            if (name_arg.check()) {
+                names.push_back(ValueRef::CloneUnique(name_arg().value_ref));
+            } else {
+                names.push_back(std::make_unique<ValueRef::Constant<std::string>>(boost::python::extract<std::string>(*it)()));
+            }
+        }
+        return condition_wrapper(std::make_shared<Condition::Field>(std::move(names)));
+    }
+
     condition_wrapper insert_has_tag_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::unique_ptr<ValueRef::ValueRef<std::string>> name;
         if (kw.has_key("name")) {
@@ -269,6 +313,12 @@ namespace {
         return condition_wrapper(std::make_shared<Condition::HasTag>(std::move(name)));
     }
 
+    condition_wrapper insert_has_starlane_(const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::unique_ptr<Condition::Condition> from = ValueRef::CloneUnique(boost::python::extract<condition_wrapper>(kw["from_"])().condition);
+
+        return condition_wrapper(std::make_shared<Condition::HasStarlaneTo>(std::move(from)));
+    }
+
     condition_wrapper insert_focus_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::vector<std::unique_ptr<ValueRef::ValueRef<std::string>>> types;
         boost::python::stl_input_iterator<boost::python::object> it_begin(kw["type"]), it_end;
@@ -538,6 +588,18 @@ namespace {
             std::move(condition)));
     }
 
+    condition_wrapper insert_produced_by_empire_(const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::unique_ptr<ValueRef::ValueRef<int>> empire;
+        auto empire_args = boost::python::extract<value_ref_wrapper<int>>(kw["empire"]);
+        if (empire_args.check()) {
+            empire = ValueRef::CloneUnique(empire_args().value_ref);
+        } else {
+            empire = std::make_unique<ValueRef::Constant<int>>(boost::python::extract<int>(kw["empire"])());
+        }
+
+        return condition_wrapper(std::make_shared<Condition::ProducedByEmpire>(std::move(empire)));
+    }
+
     condition_wrapper insert_owner_has_tech_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::unique_ptr<ValueRef::ValueRef<std::string>> name;
         auto name_args = boost::python::extract<value_ref_wrapper<std::string>>(kw["name"]);
@@ -588,6 +650,20 @@ namespace {
         return condition_wrapper(std::make_shared<Condition::InOrIsSystem>(std::move(system_id)));
     }
 
+    condition_wrapper insert_on_planet_(const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::unique_ptr<ValueRef::ValueRef<int>> planet_id;
+        if (kw.has_key("id")) {
+            auto id_args = boost::python::extract<value_ref_wrapper<int>>(kw["id"]);
+            if (id_args.check()) {
+                planet_id = ValueRef::CloneUnique(id_args().value_ref);
+            } else {
+                planet_id = std::make_unique<ValueRef::Constant<int>>(boost::python::extract<int>(kw["id"])());
+            }
+        }
+
+        return condition_wrapper(std::make_shared<Condition::OnPlanet>(std::move(planet_id)));
+    }
+
     condition_wrapper insert_turn_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::unique_ptr<ValueRef::ValueRef<int>> low;
         if (kw.has_key("low")) {
@@ -695,7 +771,6 @@ void RegisterGlobalsConditions(boost::python::dict& globals) {
     globals["Ship"] = condition_wrapper(std::make_shared<Condition::Type>(UniverseObjectType::OBJ_SHIP));
     globals["System"] = condition_wrapper(std::make_shared<Condition::Type>(UniverseObjectType::OBJ_SYSTEM));
     globals["Fleet"] = condition_wrapper(std::make_shared<Condition::Type>(UniverseObjectType::OBJ_FLEET));
-    globals["ProductionCenter"] = condition_wrapper(std::make_shared<Condition::Type>(UniverseObjectType::OBJ_PROD_CENTER));
     globals["Monster"] = condition_wrapper(std::make_shared<Condition::Monster>());
     globals["Capital"] = condition_wrapper(std::make_shared<Condition::Capital>());
     globals["Stationary"] = condition_wrapper(std::make_shared<Condition::Stationary>());
@@ -707,12 +782,14 @@ void RegisterGlobalsConditions(boost::python::dict& globals) {
     globals["Random"] = boost::python::raw_function(insert_random_);
     globals["Star"] = boost::python::raw_function(insert_star_);
     globals["InSystem"] = boost::python::raw_function(insert_in_system_);
+    globals["OnPlanet"] = boost::python::raw_function(insert_on_planet_);
     globals["ResupplyableBy"] = boost::python::raw_function(insert_resupplyable_by_);
     globals["DesignHasPart"] = boost::python::raw_function(insert_design_has_part_);
     globals["IsBuilding"] = boost::python::raw_function(insert_building_);
     globals["Location"] = boost::python::raw_function(insert_location_);
     globals["Enqueued"] = boost::python::raw_function(insert_enqueued_);
     globals["Number"] = boost::python::raw_function(insert_number_);
+    globals["ProducedByEmpire"] = boost::python::raw_function(insert_produced_by_empire_);
 
     // non_ship_part_meter_enum_grammar
     for (const auto& meter : std::initializer_list<std::pair<const char*, MeterType>>{
@@ -768,10 +845,30 @@ void RegisterGlobalsConditions(boost::python::dict& globals) {
         globals[op.first] = enum_wrapper<Condition::ContentType>(op.second);
     }
 
+    for (const auto& op : std::initializer_list<std::pair<const char*, Condition::SortingMethod>>{
+            {"MaximumNumberOf", Condition::SortingMethod::SORT_MAX},
+            {"MinimumNumberOf", Condition::SortingMethod::SORT_MIN},
+            {"ModeNumberOf",    Condition::SortingMethod::SORT_MODE},
+            {"UniqueNumberOf",  Condition::SortingMethod::SORT_UNIQUE}})
+    {
+        const auto sm = op.second;
+        globals[op.first] = boost::python::raw_function([sm](const auto& args, const auto& kw) { return insert_sorted_number_of_(args, kw, sm); });
+    }
+    globals["NumberOf"] = boost::python::raw_function([](const auto& args, const auto& kw) { return insert_sorted_number_of_(args, kw, Condition::SortingMethod::SORT_RANDOM); });
+
     globals["HasSpecies"] = boost::python::raw_function(insert_has_species_);
+    globals["IsField"] = boost::python::raw_function(insert_is_field_);
     globals["CanColonize"] = condition_wrapper(std::make_shared<Condition::CanColonize>());
+    globals["Armed"] = condition_wrapper(std::make_shared<Condition::Armed>());
+
+    globals["IsSource"] = condition_wrapper(std::make_shared<Condition::Source>());
+    globals["IsTarget"] = condition_wrapper(std::make_shared<Condition::Target>());
+    globals["IsRootCandidate"] = condition_wrapper(std::make_shared<Condition::RootCandidate>());
+    globals["IsAnyObject"] = condition_wrapper(std::make_shared<Condition::All>());
+    globals["NoObject"] = condition_wrapper(std::make_shared<Condition::None>());
 
     globals["HasTag"] = boost::python::raw_function(insert_has_tag_);
+    globals["HasStarlane"] = boost::python::raw_function(insert_has_starlane_);
     globals["Planet"] = boost::python::raw_function(insert_planet_);
     globals["Homeworld"] = boost::python::raw_function(insert_homeworld_);
     globals["HasSpecial"] = boost::python::raw_function(insert_has_special_);
