@@ -64,6 +64,14 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
         throw std::runtime_error("Python isn't initialized");
     }
 
+    m_main_thread_state = PyThreadState_Get();
+    m_parser_thread_state = Py_NewInterpreter();
+
+    if (!m_main_thread_state && !m_parser_thread_state) {
+        ErrorLogger() << "Python parser sub-interpreter isn't initialized!";
+        throw std::runtime_error("Python sub-interpreter isn't initialized");
+    }
+
     try {
         type_int = py::import("builtins").attr("int");
         type_float = py::import("builtins").attr("float");
@@ -98,12 +106,14 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
             .def(py::self_ns::self < py::self_ns::self)
             .def(py::self_ns::self < int())
             .def(py::self_ns::self > int())
+            .def(py::self_ns::self >= int())
             .def(py::self_ns::self >= py::self_ns::self)
             .def(py::self_ns::self == py::self_ns::self)
             .def(double() - py::self_ns::self)
             .def(py::self_ns::self == int())
             .def(py::self_ns::self != int())
-            .def(py::self_ns::self | py::self_ns::self);
+            .def(py::self_ns::self | py::self_ns::self)
+            .def(py::self_ns::pow(py::self_ns::self, double()));
         py::class_<value_ref_wrapper<double>>("ValueRefDouble", py::no_init)
             .def("__call__", &value_ref_wrapper<double>::call)
             .def(int() * py::self_ns::self)
@@ -121,6 +131,7 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
             .def(py::self_ns::self + py::other<value_ref_wrapper<int>>())
             .def(py::self_ns::self - double())
             .def(py::self_ns::self - py::self_ns::self)
+            .def(py::self_ns::self - py::other<value_ref_wrapper<int>>())
             .def(int() + py::self_ns::self)
             .def(int() - py::self_ns::self)
             .def(py::self_ns::self - int())
@@ -138,11 +149,15 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
             .def(py::self_ns::pow(double(), py::self_ns::self))
             .def(py::self_ns::pow(py::self_ns::self, py::self_ns::self))
             .def(py::self_ns::self & py::self_ns::self);
-        py::class_<value_ref_wrapper<std::string>>("ValueRefString", py::no_init);
+        py::class_<value_ref_wrapper<std::string>>("ValueRefString", py::no_init)
+            .def(py::self_ns::self + std::string())
+            .def(std::string() + py::self_ns::self);
+        py::class_<value_ref_wrapper<Visibility>>("ValueRefVisibility", py::no_init);
         py::class_<condition_wrapper>("Condition", py::no_init)
             .def(py::self_ns::self & py::self_ns::self)
             .def(py::self_ns::self & py::other<value_ref_wrapper<double>>())
             .def(py::self_ns::self & py::other<value_ref_wrapper<int>>())
+            .def(py::other<value_ref_wrapper<int>>() & py::self_ns::self)
             .def(py::self_ns::self | py::self_ns::self)
             .def(py::self_ns::self | py::other<value_ref_wrapper<int>>())
             .def(~py::self_ns::self);
@@ -150,6 +165,7 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
         py::class_<effect_group_wrapper>("EffectsGroup", py::no_init);
         py::class_<enum_wrapper<UnlockableItemType>>("__UnlockableItemType", py::no_init);
         py::class_<enum_wrapper<EmpireAffiliationType>>("__EmpireAffiliationType", py::no_init);
+        py::class_<enum_wrapper<MeterType>>("__MeterType", py::no_init);
         py::class_<enum_wrapper<ResourceType>>("__ResourceType", py::no_init);
         py::class_<enum_wrapper< ::PlanetEnvironment>>("__PlanetEnvironment", py::no_init);
         py::class_<enum_wrapper<PlanetSize>>("__PlanetSize", py::no_init);
@@ -162,14 +178,14 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
         py::class_<enum_wrapper<ValueRef::StatisticType>>("__StatisticType", py::no_init);
         py::class_<enum_wrapper<Condition::ContentType>>("__LocationContentType", py::no_init);
         py::class_<enum_wrapper<BuildType>>("__BuildType", py::no_init);
+        py::class_<enum_wrapper<Visibility>>("__Visibility", py::no_init);
+        py::class_<enum_wrapper<CaptureResult>>("__CaptureResult", py::no_init);
         py::class_<unlockable_item_wrapper>("UnlockableItem", py::no_init);
         py::class_<FocusType>("__FocusType", py::no_init);
-        auto py_variable_wrapper = py::class_<variable_wrapper>("__Variable", py::no_init)
-            .def(py::self_ns::self & py::other<value_ref_wrapper<double>>())
-            .def(py::self_ns::self & py::other<condition_wrapper>())
-            .def(~py::self_ns::self);
+        auto py_variable_wrapper = py::class_<variable_wrapper>("__Variable", py::no_init);
 
         for (const char* property : {"Owner",
+                                     "OwnerBeforeLastConquered",
                                      "SupplyingEmpire",
                                      "ID",
                                      "CreationTurn",
@@ -189,6 +205,7 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
                                      "NumShips",
                                      "NumStarlanes",
                                      "LastTurnActiveInBattle",
+                                     "LastTurnAnnexed",
                                      "LastTurnAttackedByShip",
                                      "LastTurnBattleHere",
                                      "LastTurnColonized",
@@ -196,11 +213,16 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
                                      "LastTurnMoveOrdered",
                                      "LastTurnResupplied",
                                      "Orbit",
+                                     "TurnsSinceAnnexation",
                                      "TurnsSinceColonization",
                                      "TurnsSinceFocusChange",
                                      "TurnsSinceLastConquered",
                                      "ETA",
-                                     "LaunchedFrom"})
+                                     "LaunchedFrom",
+                                     "OrderedColonizePlanetID",
+                                     "OwnerBeforeLastConquered",
+                                     "LastInvadedByEmpire",
+                                     "LastColonizedByEmpire"})
         {
             py_variable_wrapper.add_property(property, py::make_function(
                 [property] (const variable_wrapper& w) { return w.get_int_property(property); },
@@ -278,13 +300,18 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
                 boost::mpl::vector<variable_wrapper, const variable_wrapper&>()));
         }
 
-        py::implicitly_convertible<variable_wrapper, condition_wrapper>();
         py::implicitly_convertible<value_ref_wrapper<double>, condition_wrapper>();
         py::implicitly_convertible<value_ref_wrapper<int>, condition_wrapper>();
 
-        m_meta_path = py::extract<py::list>(py::import("sys").attr("meta_path"));
-        m_meta_path.append(boost::cref(*this));
+        m_meta_path = py::extract<py::list>(py::import("sys").attr("meta_path"))();
+        const auto meta_path_len = py::len(*m_meta_path);
+        for (std::decay_t<decltype(meta_path_len)> i = 0; i < meta_path_len; ++i)
+            m_meta_path->pop();
 
+        m_meta_path->append(boost::cref(*this));
+        m_meta_path_len = static_cast<int>(py::len(*m_meta_path));
+
+        py::import("sys").attr("modules") = py::dict();
     } catch (const boost::python::error_already_set&) {
         m_python.HandleErrorAlreadySet();
         if (!m_python.IsPythonRunning()) {
@@ -301,13 +328,19 @@ PythonParser::PythonParser(PythonCommon& _python, const boost::filesystem::path&
 
 PythonParser::~PythonParser() {
     try {
-        m_meta_path.pop(py::len(m_meta_path) - 1);
-        // ToDo: ensure type of removed parser
-        // ToDo: clean up sys.modules
+        m_meta_path->pop(m_meta_path_len - 1);
+        type_int = py::object();
+        type_float = py::object();
+        type_bool = py::object();
+        type_str = py::object();
+        m_meta_path = boost::none;
     } catch (const py::error_already_set&) {
         ErrorLogger() << "Python parser destructor throw exception";
         m_python.HandleErrorAlreadySet();
     }
+
+    Py_EndInterpreter(m_parser_thread_state);
+    PyThreadState_Swap(m_main_thread_state);
 }
 
 bool PythonParser::ParseFileCommon(const boost::filesystem::path& path,
@@ -364,8 +397,10 @@ py::object PythonParser::find_spec(const std::string& fullname, const py::object
         module_path.replace_extension("py");
         if (IsExistingFile(module_path))
             return py::object(module_spec(fullname, parent, *this));
-        else
-            return py::object();
+        else {
+            ErrorLogger() << "Couldn't find file for module spec " << fullname;
+            throw import_error("Couldn't find file for module spec " + fullname);
+        }
     }
 }
 
@@ -391,7 +426,7 @@ py::object PythonParser::exec_module(py::object& module) {
             bool read_success = ReadFile(module_path, file_contents);
             if (!read_success) {
                 ErrorLogger() << "Unable to open data file " << module_path.string();
-                throw import_error("Unreadable module");
+                throw import_error("Unreadable module " + fullname);
             }
 
             // store globals content in module namespace
