@@ -13,9 +13,20 @@
 #include "EnumPythonParser.h"
 #include "PythonParser.h"
 
+value_ref_wrapper<double> pow(const value_ref_wrapper<int>& lhs, double rhs) {
+    return value_ref_wrapper<double>(
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::EXPONENTIATE,
+            std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(lhs.value_ref)),
+            std::make_unique<ValueRef::Constant<double>>(rhs)
+        )
+    );
+}
+
 value_ref_wrapper<double> pow(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::EXPONENTIATE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::EXPONENTIATE,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
         )
@@ -24,7 +35,8 @@ value_ref_wrapper<double> pow(const value_ref_wrapper<double>& lhs, double rhs)
 
 value_ref_wrapper<double> pow(double lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::EXPONENTIATE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::EXPONENTIATE,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -33,7 +45,8 @@ value_ref_wrapper<double> pow(double lhs, const value_ref_wrapper<double>& rhs)
 
 value_ref_wrapper<double> pow(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::EXPONENTIATE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::EXPONENTIATE,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -42,8 +55,9 @@ value_ref_wrapper<double> pow(const value_ref_wrapper<double>& lhs, const value_
 
 value_ref_wrapper<double> operator*(int lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
-            std::make_unique<ValueRef::Constant<double>>(lhs),
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(lhs)),
             ValueRef::CloneUnique(rhs.value_ref)
         )
     );
@@ -51,7 +65,8 @@ value_ref_wrapper<double> operator*(int lhs, const value_ref_wrapper<double>& rh
 
 value_ref_wrapper<double> operator*(const value_ref_wrapper<int>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(lhs.value_ref)),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -60,7 +75,8 @@ value_ref_wrapper<double> operator*(const value_ref_wrapper<int>& lhs, const val
 
 value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
@@ -69,7 +85,8 @@ value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
         )
@@ -78,7 +95,8 @@ value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, double
 
 value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -87,7 +105,8 @@ value_ref_wrapper<double> operator*(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator/(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::DIVIDE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::DIVIDE,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -96,16 +115,18 @@ value_ref_wrapper<double> operator/(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator/(const value_ref_wrapper<double>& lhs, int rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::DIVIDE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::DIVIDE,
             ValueRef::CloneUnique(lhs.value_ref),
-            std::make_unique<ValueRef::Constant<double>>(rhs)
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(rhs))
         )
     );
 }
 
 value_ref_wrapper<double> operator/(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::DIVIDE,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::DIVIDE,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
         )
@@ -114,7 +135,8 @@ value_ref_wrapper<double> operator/(const value_ref_wrapper<double>& lhs, double
 
 value_ref_wrapper<double> operator*(double lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -123,7 +145,8 @@ value_ref_wrapper<double> operator*(double lhs, const value_ref_wrapper<double>&
 
 value_ref_wrapper<double> operator*(double lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::TIMES,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
@@ -133,7 +156,7 @@ value_ref_wrapper<double> operator*(double lhs, const value_ref_wrapper<int>& rh
 value_ref_wrapper<double> operator+(int lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
         std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
-            std::make_unique<ValueRef::Constant<double>>(lhs),
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(lhs)),
             ValueRef::CloneUnique(rhs.value_ref)
         )
     );
@@ -141,16 +164,18 @@ value_ref_wrapper<double> operator+(int lhs, const value_ref_wrapper<double>& rh
 
 value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, int rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
-            std::make_unique<ValueRef::Constant<double>>(rhs)
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(rhs))
         )
     );
 }
 
 value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
         )
@@ -159,7 +184,8 @@ value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, double
 
 value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -168,7 +194,8 @@ value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
@@ -177,7 +204,8 @@ value_ref_wrapper<double> operator+(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator+(double lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::PLUS,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
@@ -186,7 +214,8 @@ value_ref_wrapper<double> operator+(double lhs, const value_ref_wrapper<int>& rh
 
 value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
         )
@@ -195,7 +224,8 @@ value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, double
 
 value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -204,8 +234,9 @@ value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator-(int lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::MINUS,
-            std::make_unique<ValueRef::Constant<double>>(lhs),
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(lhs)),
             ValueRef::CloneUnique(rhs.value_ref)
         )
     );
@@ -213,7 +244,8 @@ value_ref_wrapper<double> operator-(int lhs, const value_ref_wrapper<double>& rh
 
 value_ref_wrapper<double> operator-(double lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
@@ -222,24 +254,37 @@ value_ref_wrapper<double> operator-(double lhs, const value_ref_wrapper<int>& rh
 
 value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, int rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
             ValueRef::CloneUnique(lhs.value_ref),
-            std::make_unique<ValueRef::Constant<double>>(rhs)
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(rhs))
+        )
+    );
+}
+
+value_ref_wrapper<double> operator-(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<int>& rhs) {
+    return value_ref_wrapper<double>(
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::MINUS,
+            ValueRef::CloneUnique(lhs.value_ref),
+            std::make_unique<ValueRef::StaticCast<int, double>>(ValueRef::CloneUnique(rhs.value_ref))
         )
     );
 }
 
 value_ref_wrapper<double> operator>=(const value_ref_wrapper<double>& lhs, int rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
-            std::make_unique<ValueRef::Constant<double>>(rhs)
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(rhs))
     ));
 }
 
 value_ref_wrapper<double> operator<=(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
     ));
@@ -247,7 +292,8 @@ value_ref_wrapper<double> operator<=(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator<=(double lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref)
     ));
@@ -255,7 +301,8 @@ value_ref_wrapper<double> operator<=(double lhs, const value_ref_wrapper<double>
 
 value_ref_wrapper<double> operator<=(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs)
     ));
@@ -263,7 +310,8 @@ value_ref_wrapper<double> operator<=(const value_ref_wrapper<double>& lhs, doubl
 
 value_ref_wrapper<double> operator>(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_GREATER_THAN,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -271,7 +319,8 @@ value_ref_wrapper<double> operator>(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator>=(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -279,7 +328,8 @@ value_ref_wrapper<double> operator>=(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator<(const value_ref_wrapper<double>& lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -287,7 +337,8 @@ value_ref_wrapper<double> operator<(const value_ref_wrapper<double>& lhs, const
 
 value_ref_wrapper<double> operator<(double lhs, const value_ref_wrapper<double>& rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN,
             std::make_unique<ValueRef::Constant<double>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -295,7 +346,8 @@ value_ref_wrapper<double> operator<(double lhs, const value_ref_wrapper<double>&
 
 value_ref_wrapper<double> operator<(const value_ref_wrapper<double>& lhs, double rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_LESS_THAN,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_LESS_THAN,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<double>>(rhs))
     );
@@ -303,15 +355,17 @@ value_ref_wrapper<double> operator<(const value_ref_wrapper<double>& lhs, double
 
 value_ref_wrapper<double> operator!=(const value_ref_wrapper<double>& lhs, int rhs) {
     return value_ref_wrapper<double>(
-        std::make_shared<ValueRef::Operation<double>>(ValueRef::OpType::COMPARE_NOT_EQUAL,
+        std::make_shared<ValueRef::Operation<double>>(
+            ValueRef::OpType::COMPARE_NOT_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
-            std::make_unique<ValueRef::Constant<double>>(rhs))
+            std::make_unique<ValueRef::Constant<double>>(static_cast<double>(rhs)))
     );
 }
 
 value_ref_wrapper<int> operator*(int lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::TIMES,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::TIMES,
             std::make_unique<ValueRef::Constant<int>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -320,7 +374,8 @@ value_ref_wrapper<int> operator*(int lhs, const value_ref_wrapper<int>& rhs) {
 
 value_ref_wrapper<int> operator-(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::MINUS,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs)
         )
@@ -329,7 +384,8 @@ value_ref_wrapper<int> operator-(const value_ref_wrapper<int>& lhs, int rhs) {
 
 value_ref_wrapper<int> operator-(int lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::MINUS,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::MINUS,
             std::make_unique<ValueRef::Constant<int>>(lhs),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -338,7 +394,8 @@ value_ref_wrapper<int> operator-(int lhs, const value_ref_wrapper<int>& rhs) {
 
 value_ref_wrapper<int> operator+(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs)
         )
@@ -347,7 +404,8 @@ value_ref_wrapper<int> operator+(const value_ref_wrapper<int>& lhs, int rhs) {
 
 value_ref_wrapper<int> operator+(const value_ref_wrapper<int>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::PLUS,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::PLUS,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref)
         )
@@ -356,7 +414,8 @@ value_ref_wrapper<int> operator+(const value_ref_wrapper<int>& lhs, const value_
 
 value_ref_wrapper<int> operator<(const value_ref_wrapper<int>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_LESS_THAN,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_LESS_THAN,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -364,7 +423,8 @@ value_ref_wrapper<int> operator<(const value_ref_wrapper<int>& lhs, const value_
 
 value_ref_wrapper<int> operator<(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_LESS_THAN,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_LESS_THAN,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs))
     );
@@ -372,7 +432,17 @@ value_ref_wrapper<int> operator<(const value_ref_wrapper<int>& lhs, int rhs) {
 
 value_ref_wrapper<int> operator>(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_GREATER_THAN,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN,
+            ValueRef::CloneUnique(lhs.value_ref),
+            std::make_unique<ValueRef::Constant<int>>(rhs))
+    );
+}
+
+value_ref_wrapper<int> operator>=(const value_ref_wrapper<int>& lhs, int rhs) {
+    return value_ref_wrapper<int>(
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs))
     );
@@ -380,7 +450,8 @@ value_ref_wrapper<int> operator>(const value_ref_wrapper<int>& lhs, int rhs) {
 
 value_ref_wrapper<int> operator>=(const value_ref_wrapper<int>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_GREATER_THAN_OR_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -388,7 +459,8 @@ value_ref_wrapper<int> operator>=(const value_ref_wrapper<int>& lhs, const value
 
 value_ref_wrapper<int> operator==(const value_ref_wrapper<int>& lhs, const value_ref_wrapper<int>& rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_EQUAL,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             ValueRef::CloneUnique(rhs.value_ref))
     );
@@ -396,7 +468,8 @@ value_ref_wrapper<int> operator==(const value_ref_wrapper<int>& lhs, const value
 
 value_ref_wrapper<int> operator==(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_EQUAL,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs))
     );
@@ -404,12 +477,31 @@ value_ref_wrapper<int> operator==(const value_ref_wrapper<int>& lhs, int rhs) {
 
 value_ref_wrapper<int> operator!=(const value_ref_wrapper<int>& lhs, int rhs) {
     return value_ref_wrapper<int>(
-        std::make_shared<ValueRef::Operation<int>>(ValueRef::OpType::COMPARE_NOT_EQUAL,
+        std::make_shared<ValueRef::Operation<int>>(
+            ValueRef::OpType::COMPARE_NOT_EQUAL,
             ValueRef::CloneUnique(lhs.value_ref),
             std::make_unique<ValueRef::Constant<int>>(rhs))
     );
 }
 
+value_ref_wrapper<std::string> operator+(const value_ref_wrapper<std::string>& lhs, const std::string& rhs) {
+    return value_ref_wrapper<std::string>(
+        std::make_shared<ValueRef::Operation<std::string>>(
+            ValueRef::OpType::PLUS,
+            ValueRef::CloneUnique(lhs.value_ref),
+            std::make_unique<ValueRef::Constant<std::string>>(rhs))
+    );
+}
+
+value_ref_wrapper<std::string> operator+(const std::string& lhs, const value_ref_wrapper<std::string>& rhs) {
+    return value_ref_wrapper<std::string>(
+        std::make_shared<ValueRef::Operation<std::string>>(
+            ValueRef::OpType::PLUS,
+            std::make_unique<ValueRef::Constant<std::string>>(lhs),
+            ValueRef::CloneUnique(rhs.value_ref))
+    );
+}
+
 namespace {
     template<typename T>
     value_ref_wrapper<T> insert_named_lookup_(const boost::python::tuple& args, const boost::python::dict& kw) {
@@ -422,13 +514,12 @@ namespace {
     value_ref_wrapper<T> insert_named_(const boost::python::tuple& args, const boost::python::dict& kw) {
         auto name = boost::python::extract<std::string>(kw["name"])();
         std::unique_ptr<ValueRef::ValueRef<T>> value;
-        
+
         auto value_arg = boost::python::extract<value_ref_wrapper<T>>(kw["value"]);
-        if (value_arg.check()) {
+        if (value_arg.check())
             value = ValueRef::CloneUnique(value_arg().value_ref);
-        } else {
+        else
             value = std::make_unique<ValueRef::Constant<T>>(boost::python::extract<T>(kw["value"])());
-        }
 
         ::RegisterValueRef<T>(name, std::move(value));
 
@@ -470,7 +561,18 @@ namespace {
                     operands.push_back(std::make_unique<ValueRef::Constant<std::string>>(boost::python::extract<std::string>(args[i])()));
             }
             return boost::python::object(value_ref_wrapper<std::string>(std::make_shared<ValueRef::Operation<std::string>>(op, std::move(operands))));
-        } else {
+        } else if (args[0] == "Visibility") {
+            std::vector<std::unique_ptr<ValueRef::ValueRef<Visibility>>> operands;
+            operands.reserve(boost::python::len(args) - 1);
+            for (auto i = 1; i < boost::python::len(args); i++) {
+                auto arg = boost::python::extract<value_ref_wrapper<Visibility>>(args[i]);
+                if (arg.check())
+                    operands.push_back(ValueRef::CloneUnique(arg().value_ref));
+                else
+                    operands.push_back(std::make_unique<ValueRef::Constant<Visibility>>(boost::python::extract<enum_wrapper<Visibility>>(args[i])().value));
+            }
+            return boost::python::object(value_ref_wrapper<Visibility>(std::make_shared<ValueRef::Operation<Visibility>>(op, std::move(operands))));
+        }{
             ErrorLogger() << "Unsupported type for min/max/oneof : " << boost::python::extract<std::string>(boost::python::str(args[0]))();
 
             throw std::runtime_error(std::string("Not implemented ") + __func__);
@@ -566,7 +668,7 @@ namespace {
     }
 
     boost::python::object insert_game_rule_(const PythonParser& parser, const boost::python::tuple& args, const boost::python::dict& kw) {
-        auto name = boost::python::extract<std::string>(kw["name"])();
+        auto name{boost::python::extract<std::string>(kw["name"])()};
         auto type_ = kw["type"];
 
         if (type_ == parser.type_int) {
@@ -575,7 +677,7 @@ namespace {
                 nullptr,
                 nullptr,
                 nullptr,
-                std::make_unique<ValueRef::Constant<std::string>>(name),
+                std::make_unique<ValueRef::Constant<std::string>>(std::move(name)),
                 nullptr)));
         } else if (type_ == parser.type_float) {
             return boost::python::object(value_ref_wrapper<double>(std::make_shared<ValueRef::ComplexVariable<double>>(
@@ -583,10 +685,11 @@ namespace {
                 nullptr,
                 nullptr,
                 nullptr,
-                std::make_unique<ValueRef::Constant<std::string>>(name),
+                std::make_unique<ValueRef::Constant<std::string>>(std::move(name)),
                 nullptr)));
         } else {
-            ErrorLogger() << "Unsupported type for rule " << name << ": " << boost::python::extract<std::string>(boost::python::str(type_))();
+            ErrorLogger() << "Unsupported type for rule " << name << ": "
+                          << boost::python::extract<std::string>(boost::python::str(type_))();
 
             throw std::runtime_error(std::string("Not implemented ") + __func__);
         }
@@ -594,7 +697,10 @@ namespace {
         return boost::python::object();
     }
 
-    boost::python::object insert_int_complex_variable_(const char* variable, const boost::python::tuple& args, const boost::python::dict& kw) {
+    boost::python::object insert_int_complex_variable_(const char* variable,
+                                                       const boost::python::tuple& args,
+                                                       const boost::python::dict& kw)
+    {
         std::unique_ptr<ValueRef::ValueRef<int>> empire;
         if (kw.has_key("empire")) {
             auto empire_args = boost::python::extract<value_ref_wrapper<int>>(kw["empire"]);
@@ -644,6 +750,33 @@ namespace {
             nullptr));
     }
 
+    value_ref_wrapper<double> insert_double_complex_variable_species_(const char* variable, const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::unique_ptr<ValueRef::ValueRef<std::string>> species;
+        auto species_args = boost::python::extract<value_ref_wrapper<std::string>>(kw["species"]);
+        if (species_args.check()) {
+            species = ValueRef::CloneUnique(species_args().value_ref);
+        } else {
+            species = std::make_unique<ValueRef::Constant<std::string>>(boost::python::extract<std::string>(kw["species"])());
+        }
+
+        std::unique_ptr<ValueRef::ValueRef<int>> empire;
+        auto empire_args = boost::python::extract<value_ref_wrapper<int>>(kw["empire"]);
+        if (empire_args.check()) {
+            empire = ValueRef::CloneUnique(empire_args().value_ref);
+        } else {
+            empire = std::make_unique<ValueRef::Constant<int>>(boost::python::extract<int>(kw["empire"])());
+        }
+
+        return value_ref_wrapper<double>(std::make_shared<ValueRef::ComplexVariable<double>>(
+            variable,
+            std::move(empire),
+            nullptr,
+            nullptr,
+            std::move(species),
+            nullptr));
+
+    }
+
     value_ref_wrapper<double> insert_direct_distance_between_(boost::python::object arg1, boost::python::object arg2) {
         std::unique_ptr<ValueRef::ValueRef<int>> id1;
         auto id1_args = boost::python::extract<value_ref_wrapper<int>>(arg1);
@@ -740,6 +873,38 @@ namespace {
         ));
     }
 
+    value_ref_wrapper<double> insert_ship_part_meter_(const boost::python::tuple& args, const boost::python::dict& kw) {
+        std::unique_ptr<ValueRef::ValueRef<std::string>> part;
+        if (kw.has_key("part")) {
+            auto part_args = boost::python::extract<value_ref_wrapper<std::string>>(kw["part"]);
+            if (part_args.check()) {
+                part = ValueRef::CloneUnique(part_args().value_ref);
+            } else {
+                part = std::make_unique<ValueRef::Constant<std::string>>(boost::python::extract<std::string>(kw["part"])());
+            }
+        }
+
+        auto meter_name = ValueRef::MeterToName(boost::python::extract<enum_wrapper<MeterType>>(kw["meter"])().value);
+        auto meter_type = std::make_unique<ValueRef::Constant<std::string>>(std::string{meter_name});
+
+        std::unique_ptr<ValueRef::ValueRef<int>> ship_id;
+        auto ship_args = boost::python::extract<value_ref_wrapper<int>>(kw["ship"]);
+        if (ship_args.check()) {
+            ship_id = ValueRef::CloneUnique(ship_args().value_ref);
+        } else {
+            ship_id = std::make_unique<ValueRef::Constant<int>>(boost::python::extract<int>(kw["ship"])());
+        }
+
+        return value_ref_wrapper<double>(std::make_shared<ValueRef::ComplexVariable<double>>(
+            "ShipPartMeter",
+            std::move(ship_id),
+            nullptr,
+            nullptr,
+            std::move(part),
+            std::move(meter_type)
+        ));
+    }
+
     value_ref_wrapper<double> insert_empire_meter_value_(const boost::python::tuple& args, const boost::python::dict& kw) {
         std::unique_ptr<ValueRef::ValueRef<int>> empire;
         auto empire_args = boost::python::extract<value_ref_wrapper<int>>(kw["empire"]);
@@ -756,7 +921,7 @@ namespace {
             std::move(empire),
             nullptr,
             nullptr,
-            std::move(std::make_unique<ValueRef::Constant<std::string>>(meter)),
+            std::make_unique<ValueRef::Constant<std::string>>(meter),
             nullptr
         ));
     }
@@ -787,7 +952,7 @@ namespace {
             std::move(empire),
             nullptr,
             nullptr,
-            std::move(std::make_unique<ValueRef::Constant<std::string>>(resource_str)),
+            std::make_unique<ValueRef::Constant<std::string>>(resource_str),
             nullptr
         ));
     }
@@ -799,6 +964,7 @@ void RegisterGlobalsValueRefs(boost::python::dict& globals, const PythonParser&
     globals["NamedReal"] = boost::python::raw_function(insert_named_<double>);
     globals["NamedRealLookup"] = boost::python::raw_function(insert_named_lookup_<double>);
     globals["Value"] = value_ref_wrapper<double>(std::make_shared<ValueRef::Variable<double>>(ValueRef::ReferenceType::EFFECT_TARGET_VALUE_REFERENCE));
+    globals["ValueVisibility"] = value_ref_wrapper<Visibility>(std::make_shared<ValueRef::Variable<Visibility>>(ValueRef::ReferenceType::EFFECT_TARGET_VALUE_REFERENCE));
 
     // free variable name
     for (const char* variable : {"CombatBout",
@@ -852,6 +1018,13 @@ void RegisterGlobalsValueRefs(boost::python::dict& globals, const PythonParser&
         globals[variable] = boost::python::raw_function(f_insert_name_property);
     }
 
+    // species_empire_opinion
+    for (const char* variable : {"SpeciesEmpireOpinion",
+                                 "SpeciesEmpireTargetOpinion"})
+    {
+        globals[variable] = boost::python::raw_function([variable](const boost::python::tuple& args, const boost::python::dict& kw) { return insert_double_complex_variable_species_(variable, args, kw); });
+    }
+
     // single-parameter math functions
     for (const auto& op : std::initializer_list<std::pair<const char*, ValueRef::OpType>>{
             {"Sin",   ValueRef::OpType::SINE},
@@ -868,8 +1041,9 @@ void RegisterGlobalsValueRefs(boost::python::dict& globals, const PythonParser&
     }
 
     // CurrentContent
-    const auto current_content = value_ref_wrapper<std::string>(std::make_shared<ValueRef::Constant<std::string>>("CurrentContent"));
-    for (const char* variable : {"CurrentContent",
+    const auto current_content = value_ref_wrapper<std::string>(
+        std::make_shared<ValueRef::Constant<std::string>>(std::string{ValueRef::Constant<std::string>::current_content}));
+    for (const char* variable : {ValueRef::Constant<std::string>::current_content.data(),
                                  "ThisBuilding",
                                  "ThisField",
                                  "ThisHull",
@@ -909,6 +1083,7 @@ void RegisterGlobalsValueRefs(boost::python::dict& globals, const PythonParser&
     globals["JumpsBetween"] = insert_jumps_between_;
     globals["UserString"] = insert_user_string;
     globals["PartsInShipDesign"] = boost::python::raw_function(insert_parts_in_ship_design_);
+    globals["ShipPartMeter"] = boost::python::raw_function(insert_ship_part_meter_);
     globals["EmpireMeterValue"] = boost::python::raw_function(insert_empire_meter_value_);
     globals["EmpireStockpile"] = boost::python::raw_function(insert_empire_stockpile_);
 
