@@ -1,3 +1,4 @@
+#include "../util/boost_fix.h"
 #include <boost/mpl/vector.hpp>
 #include <boost/python.hpp>
 #include <boost/python/suite/indexing/map_indexing_suite.hpp>
@@ -14,8 +15,6 @@
 #include "../universe/NamedValueRefManager.h"
 #include "../universe/Pathfinder.h"
 #include "../universe/Planet.h"
-#include "../universe/PopCenter.h"
-#include "../universe/ResourceCenter.h"
 #include "../universe/ScriptingContext.h"
 #include "../universe/ShipDesign.h"
 #include "../universe/Ship.h"
@@ -56,7 +55,7 @@ namespace {
 
         std::transform(tags.first.begin(), tags.first.end(), std::back_inserter(result),
                        [](std::string_view sv) { return std::string{sv}; });
-        std::transform(tags.first.begin(), tags.first.end(), std::back_inserter(result),
+        std::transform(tags.second.begin(), tags.second.end(), std::back_inserter(result),
                        [](std::string_view sv) { return std::string{sv}; });
         return result;
     }
@@ -98,14 +97,13 @@ namespace {
         auto it = species_homeworlds.find(species.Name());
         if (it == species_homeworlds.end())
             return {};
-        return it->second;
+        return {it->second.begin(), it->second.end()};
     }
 
-    void UpdateMetersWrapper(Universe& universe, const py::object& objIter)
+    void UpdateMetersWrapper(Universe& universe, const py::object&)
     {
-        py::stl_input_iterator<int> begin(objIter), end;
-        std::vector<int> objvec(begin, end);
-        ScriptingContext context{universe, Empires(), GetGalaxySetupData(), GetSpeciesManager(), GetSupplyManager()};
+        ScriptingContext context{universe, Empires(), GetGalaxySetupData(),
+                                 GetSpeciesManager(), GetSupplyManager()};
         universe.UpdateMeterEstimates(context);
     }
 
@@ -199,24 +197,24 @@ namespace {
 
     auto HullProductionLocation(const ShipHull& hull, int location_id) -> bool
     {
-        auto location = Objects().getRaw(location_id);
+        UniverseObject* location = Objects().getRaw(location_id); // intentionally getting a mutable pointer to construct context with
         if (!location) {
             ErrorLogger() << "UniverseWrapper::HullProductionLocation Could not find location with id " << location_id;
             return false;
         }
-        ScriptingContext location_as_source_context{location, location};
-        return hull.Location()->Eval(location_as_source_context, location);
+        const ScriptingContext location_as_source_context{location, location};
+        return hull.Location()->EvalOne(location_as_source_context, location);
     }
 
     auto ShipPartProductionLocation(const ShipPart& part_type, int location_id) -> bool
     {
-        auto location = Objects().getRaw(location_id);
+        UniverseObject*  location = Objects().getRaw(location_id);
         if (!location) {
             ErrorLogger() << "UniverseWrapper::PartTypeProductionLocation Could not find location with id " << location_id;
             return false;
         }
-        ScriptingContext location_as_source_context{location, location};
-        return part_type.Location()->Eval(location_as_source_context, location);
+        const ScriptingContext location_as_source_context{location, location};
+        return part_type.Location()->EvalOne(location_as_source_context, location);
     }
 
     template <typename X>
@@ -234,6 +232,17 @@ namespace {
         return retval;
     }
 
+    std::vector<std::string> ViewsToStrings(const std::vector<std::string_view>& svs) {
+        std::vector<std::string> retval;
+        retval.reserve(svs.size());
+        std::transform(svs.begin(), svs.end(), std::back_inserter(retval),
+                       [](const auto& sv) { return std::string{sv}; });
+        return retval;
+    }
+
+    template <typename T>
+    std::vector<T> ToVec(const boost::container::flat_set<T>& in)
+    { return std::vector<T>(in.begin(), in.end()); }
 }
 
 namespace FreeOrionPython {
@@ -283,8 +292,10 @@ namespace FreeOrionPython {
 
         py::to_python_converter<std::pair<MeterType, std::string>,
                                 PairToTupleConverter<std::pair<MeterType, std::string>>>();
+        py::to_python_converter<std::pair<std::string, MeterType>,
+                                PairToTupleConverter<std::pair<std::string, MeterType>>>();
 
-        py::class_<Ship::PartMeterMap>("MeterTypeStringPairMeterMap")
+        py::class_<Ship::PartMeterMap>("StringMeterTypePairMeterMap")
             .def(py::map_indexing_suite<Ship::PartMeterMap>())
         ;
 
@@ -392,7 +403,6 @@ namespace FreeOrionPython {
                                                 py::return_value_policy<py::reference_existing_object>(),
                                                 "Returns the ship design (ShipDesign) with the indicated name (string).")
 
-            .add_property("allObjectIDs",       make_function(ObjectIDs<UniverseObject>,py::return_value_policy<py::return_by_value>()))
             .add_property("fleetIDs",           make_function(ObjectIDs<Fleet>,         py::return_value_policy<py::return_by_value>()))
             .add_property("systemIDs",          make_function(ObjectIDs<System>,        py::return_value_policy<py::return_by_value>()))
             .add_property("fieldIDs",           make_function(ObjectIDs<Field>,         py::return_value_policy<py::return_by_value>()))
@@ -480,7 +490,7 @@ namespace FreeOrionPython {
             .def("specialAddedOnTurn",          &UniverseObject::SpecialAddedOnTurn)
             .def("contains",                    &UniverseObject::Contains)
             .def("containedBy",                 &UniverseObject::ContainedBy)
-            .add_property("containedObjects",   make_function(&UniverseObject::ContainedObjectIDs,  py::return_value_policy<py::return_by_value>()))
+            .add_property("containedObjects",   +[](const UniverseObject& o) { return ToVec(o.ContainedObjectIDs()); })
             .add_property("containerObject",    &UniverseObject::ContainerObjectID)
             .def("currentMeterValue",           ObjectCurrentMeterValue,             py::return_value_policy<py::return_by_value>())
             .def("initialMeterValue",           ObjectInitialMeterValue,             py::return_value_policy<py::return_by_value>())
@@ -520,7 +530,7 @@ namespace FreeOrionPython {
             .add_property("hasTroopShips",             +[](const Fleet& fleet) -> bool { return fleet.HasTroopShips(GetUniverse()); })
             .add_property("numShips",                  &Fleet::NumShips)
             .add_property("empty",                     &Fleet::Empty)
-            .add_property("shipIDs",                   make_function(&Fleet::ShipIDs,      py::return_internal_reference<>()))
+            .add_property("shipIDs",                   +[](const Fleet& fleet) { return ToVec(fleet.ShipIDs()); })
         ;
 
         //////////////////
@@ -656,6 +666,7 @@ namespace FreeOrionPython {
             .add_property("stealth",            &ShipHull::Stealth)
             .add_property("fuel",               &ShipHull::Fuel)
             .add_property("speed",              &ShipHull::Speed)
+            .add_property("detection",          &ShipHull::Detection)
             .def("numSlotsOfSlotType",          +[](const ShipHull& hull, ShipSlotType slot_type) -> unsigned int { return hull.NumSlots(slot_type); })
             .add_property("slots",              make_function(
                                                     HullSlots,
@@ -688,9 +699,9 @@ namespace FreeOrionPython {
         py::class_<BuildingType, boost::noncopyable>("buildingType", py::no_init)
             .add_property("name",               make_function(&BuildingType::Name,          py::return_value_policy<py::copy_const_reference>()))
             .add_property("description",        make_function(&BuildingType::Description,   py::return_value_policy<py::copy_const_reference>()))
-            .def("productionCost",              +[](const BuildingType& bt, int empire_id, int location_id) -> float { return bt.ProductionCost(empire_id, location_id); })
-            .def("productionTime",              +[](const BuildingType& bt, int empire_id, int location_id) -> int { return bt.ProductionTime(empire_id, location_id); })
-            .def("perTurnCost",                 +[](const BuildingType& bt, int empire_id, int location_id) -> float { return bt.PerTurnCost(empire_id, location_id); })
+            .def("productionCost",              +[](const BuildingType& bt, int empire_id, int location_id) -> float { return bt.ProductionCost(empire_id, location_id, ScriptingContext{}); })
+            .def("productionTime",              +[](const BuildingType& bt, int empire_id, int location_id) -> int { return bt.ProductionTime(empire_id, location_id, ScriptingContext{}); })
+            .def("perTurnCost",                 +[](const BuildingType& bt, int empire_id, int location_id) -> float { return bt.PerTurnCost(empire_id, location_id, ScriptingContext{}); })
             .def("captureResult",               &BuildingType::GetCaptureResult)
             .def("canBeProduced",               +[](const BuildingType& building_type, int empire_id, int loc_id) -> bool { const ScriptingContext context; return building_type.ProductionLocation(empire_id, loc_id, context); })
             .def("canBeEnqueued",               +[](const BuildingType& building_type, int empire_id, int loc_id) -> bool { const ScriptingContext context; return building_type.EnqueueLocation(empire_id, loc_id, context); })
@@ -701,44 +712,33 @@ namespace FreeOrionPython {
         py::def("getBuildingType",              +[](const std::string& name) { return GetBuildingType(name); },
                                                 py::return_value_policy<py::reference_existing_object>(),
                                                 "Returns the building type (BuildingType) with the indicated name (string).");
-        ////////////////////
-        // ResourceCenter //
-        ////////////////////
-        py::class_<ResourceCenter, boost::noncopyable>("resourceCenter", py::no_init)
-            .add_property("focus",                  make_function(&ResourceCenter::Focus,   py::return_value_policy<py::copy_const_reference>()))
-            .add_property("turnsSinceFocusChange" , +[](const ResourceCenter& rc) { return rc.TurnsSinceFocusChange(CurrentTurn()); })
-            .add_property("availableFoci",          +[](const ResourceCenter& rc) -> std::vector<std::string> { return rc.AvailableFoci(ScriptingContext{}); })
-        ;
-
-        ///////////////////
-        //   PopCenter   //
-        ///////////////////
-        py::class_<PopCenter, boost::noncopyable>("popCenter", py::no_init)
-            .add_property("speciesName",        make_function(&PopCenter::SpeciesName,      py::return_value_policy<py::copy_const_reference>()))
-        ;
 
         //////////////////
         //    Planet    //
         //////////////////
-        py::class_<Planet, py::bases<UniverseObject, PopCenter, ResourceCenter>, boost::noncopyable>("planet", py::no_init)
+        py::class_<Planet, py::bases<UniverseObject>, boost::noncopyable>("planet", py::no_init)
+            .add_property("speciesName",                    make_function(&Planet::SpeciesName, py::return_value_policy<py::copy_const_reference>()))
+            .add_property("focus",                          make_function(&Planet::Focus,       py::return_value_policy<py::copy_const_reference>()))
+            .add_property("turnsSinceFocusChange" ,         +[](const Planet& rc) { return rc.TurnsSinceFocusChange(CurrentTurn()); })
+            .add_property("availableFoci",                  +[](const Planet& rc) -> std::vector<std::string> { return ViewsToStrings(rc.AvailableFoci(ScriptingContext{})); })
             .add_property("size",                           &Planet::Size)
             .add_property("type",                           &Planet::Type)
             .add_property("originalType",                   &Planet::OriginalType)
             .add_property("distanceFromOriginalType",       &Planet::DistanceFromOriginalType)
-            .def("environmentForSpecies",                   +[](const Planet& planet, const std::string& species) { return planet.EnvironmentForSpecies(ScriptingContext{}, species); })
+            .def("environmentForSpecies",                   +[](const Planet& planet, const std::string& species) { return planet.EnvironmentForSpecies(GetSpeciesManager(), species); })
             .def("nextBetterPlanetTypeForSpecies",          &Planet::NextBetterPlanetTypeForSpecies)
             .add_property("clockwiseNextPlanetType",        &Planet::ClockwiseNextPlanetType)
             .add_property("counterClockwiseNextPlanetType", &Planet::CounterClockwiseNextPlanetType)
             .add_property("nextLargerPlanetSize",           &Planet::NextLargerPlanetSize)
             .add_property("nextSmallerPlanetSize",          &Planet::NextSmallerPlanetSize)
-            .add_property("OrbitalPeriod",                  &Planet::OrbitalPeriod)
-            .add_property("InitialOrbitalPosition",         &Planet::InitialOrbitalPosition)
             .def("OrbitalPositionOnTurn",                   &Planet::OrbitalPositionOnTurn)
-            .add_property("RotationalPeriod",               &Planet::RotationalPeriod)
-            .add_property("LastTurnAttackedByShip",         &Planet::LastTurnAttackedByShip)
-            .add_property("LastTurnColonized",              &Planet::LastTurnColonized)
-            .add_property("LastTurnConquered",              &Planet::LastTurnConquered)
-            .add_property("buildingIDs",                    make_function(&Planet::BuildingIDs,     py::return_internal_reference<>()))
+            .add_property("lastTurnAttackedByShip",         &Planet::LastTurnAttackedByShip)
+            .add_property("lastTurnColonized",              &Planet::LastTurnColonized)
+            .add_property("lastTurnConquered",              &Planet::LastTurnConquered)
+            .add_property("ownerBeforeLastConquered",       &Planet::OwnerBeforeLastConquered)
+            .add_property("lastInvadedByEmpire",            &Planet::LastInvadedByEmpire)
+            .add_property("lastColonizedByEmpire",          &Planet::LastColonizedByEmpire)
+            .add_property("buildingIDs",                    +[](const Planet& planet) { return ToVec(planet.BuildingIDs()); })
             .add_property("habitableSize",                  &Planet::HabitableSize)
         ;
 
@@ -747,16 +747,15 @@ namespace FreeOrionPython {
         //////////////////
         py::class_<System, py::bases<UniverseObject>, boost::noncopyable>("system", py::no_init)
             .add_property("starType",           &System::GetStarType)
-            .add_property("numStarlanes",       &System::NumStarlanes)
-            .add_property("numWormholes",       &System::NumWormholes, "Currently unused.")
-            .def("HasStarlaneToSystemID",       &System::HasStarlaneTo)
-            .def("HasWormholeToSystemID",       &System::HasWormholeTo, "Currently unused.")
-            .add_property("starlanesWormholes", make_function(&System::StarlanesWormholes,  py::return_value_policy<py::return_by_value>()), "Currently unused.")
-            .add_property("planetIDs",          make_function(&System::PlanetIDs,           py::return_value_policy<py::return_by_value>()))
-            .add_property("buildingIDs",        make_function(&System::BuildingIDs,         py::return_value_policy<py::return_by_value>()))
-            .add_property("fleetIDs",           make_function(&System::FleetIDs,            py::return_value_policy<py::return_by_value>()))
-            .add_property("shipIDs",            make_function(&System::ShipIDs,             py::return_value_policy<py::return_by_value>()))
-            .add_property("fieldIDs",           make_function(&System::FieldIDs,            py::return_value_policy<py::return_by_value>()))
+            .add_property("numStarlanes",       &System::NumStarlanes, "Number of starlanes connecting to this sytsem")
+            .def("HasStarlaneToSystemID",       &System::HasStarlaneTo, "true if the passed in ID (int) is the ID of a system this system has a starlane connection with")
+            .add_property("starlanesWormholes", +[](const System& system) { return ToVec(system.Starlanes()); }, "[deprecated] use starlanes")
+            .add_property("starlanes",          +[](const System& system) { return ToVec(system.Starlanes()); }, "IDs of systems to which this system has starlane connections")
+            .add_property("planetIDs",          +[](const System& system) { return ToVec(system.PlanetIDs()); }, "IDs of planets in this system")
+            .add_property("buildingIDs",        +[](const System& system) { return ToVec(system.BuildingIDs()); }, "IDs of buildings in this system")
+            .add_property("fleetIDs",           +[](const System& system) { return ToVec(system.FleetIDs()); }, "IDs of fleets in this system")
+            .add_property("shipIDs",            +[](const System& system) { return ToVec(system.ShipIDs()); }, "IDs of ships in this system")
+            .add_property("fieldIDs",           +[](const System& system) { return ToVec(system.FieldIDs()); }, "IDs of fields in this system. Other fields may enclose this system but not be contained within this system.")
             .add_property("lastTurnBattleHere", &System::LastTurnBattleHere)
         ;
 
@@ -802,7 +801,7 @@ namespace FreeOrionPython {
         py::class_<Species, boost::noncopyable>("species", py::no_init)
             .add_property("name",               make_function(&Species::Name,           py::return_value_policy<py::copy_const_reference>()))
             .add_property("description",        make_function(&Species::Description,    py::return_value_policy<py::copy_const_reference>()))
-            .add_property("homeworlds",         &SpeciesHomeworlds)
+            .add_property("homeworlds",         &SpeciesHomeworlds) // TODO: SpeciesManager::SpeciesShipsDestroyed, GetSpeciesEmpireOpinionsMap, GetSpeciesSpeciesOpinionsMap
             .add_property("foci",               &SpeciesFoci)
             .add_property("preferredFocus",     make_function(&Species::DefaultFocus,   py::return_value_policy<py::copy_const_reference>()))
             .add_property("canColonize",        make_function(&Species::CanColonize,    py::return_value_policy<py::return_by_value>()))
