@@ -55,15 +55,15 @@ namespace {
 
         } else {
             // Consider the first player the main player
-            const PlayerSaveGameData* player = &(*player_save_game_data.begin());
+            const PlayerSaveGameData* player = &player_save_game_data.front();
 
             // If there are human players, the first of them should be the main player
             int16_t humans = 0;
             for (const PlayerSaveGameData& psgd : player_save_game_data) {
                 if (psgd.client_type == Networking::ClientType::CLIENT_TYPE_HUMAN_PLAYER) {
                     if (player->client_type != Networking::ClientType::CLIENT_TYPE_HUMAN_PLAYER &&
-                       player->client_type != Networking::ClientType::CLIENT_TYPE_HUMAN_OBSERVER &&
-                       player->client_type != Networking::ClientType::CLIENT_TYPE_HUMAN_MODERATOR)
+                        player->client_type != Networking::ClientType::CLIENT_TYPE_HUMAN_OBSERVER &&
+                        player->client_type != Networking::ClientType::CLIENT_TYPE_HUMAN_MODERATOR)
                     {
                         player = &psgd;
                     }
@@ -104,7 +104,7 @@ std::map<int, SaveGameEmpireData> CompileSaveGameEmpireData(const EmpireManager&
 }
 
 namespace {
-    constexpr std::size_t Pow(std::size_t base, std::size_t exp) {
+    consteval std::size_t Pow(std::size_t base, std::size_t exp) noexcept {
         std::size_t retval = 1;
         while (exp--)
             retval *= base;
@@ -372,7 +372,7 @@ void LoadGame(const std::string& filename, ServerSaveGameData& server_save_game_
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
 
         std::string signature(5, '\0');
-        if (!ifs.read(&signature[0], 5))
+        if (!ifs.read(signature.data(), 5))
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
         boost::iostreams::seek(ifs, 0, std::ios_base::beg);
 
@@ -496,7 +496,7 @@ void LoadGalaxySetupData(const std::string& filename, GalaxySetupData& galaxy_se
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
 
         std::string signature(5, '\0');
-        if (!ifs.read(&signature[0], 5))
+        if (!ifs.read(signature.data(), 5))
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
         boost::iostreams::seek(ifs, 0, std::ios_base::beg);
 
@@ -539,7 +539,7 @@ void LoadPlayerSaveHeaderData(const std::string& filename, std::vector<PlayerSav
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
 
         std::string signature(5, '\0');
-        if (!ifs.read(&signature[0], 5))
+        if (!ifs.read(signature.data(), 5))
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
         boost::iostreams::seek(ifs, 0, std::ios_base::beg);
 
@@ -591,7 +591,7 @@ void LoadEmpireSaveGameData(const std::string& filename,
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
 
         std::string signature(5, '\0');
-        if (!ifs.read(&signature[0], 5))
+        if (!ifs.read(signature.data(), 5))
             throw std::runtime_error(UNABLE_TO_OPEN_FILE);
         boost::iostreams::seek(ifs, 0, std::ios_base::beg);
 
