@@ -3,6 +3,7 @@
 
 #include <set>
 #include <vector>
+#include "../util/boost_fix.h"
 #include <boost/circular_buffer.hpp>
 #include <boost/asio/high_resolution_timer.hpp>
 #include "ServerFramework.h"
@@ -22,7 +23,7 @@ struct SaveGameUIData;
 struct ServerFSM;
 
 /** the application framework class for the FreeOrion server. */
-class ServerApp : public IApp {
+class ServerApp final : public IApp {
 public:
     ServerApp();
     ServerApp(const ServerApp&) = delete;
@@ -33,28 +34,24 @@ public:
     ServerApp& operator=(IApp&&) = delete;
 
     /** Returns a ClientApp pointer to the singleton instance of the app. */
-    [[nodiscard]] static ServerApp* GetApp();
+    [[nodiscard]] static ServerApp* GetApp() noexcept { return static_cast<ServerApp*>(s_app); }
     [[nodiscard]] Universe& GetUniverse() noexcept override { return m_universe; }
-    [[nodiscard]] EmpireManager& Empires() override { return m_empires; }
+    [[nodiscard]] EmpireManager& Empires() noexcept override { return m_empires; }
     [[nodiscard]] Empire* GetEmpire(int id) override;
-    [[nodiscard]] SupplyManager& GetSupplyManager() override { return m_supply_manager; }
-    [[nodiscard]] SpeciesManager& GetSpeciesManager() override { return m_species_manager; }
-    [[nodiscard]] const Species* GetSpecies(std::string_view name) override;
-
-    /** Returns the server's map for known objects of specified empire. */
-    [[nodiscard]] ObjectMap& EmpireKnownObjects(int empire_id) override;
+    [[nodiscard]] SupplyManager& GetSupplyManager() noexcept override { return m_supply_manager; }
+    [[nodiscard]] SpeciesManager& GetSpeciesManager() noexcept override { return m_species_manager; }
 
     [[nodiscard]] std::string GetVisibleObjectName(const UniverseObject& object) override;
 
-    [[nodiscard]] int EmpireID() const override { return ALL_EMPIRES; }
-    [[nodiscard]] int CurrentTurn() const override { return m_current_turn; }
+    [[nodiscard]] int EmpireID() const noexcept override { return ALL_EMPIRES; }
+    [[nodiscard]] int CurrentTurn() const noexcept override { return m_current_turn; }
 
     [[nodiscard]] int SelectedSystemID() const override { throw std::runtime_error{"Server cannot access selected object ID"}; }
     [[nodiscard]] int SelectedPlanetID() const override { throw std::runtime_error{"Server cannot access selected object ID"}; }
     [[nodiscard]] int SelectedFleetID() const override { throw std::runtime_error{"Server cannot access selected object ID"}; }
     [[nodiscard]] int SelectedShipID() const override { throw std::runtime_error{"Server cannot access selected object ID"}; }
 
-    [[nodiscard]] const GalaxySetupData& GetGalaxySetupData() const override { return m_galaxy_setup_data; }
+    [[nodiscard]] const GalaxySetupData& GetGalaxySetupData() const noexcept override { return m_galaxy_setup_data; }
 
     /** Checks if player with ID \a player_id is a human player
         who's client runs on the same machine as the server */
@@ -87,7 +84,7 @@ public:
     /** Extracts player save game data. */
     [[nodiscard]] std::vector<PlayerSaveGameData> GetPlayerSaveGameData() const;
 
-    [[nodiscard]] bool IsTurnExpired() const;
+    [[nodiscard]] bool IsTurnExpired() const noexcept { return m_turn_expired; }
 
     [[nodiscard]] bool IsHaveWinner() const;
 
@@ -113,7 +110,7 @@ public:
     void RevokeEmpireTurnReadyness(int empire_id);
 
     /** Sets all empire turn orders to an empty set. */
-    void ClearEmpireTurnOrders();
+    void ClearEmpireTurnOrders(int empire_id = ALL_EMPIRES);
 
     /** Determines if all empired have submitted their orders for this turn It
       * will loop the turn squence vector and check for a set order_set. A
@@ -210,10 +207,8 @@ public:
     /** Loads chat history via python script. */
     void LoadChatHistory();
 
-    void PushChatMessage(const std::string& text,
-                         const std::string& player_name,
-                         std::array<uint8_t, 4> text_color,
-                         const boost::posix_time::ptime& timestamp);
+    void PushChatMessage(std::string text, std::string player_name, std::array<uint8_t, 4> text_color,
+                         boost::posix_time::ptime timestamp);
 
     [[nodiscard]] ServerNetworking& Networking() noexcept { return m_networking; };
 
@@ -310,16 +305,21 @@ private:
     /** Called when asyncio timer ends. Executes Python asyncio callbacks if any was generated. */
     void AsyncIOTimedoutHandler(const boost::system::error_code& error);
 
+    /** Called when new \a turn state received by player playing \a empire_id. */
+    void UpdateEmpireTurnReceived(bool success, int empire_id, int turn);
+
     boost::asio::io_context m_io_context;
     boost::asio::signal_set m_signals;
     boost::asio::high_resolution_timer m_timer;
 
-    Universe                m_universe;
-    EmpireManager           m_empires;
-    SpeciesManager          m_species_manager;
-    SupplyManager           m_supply_manager;
-    ServerNetworking        m_networking;
-    ServerFSM*              m_fsm;
+    Universe         m_universe;
+    EmpireManager    m_empires;
+    SpeciesManager   m_species_manager;
+    SupplyManager    m_supply_manager;
+    ServerNetworking m_networking;
+
+    std::unique_ptr<ServerFSM> m_fsm;
+
     PythonServer            m_python_server;
     std::map<int, int>      m_player_empire_ids;                ///< map from player id to empire id that the player controls.
     int                     m_current_turn = INVALID_GAME_TURN; ///< current turn number
@@ -338,6 +338,15 @@ private:
       * */
     std::map<int, std::unique_ptr<PlayerSaveGameData>> m_turn_sequence;
 
+    // storage for cached costs between pre- and post-combat update steps
+    void CacheCostsTimes(const ScriptingContext& context);
+    std::map<int, std::vector<std::pair<std::string_view, double>>> m_cached_empire_policy_adoption_costs;
+    std::map<int, std::vector<std::tuple<std::string_view, double, int>>> m_cached_empire_research_costs_times;
+    std::map<int, std::vector<std::tuple<std::string_view, int, float, int>>> m_cached_empire_production_costs_times;
+    std::map<int, std::vector<std::pair<int, double>>> m_cached_empire_annexation_costs;
+
+    std::map<int, std::vector<int>> m_empire_fleet_combat_initiation_vis_overrides;
+
     // Give FSM and its states direct access.  We are using the FSM code as a
     // control-flow mechanism; it is all notionally part of this class.
     friend struct ServerFSM;
