@@ -293,8 +293,8 @@ namespace {
         if (system1 == system2) return true; // system is always connected to itself
         if (0 == maxLaneJumps) return false; // no system is connected to any other system by less than 1 jump
 
-        if (!system_lanes.count(system1)) return false;     // start system not in lanes map
-        if (!system_lanes.count(system2)) return false;     // destination system not in lanes map
+        if (!system_lanes.contains(system1)) return false;     // start system not in lanes map
+        if (!system_lanes.contains(system2)) return false;     // destination system not in lanes map
 
         if (system_lanes.at(system1).empty()) return false; // no lanes out of start system
         if (system_lanes.at(system2).empty()) return false; // no lanes out of destination system
@@ -312,10 +312,9 @@ namespace {
         // without finding system2
         boost::container::flat_map<int, int> accessibleSystemsMap;
         accessibleSystemsMap.reserve(system_lanes.size());
-        for (auto& [sys, lanes] : system_lanes) {
-            (void)lanes;
+        for (const auto sys : system_lanes | range_keys)
             accessibleSystemsMap.emplace_hint(accessibleSystemsMap.end(), sys, -1);
-        }
+
         static constexpr int mapped_type_max = std::numeric_limits<decltype(accessibleSystemsMap)::mapped_type>::max();
         maxLaneJumps = std::min(maxLaneJumps, mapped_type_max);
 
@@ -414,7 +413,7 @@ namespace {
                     std::pair{cur_sys_id, dest1} : std::pair{dest1, cur_sys_id}};
 
                 // check if this lane has already been added to the set of lanes to remove
-                if (lanesToRemoveSet.count(lane1)) {
+                if (lanesToRemoveSet.contains(lane1)) {
                     ++laneSetIter1;
                     continue;
                 }
@@ -439,7 +438,7 @@ namespace {
 
                     // check if this lane has already been added to the
                     // set of lanes to remove
-                    if (lanesToRemoveSet.count(lane2)) {
+                    if (lanesToRemoveSet.contains(lane2)) {
                         ++laneSetIter2;
                         continue;
                     }
@@ -567,8 +566,8 @@ namespace {
         // Iterate through set of lanes to remove, and remove them in turn.
         // Lanes are sorted by length, so are processed longest-first
         for (const auto& [dist, lane] : lanes_to_remove) {
-            if (system_lanes[lane.first].count(lane.second) == 0 ||
-                system_lanes[lane.second].count(lane.first) == 0)
+            if (!system_lanes[lane.first].contains(lane.second) ||
+                !system_lanes[lane.second].contains(lane.first))
             { continue; }
 
             // remove lane
@@ -734,15 +733,10 @@ void SetActiveMetersToTargetMaxCurrentValues(ObjectMap& object_map) {
     // check for each pair of meter types.  if both exist, set active
     // meter current value equal to target meter current value.
     for (const auto& object : object_map.all()) {
-        TraceLogger(effects) << "  object: " << object->Name() << " (" << object->ID() << ")";
         for (auto& entry : AssociatedMeterTypes()) {
             if (Meter* meter = object->GetMeter(entry.first)) {
-                if (Meter* targetmax_meter = object->GetMeter(entry.second)) {
-                    TraceLogger(effects) << "    meter: " << entry.first
-                                         << "  before: " << meter->Current()
-                                         << "  set to: " << targetmax_meter->Current();
+                if (Meter* targetmax_meter = object->GetMeter(entry.second))
                     meter->SetCurrent(targetmax_meter->Current());
-                }
             }
         }
     }
