@@ -1,5 +1,12 @@
-from common.base_prod import TECH_COST_MULTIPLIER
-from common.priorities import TARGET_POPULATION_AFTER_SCALING_PRIORITY
+from macros.base_prod import TECH_COST_MULTIPLIER
+from macros.priorities import TARGET_POPULATION_AFTER_SCALING_PRIORITY
+
+try:
+    from focs._effects import EffectsGroup, HasSpecies, OwnedBy, SetTargetPopulation, Source, Target, Value
+    from focs._tech import *
+except ModuleNotFoundError:
+    pass
+
 
 Tech(
     name="CON_ORBITAL_HAB",
