@@ -14,7 +14,9 @@
 #include "../util/AppInterface.h"
 #include "../util/CheckSums.h"
 #include "../util/GameRules.h"
+#include "../util/GameRuleRanks.h"
 #include "../util/i18n.h"
+#include <numeric>
 
 
 namespace {
@@ -22,11 +24,11 @@ namespace {
         // makes all ships cost 1 PP and take 1 turn to produce
         rules.Add<bool>(UserStringNop("RULE_CHEAP_AND_FAST_SHIP_PRODUCTION"),
                         UserStringNop("RULE_CHEAP_AND_FAST_SHIP_PRODUCTION_DESC"),
-                        "TEST", false, true);
+                        GameRuleCategories::GameRuleCategory::TEST, false, true,
+                        GameRuleRanks::RULE_CHEAP_AND_FAST_SHIP_PRODUCTION_RANK);
     }
     bool temp_bool = RegisterGameRules(&AddRules);
 
-    const std::string EMPTY_STRING;
     constexpr float ARBITRARY_LARGE_COST = 999999.9f;
 
     bool DesignsTheSame(const ShipDesign& one, const ShipDesign& two) {
@@ -141,9 +143,9 @@ const std::string& ShipDesign::Name(bool stringtable_lookup) const {
         return m_name;
 }
 
-void ShipDesign::SetName(const std::string& name) { // TODO: pass by value, make noexcept
+void ShipDesign::SetName(std::string name) noexcept {
     if (!name.empty() && !m_name.empty())
-        m_name = name; // TODO: pass by value with move
+        m_name = std::move(name);
 }
 
 const std::string& ShipDesign::Description(bool stringtable_lookup) const {
@@ -184,12 +186,9 @@ float ShipDesign::ProductionCost(int empire_id, int location_id, const Scripting
     if (const ShipHull* hull = GetShipHull(m_hull))
         cost_accumulator += hull->ProductionCost(empire_id, location_id, context, m_id);
 
-    int part_count = 0;
     for (const std::string& part_name : m_parts) {
-        if (const ShipPart* part = GetShipPart(part_name)) {
+        if (const ShipPart* part = GetShipPart(part_name))
             cost_accumulator += part->ProductionCost(empire_id, location_id, context, m_id);
-            part_count++;
-        }
     }
 
     // Assuming no reasonable combination of parts and hull will add up to more
@@ -334,10 +333,8 @@ std::vector<std::string> ShipDesign::Weapons() const {
 }
 
 int ShipDesign::PartCount() const {
-    int count = 0;
-    for (auto& entry : m_num_part_classes)
-         count += entry.second;
-    return count;
+    auto rng = m_num_part_classes | range_values;
+    return std::accumulate(rng.begin(), rng.end(), 0);
 }
 
 bool ShipDesign::ProductionLocation(int empire_id, int location_id, const ScriptingContext& context) const {
@@ -371,9 +368,6 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id, const Script
     // ships can only be produced by species that are not planetbound
     if (!species->CanProduceShips())
         return false;
-    // also, species that can't colonize can't produce colony ships
-    if (this->CanColonize() && !species->CanColonize())
-        return false;
 
     // apply hull location conditions to potential location
     const ShipHull* hull = GetShipHull(m_hull);
@@ -383,7 +377,7 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id, const Script
     }
     // evaluate using location as the source, as it should be an object owned by this empire.
     const ScriptingContext location_as_source_context{location, context};
-    if (!hull->Location()->Eval(location_as_source_context, location))
+    if (!hull->Location()->EvalOne(location_as_source_context, location))
         return false;
 
     // apply external and internal parts' location conditions to potential location
@@ -396,7 +390,7 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id, const Script
             ErrorLogger() << "ShipDesign::ProductionLocation  ShipDesign couldn't get part with name " << part_name;
             return false;
         }
-        if (!part->Location()->Eval(location_as_source_context, location))
+        if (!part->Location()->EvalOne(location_as_source_context, location))
             return false;
     }
     // location matched all hull and part conditions, so is a valid build location
@@ -406,10 +400,8 @@ bool ShipDesign::ProductionLocation(int empire_id, int location_id, const Script
 void ShipDesign::SetID(int id)
 { m_id = id; }
 
-bool ShipDesign::ValidDesign(const std::string& hull, const std::vector<std::string>& parts_in) {
-    auto parts = parts_in;
-    return !MaybeInvalidDesign(hull, parts, true);
-}
+bool ShipDesign::ValidDesign(const std::string& hull, const std::vector<std::string>& parts_in)
+{ return !MaybeInvalidDesign(hull, std::vector<std::string>(parts_in), true); }
 
 boost::optional<std::pair<std::string, std::vector<std::string>>>
 ShipDesign::MaybeInvalidDesign(std::string hull, std::vector<std::string> parts, bool produce_log)
@@ -434,7 +426,7 @@ ShipDesign::MaybeInvalidDesign(std::string hull, std::vector<std::string> parts,
                 ErrorLogger() << "Invalid ShipDesign no available hulls ";
             hull.clear();
             parts.clear();
-            return std::make_pair(std::move(hull), std::move(parts));
+            return std::pair(std::move(hull), std::move(parts));
         }
     }
 
@@ -481,7 +473,7 @@ ShipDesign::MaybeInvalidDesign(std::string hull, std::vector<std::string> parts,
     for (auto& part_name : parts) {
         if (part_name.empty())
             continue;
-        if (hull_exclusions.count(part_name)) {
+        if (std::count(hull_exclusions.begin(), hull_exclusions.end(), part_name)) {
             is_valid = false;
             if (produce_log)
                 WarnLogger() << "Invalid ShipDesign part \"" << part_name << "\" is excluded by hull \""
@@ -555,7 +547,7 @@ ShipDesign::MaybeInvalidDesign(std::string hull, std::vector<std::string> parts,
     if (is_valid) // if valid, return none to indicate no modifications needed
         return boost::none;
     else
-        return std::make_pair(std::move(hull), std::move(parts)); // return modified design
+        return std::pair(std::move(hull), std::move(parts)); // return modified design
 }
 
 void ShipDesign::ForceValidDesignOrThrow(const boost::optional<std::invalid_argument>& should_throw,
@@ -701,10 +693,8 @@ void ShipDesign::BuildStatCaches() {
     auto last = std::unique(tags.begin(), tags.end());
 
     // compile concatenated tags into contiguous storage
-    // TODO: transform_reduce when available on all platforms...
-    std::size_t tags_sz = 0;
-    std::for_each(tags.begin(), last, [&tags_sz](auto str) { tags_sz += str.size(); });
-
+    std::size_t tags_sz = std::transform_reduce(tags.begin(), tags.end(), 0u, std::plus{},
+                                                [](const auto& tag) { return tag.size(); });
     m_tags_concatenated.reserve(tags_sz);
     m_tags.clear();
     m_tags.reserve(tags.size());
@@ -729,7 +719,7 @@ std::string ShipDesign::Dump(uint8_t ntabs) const {
     if (m_parts.empty()) {
         retval += "[]\n";
     } else if (m_parts.size() == 1) {
-        retval += "\"" + *m_parts.begin() + "\"\n";
+        retval += "\"" + m_parts.front() + "\"\n";
     } else {
         retval += "[\n";
         for (const std::string& part_name : m_parts) {
@@ -770,10 +760,10 @@ bool operator ==(const ShipDesign& first, const ShipDesign& second) {
 
     // don't care if order is different, as long as the types and numbers of parts is the same
     for (const std::string& part_name : first.Parts())
-    { ++first_parts[part_name]; }
+        ++first_parts[part_name];
 
     for (const std::string& part_name : second.Parts())
-    { ++second_parts[part_name]; }
+        ++second_parts[part_name];
 
     return first_parts == second_parts;
 }
@@ -782,16 +772,6 @@ bool operator ==(const ShipDesign& first, const ShipDesign& second) {
 /////////////////////////////////////
 // PredefinedShipDesignManager     //
 /////////////////////////////////////
-PredefinedShipDesignManager* PredefinedShipDesignManager::s_instance = nullptr;
-
-PredefinedShipDesignManager::PredefinedShipDesignManager() {
-    if (s_instance)
-        throw std::runtime_error("Attempted to create more than one PredefinedShipDesignManager.");
-
-    // Only update the global pointer on sucessful construction.
-    s_instance = this;
-}
-
 namespace {
     void AddDesignToUniverse(Universe& universe, std::unordered_map<std::string, int>& design_generic_ids,
                              const std::unique_ptr<ShipDesign>& design, bool monster)
@@ -800,35 +780,24 @@ namespace {
             return;
 
         /* check if there already exists this same design in the universe. */
-        for (auto [existing_id, existing_design] : universe.ShipDesigns()) {
-            (void)existing_id;
-            if (!existing_design) {
-                ErrorLogger() << "PredefinedShipDesignManager::AddShipDesignsToUniverse found an invalid design in the Universe";
-                continue;
-            }
-
-            if (DesignsTheSame(*existing_design, *design)) {
+        for (const auto& [existing_id, existing_design] : universe.ShipDesigns()) {
+            if (DesignsTheSame(existing_design, *design)) {
                 WarnLogger() << "AddShipDesignsToUniverse found an exact duplicate of ship design "
                              << design->Name() << "to be added, so is not re-adding it";
-                design_generic_ids[design->Name(false)] = existing_design->ID();
+                design_generic_ids[design->Name(false)] = existing_id;
                 return; // design already added; don't need to do so again
             }
         }
 
         // duplicate design to add to Universe
-        ShipDesign* copy = new ShipDesign(*design);
-
-        bool success = universe.InsertShipDesign(copy);
-        if (!success) {
+        const auto new_design_id = universe.InsertShipDesign(*design);
+        if (new_design_id == INVALID_DESIGN_ID) {
             ErrorLogger() << "Empire::AddShipDesign Unable to add new design to universe";
-            delete copy;
             return;
         }
 
-        auto new_design_id = copy->ID();
         design_generic_ids[design->Name(false)] = new_design_id;
-        TraceLogger() << "AddShipDesignsToUniverse added ship design "
-                      << design->Name() << " to universe.";
+        TraceLogger() << "AddShipDesignsToUniverse added ship design " << design->Name() << " to universe.";
     };
 }
 
@@ -868,7 +837,7 @@ std::vector<const ShipDesign*> PredefinedShipDesignManager::GetOrderedMonsterDes
 
 int PredefinedShipDesignManager::GetDesignID(const std::string& name) const {
     CheckPendingDesignsTypes();
-    const auto& it = m_design_generic_ids.find(name);
+    const auto it = m_design_generic_ids.find(name);
     if (it == m_design_generic_ids.end())
         return INVALID_DESIGN_ID;
     return it->second;
@@ -882,7 +851,7 @@ uint32_t PredefinedShipDesignManager::GetCheckSum() const {
         for (auto const& uuid : ordering) {
             auto it = m_designs.find(uuid);
             if (it != m_designs.end())
-                CheckSums::CheckSumCombine(retval, std::make_pair(it->second->Name(false), *it->second));
+                CheckSums::CheckSumCombine(retval, std::pair(it->second->Name(false), *it->second));
         }
         CheckSums::CheckSumCombine(retval, ordering.size());
     };
@@ -904,10 +873,9 @@ void PredefinedShipDesignManager::SetMonsterDesignTypes(
 { m_pending_monsters = std::move(pending_designs); }
 
 namespace {
-    template <typename Map1, typename Map2, typename Ordering>
     void FillDesignsOrderingAndNameTables(
         PredefinedShipDesignManager::ParsedShipDesignsType& parsed_designs,
-        Map1& designs, Ordering& ordering, Map2& name_to_uuid)
+        auto& designs, auto& ordering, auto& name_to_uuid)
     {
         // Remove the old designs
         for (const auto& name_and_uuid: name_to_uuid)
@@ -924,14 +892,14 @@ namespace {
         for (auto& uuid_and_design : disk_designs) {
             auto& design = uuid_and_design.second.first;
 
-            if (designs.count(design->UUID())) {
+            if (designs.contains(design->UUID())) {
                 ErrorLogger() << design->Name() << " ship design does not have a unique UUID for "
                               << "its type monster or pre-defined. "
                               << designs[design->UUID()]->Name() << " has the same UUID.";
                 continue;
             }
 
-            if (name_to_uuid.count(design->Name())) {
+            if (name_to_uuid.contains(design->Name())) {
                 ErrorLogger() << design->Name() << " ship design does not have a unique name for "
                               << "its type monster or pre-defined.";
                 continue;
@@ -943,9 +911,8 @@ namespace {
         }
     }
 
-    template <typename PendingShips, typename Map1, typename Map2, typename Ordering>
     void CheckPendingAndFillDesignsOrderingAndNameTables(
-        PendingShips& pending, Map1& designs, Ordering& ordering, Map2& name_to_uuid, bool are_monsters)
+        auto& pending, auto& designs, auto& ordering, auto& name_to_uuid, bool are_monsters)
     {
         if (!pending)
             return;
@@ -957,8 +924,7 @@ namespace {
         DebugLogger() << "Populating pre-defined ships with "
                       << std::string(are_monsters ? "monster" : "ship") << " designs.";
 
-        FillDesignsOrderingAndNameTables(
-            *parsed, designs, ordering, name_to_uuid);
+        FillDesignsOrderingAndNameTables(*parsed, designs, ordering, name_to_uuid);
 
         // Make the monsters monstrous
         if (are_monsters)
@@ -1025,11 +991,11 @@ LoadShipDesignsAndManifestOrderFromParseResults(
                           << design->UUID() << " for name " << design->Name();
         }
 
-        if (!saved_designs.count(design->UUID())) {
+        if (!saved_designs.contains(design->UUID())) {
             TraceLogger() << "Added saved design UUID " << design->UUID()
                           << " with name " << design->Name();
             auto uuid = design->UUID();
-            saved_designs.emplace(std::move(uuid), std::make_pair(std::move(design), design_and_path.second));
+            saved_designs.emplace(std::move(uuid), std::pair(std::move(design), design_and_path.second));
         } else {
             WarnLogger() << "Duplicate ship design UUID " << design->UUID()
                          << " found for ship design " << design->Name()
@@ -1037,44 +1003,44 @@ LoadShipDesignsAndManifestOrderFromParseResults(
         }
     }
 
+    static constexpr auto not_nil = [](const boost::uuids::uuid uuid) noexcept { return !uuid.is_nil(); };
+
     // Verify that all UUIDs in ordering exist
     std::vector<boost::uuids::uuid> ordering;
     ordering.reserve(disk_ordering.size());
     bool ship_manifest_inconsistent = false;
-    for (auto& uuid : disk_ordering) {
-        // Skip the nil UUID.
-        if (uuid.is_nil())
-            continue;
-
-        if (!saved_designs.count(uuid)) {
-            WarnLogger() << "UUID " << uuid << " is in ship design manifest for "
-                         << "a ship design that does not exist.";
+    for (const auto uuid : disk_ordering | range_filter(not_nil)) { // Skip the nil UUID.
+        if (saved_designs.contains(uuid)) {
+            ordering.push_back(uuid);
+        } else {
+            WarnLogger() << "UUID " << uuid << " is in ship design manifest for a ship design that does not exist.";
             ship_manifest_inconsistent = true;
-            continue;
         }
-        ordering.push_back(uuid);
     }
 
     // Verify that every design in saved_designs is in ordering.
     if (ordering.size() != saved_designs.size()) {
         // Add any missing designs in alphabetical order to the end of the list
-        std::unordered_set<boost::uuids::uuid, boost::hash<boost::uuids::uuid>>
-            uuids_in_ordering{ordering.begin(), ordering.end()};
-        std::map<std::string, boost::uuids::uuid> missing_uuids_sorted_by_name;
-        for (auto& uuid_to_design_and_filename : saved_designs) {
-            if (uuids_in_ordering.count(uuid_to_design_and_filename.first))
+        std::vector<std::pair<std::string_view, boost::uuids::uuid>> names_and_missing_uuids;
+        names_and_missing_uuids.reserve(saved_designs.size());
+
+        const auto in_ordering = [&ordering](const auto& uuid) {
+            return std::any_of(ordering.begin(), ordering.end(),
+                               [uuid](const auto uuid_in_order) noexcept { return uuid == uuid_in_order; });
+        };
+
+        for (auto& [uuid, design_and_filename] : saved_designs) {
+            if (!in_ordering(uuid)) // using range_filter above may cause an internal compiler error in MSVC
                 continue;
             ship_manifest_inconsistent = true;
-            missing_uuids_sorted_by_name.emplace(
-                uuid_to_design_and_filename.second.first->Name(),
-                uuid_to_design_and_filename.first);
+            names_and_missing_uuids.emplace_back(design_and_filename.first->Name(), uuid);
         }
+        std::sort(names_and_missing_uuids.begin(), names_and_missing_uuids.end(),
+                  [](const auto& lhs, const auto& rhs) noexcept { return lhs.first < rhs.first; });
 
-        for (auto& name_and_uuid: missing_uuids_sorted_by_name) {
-            WarnLogger() << "Missing ship design " << name_and_uuid.second
-                         << " called " << name_and_uuid.first
-                         << " added to the manifest.";
-            ordering.push_back(name_and_uuid.second);
+        for (auto& [name, uuid] : names_and_missing_uuids) {
+            WarnLogger() << "Missing ship design " << uuid << " called " << name << " added to the manifest.";
+            ordering.push_back(uuid);
         }
     }
 
