@@ -12,15 +12,17 @@
 #include "../util/Logger.h"
 #include "../util/OptionsDB.h"
 #include "../util/GameRules.h"
+#include "../util/GameRuleRanks.h"
 #include "../util/i18n.h"
 
 
 namespace {
     void AddRules(GameRules& rules) {
-        // makes all PRNG be reseeded frequently
+        // whether systems detected only at basic visibility have their name and star type revealed or not
         rules.Add<bool>(UserStringNop("RULE_BASIC_VIS_SYSTEM_INFO_SHOWN"),
                         UserStringNop("RULE_BASIC_VIS_SYSTEM_INFO_SHOWN_DESC"),
-                        "", false, true);
+                        GameRuleCategories::GameRuleCategory::GENERAL, false, true,
+                        GameRuleRanks::RULE_BASIC_VIS_SYSTEM_INFO_SHOWN_RANK);
     }
     bool temp_bool = RegisterGameRules(&AddRules);
 }
@@ -42,100 +44,105 @@ System::System(StarType star, std::string name, double x, double y, int current_
     UniverseObject::Init();
 }
 
-System* System::Clone(const Universe& universe, int empire_id) const {
-    Visibility vis = universe.GetObjectVisibilityByEmpire(this->ID(), empire_id);
+std::shared_ptr<UniverseObject> System::Clone(const Universe& universe, int empire_id) const {
+    const Visibility vis = empire_id == ALL_EMPIRES ?
+        Visibility::VIS_FULL_VISIBILITY : universe.GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     if (!(vis >= Visibility::VIS_BASIC_VISIBILITY && vis <= Visibility::VIS_FULL_VISIBILITY))
         return nullptr;
 
-    auto retval = std::make_unique<System>();
-    retval->Copy(shared_from_this(), universe, empire_id);
-    return retval.release();
+    auto retval = std::make_shared<System>();
+    retval->Copy(*this, universe, empire_id);
+    return retval;
 }
 
-void System::Copy(std::shared_ptr<const UniverseObject> copied_object,
-                  const Universe& universe, int empire_id)
-{
-    if (copied_object.get() == this)
+void System::Copy(const UniverseObject& copied_object, const Universe& universe, int empire_id) {
+    if (&copied_object == this)
         return;
-    auto copied_system = std::dynamic_pointer_cast<const System>(copied_object);
-    if (!copied_system) {
+    if (copied_object.ObjectType() != UniverseObjectType::OBJ_SYSTEM) {
         ErrorLogger() << "System::Copy passed an object that wasn't a System";
         return;
     }
 
-    int copied_object_id = copied_object->ID();
-    Visibility vis = universe.GetObjectVisibilityByEmpire(copied_object_id, empire_id);
-    auto visible_specials = universe.GetObjectVisibleSpecialsByEmpire(copied_object_id, empire_id);
+    Copy(static_cast<const System&>(copied_object), universe, empire_id);
+}
 
-    UniverseObject::Copy(std::move(copied_object), vis, visible_specials, universe);
+void System::Copy(const System& copied_system, const Universe& universe, int empire_id) {
+    if (&copied_system == this)
+        return;
+
+    const int copied_object_id = copied_system.ID();
+    const Visibility vis = empire_id == ALL_EMPIRES ?
+        Visibility::VIS_FULL_VISIBILITY : universe.GetObjectVisibilityByEmpire(copied_object_id, empire_id);
+    const auto visible_specials = universe.GetObjectVisibleSpecialsByEmpire(copied_object_id, empire_id);
+
+    UniverseObject::Copy(copied_system, vis, visible_specials, universe);
 
     if (vis >= Visibility::VIS_BASIC_VISIBILITY) {
-        if (GetGameRules().Get<bool>("RULE_BASIC_VIS_SYSTEM_INFO_SHOWN")) {
-            this->m_name = copied_system->m_name;
-            this->m_star = copied_system->m_star;
-        }
-
         // copy visible info of visible contained objects
-        this->m_objects = copied_system->VisibleContainedObjectIDs(empire_id, universe.GetEmpireObjectVisibility());
+        this->m_objects = copied_system.VisibleContainedObjectIDs(
+            empire_id, universe.GetEmpireObjectVisibility());
 
         // only copy orbit info for visible planets
         auto orbits_size = m_orbits.size();
         m_orbits.clear();
         m_orbits.assign(orbits_size, INVALID_OBJECT_ID);
-        for (std::size_t o = 0u; o < copied_system->m_orbits.size(); ++o) {
-            int planet_id = copied_system->m_orbits[o];
-            if (m_objects.count(planet_id))
+        for (std::size_t o = 0u; o < copied_system.m_orbits.size(); ++o) {
+            int planet_id = copied_system.m_orbits[o];
+            if (m_objects.contains(planet_id))
                 m_orbits[o] = planet_id;
         }
 
         // copy visible contained object per-type info
         m_planets.clear();
-        for (int planet_id : copied_system->m_planets) {
-            if (m_objects.count(planet_id))
+        for (int planet_id : copied_system.m_planets) {
+            if (m_objects.contains(planet_id))
                 m_planets.insert(planet_id);
         }
 
         m_buildings.clear();
-        for (int building_id : copied_system->m_buildings) {
-            if (m_objects.count(building_id))
+        for (int building_id : copied_system.m_buildings) {
+            if (m_objects.contains(building_id))
                 m_buildings.insert(building_id);
         }
 
         m_fleets.clear();
-        for (int fleet_id : copied_system->m_fleets) {
-            if (m_objects.count(fleet_id))
+        for (int fleet_id : copied_system.m_fleets) {
+            if (m_objects.contains(fleet_id))
                 m_fleets.insert(fleet_id);
         }
 
         m_ships.clear();
-        for (int ship_id : copied_system->m_ships) {
-            if (m_objects.count(ship_id))
+        for (int ship_id : copied_system.m_ships) {
+            if (m_objects.contains(ship_id))
                 m_ships.insert(ship_id);
         }
 
         m_fields.clear();
-        for (int field_id : copied_system->m_fields) {
-            if (m_objects.count(field_id))
+        for (int field_id : copied_system.m_fields) {
+            if (m_objects.contains(field_id))
                 m_fields.insert(field_id);
         }
 
 
         if (vis >= Visibility::VIS_PARTIAL_VISIBILITY) {
-            this->m_name =                  copied_system->m_name;
-            this->m_star =                  copied_system->m_star;
-            this->m_last_turn_battle_here = copied_system->m_last_turn_battle_here;
+            this->m_name =                  copied_system.m_name;
+            this->m_star =                  copied_system.m_star;
+            this->m_last_turn_battle_here = copied_system.m_last_turn_battle_here;
 
             // update lanes to be just those that are visible, erasing any
             // previously known that aren't visible now, as these are thus
             // known not to exist any more
-            this->m_starlanes_wormholes = copied_system->VisibleStarlanesWormholes(empire_id, universe);
+            this->m_starlanes = copied_system.VisibleStarlanes(empire_id, universe);
 
         } else {
+            if (GetGameRules().Get<bool>("RULE_BASIC_VIS_SYSTEM_INFO_SHOWN")) {
+                this->m_name = copied_system.m_name;
+                this->m_star = copied_system.m_star;
+            }
+
             // add any visible lanes, without removing existing entries
-            for (const auto& [lane_id, lane_or_hole] :
-                 copied_system->VisibleStarlanesWormholes(empire_id, universe))
-            { this->m_starlanes_wormholes[lane_id] = lane_or_hole; }
+            this->m_starlanes.merge(copied_system.VisibleStarlanes(empire_id, universe));
         }
     }
 }
@@ -162,10 +169,10 @@ std::string System::Dump(uint8_t ntabs) const {
     }
 
     retval.append("  starlanes: ");
-    for (auto it = m_starlanes_wormholes.begin(); it != m_starlanes_wormholes.end();) {
-        int lane_end_id = it->first;
+    for (auto it = m_starlanes.begin(); it != m_starlanes.end();) {
+        const int lane_end_id = *it;
         ++it;
-        retval.append(std::to_string(lane_end_id)).append(it == m_starlanes_wormholes.end() ? "" : ", ");
+        retval.append(std::to_string(lane_end_id)).append(it == m_starlanes.end() ? "" : ", ");
     }
 
     retval.append("  objects: ");
@@ -178,10 +185,13 @@ std::string System::Dump(uint8_t ntabs) const {
     return retval;
 }
 
-std::string System::ApparentName(int empire_id, const Universe& u,
-                                 bool blank_unexplored_and_none) const
-{
+std::string System::ApparentName(int empire_id, const Universe& u, bool blank_unexplored_and_none) const {
+    // this one line requires a higher __GNUC__ version to compile for several Docker / Fedora test builds. No idea why it's different from the other similar cases.
+#if defined(__cpp_lib_constexpr_string) && (!defined(__GNUC__) || (__GNUC__ > 12) || (__GNUC__ == 12 && __GNU_MINOR__ >= 2)) && ((!defined(_MSC_VER) || (_MSC_VER >= 1934))) && ((!defined(__clang_major__) || (__clang_major__ >= 10)))
+    static constexpr std::string EMPTY_STRING;
+#else
     static const std::string EMPTY_STRING;
+#endif
 
     const ObjectMap& o = u.Objects();
 
@@ -190,7 +200,7 @@ std::string System::ApparentName(int empire_id, const Universe& u,
 
     // has the indicated empire ever detected this system?
     const auto& vtm = u.GetObjectVisibilityTurnMapByEmpire(this->ID(), empire_id);
-    if (!vtm.count(Visibility::VIS_PARTIAL_VISIBILITY)) {
+    if (!vtm.contains(Visibility::VIS_PARTIAL_VISIBILITY)) {
         if (blank_unexplored_and_none)
             return EMPTY_STRING;
 
@@ -213,10 +223,10 @@ std::string System::ApparentName(int empire_id, const Universe& u,
         return m_name + UserString("EMPTY_SPACE");
     }
 
-    return this->PublicName(empire_id, u); // todo get Objects from inputs
+    return this->PublicName(empire_id, u);
 }
 
-StarType System::NextOlderStarType() const {
+StarType System::NextOlderStarType() const noexcept {
     if (m_star <= StarType::INVALID_STAR_TYPE || m_star >= StarType::NUM_STAR_TYPES)
         return StarType::INVALID_STAR_TYPE;
     if (m_star >= StarType::STAR_RED)
@@ -224,7 +234,7 @@ StarType System::NextOlderStarType() const {
     return StarType(int(m_star) + 1);   // STAR_BLUE -> STAR_WHITE -> STAR_YELLOW -> STAR_ORANGE -> STAR_RED
 }
 
-StarType System::NextYoungerStarType() const {
+StarType System::NextYoungerStarType() const noexcept {
     if (m_star <= StarType::INVALID_STAR_TYPE || m_star >= StarType::NUM_STAR_TYPES)
         return StarType::INVALID_STAR_TYPE;
     if (m_star > StarType::STAR_RED)
@@ -234,54 +244,39 @@ StarType System::NextYoungerStarType() const {
     return StarType(int(m_star) - 1);   // STAR_BLUE <- STAR_WHITE <- STAR_YELLOW <- STAR_ORANGE <- STAR_RED
 }
 
-int System::NumStarlanes() const {
-    return std::count_if(m_starlanes_wormholes.begin(), m_starlanes_wormholes.end(),
-                         [](const auto id_whflag) { return !id_whflag.second; });
-}
-
-int System::NumWormholes() const {
-    return std::count_if(m_starlanes_wormholes.begin(), m_starlanes_wormholes.end(),
-                         [](const auto id_whflag) { return id_whflag.second; });
-}
-
-bool System::HasStarlaneTo(int id) const {
-    auto it = m_starlanes_wormholes.find(id);
-    return (it == m_starlanes_wormholes.end() ? false : it->second == false);
-}
-
-bool System::HasWormholeTo(int id) const {
-    auto it = m_starlanes_wormholes.find(id);
-    return (it == m_starlanes_wormholes.end() ? false : it->second == true);
-}
-
-int System::EffectiveOwner(const ObjectMap& objects) const {
-    // Check if all of the owners are the same empire.
-    int first_owner_found = ALL_EMPIRES;
-    for (const auto* planet : objects.findRaw<Planet>(m_planets)) {
-        const int owner = planet->Owner();
-        if (owner == ALL_EMPIRES)
-            continue;
-        if (first_owner_found == ALL_EMPIRES)
-            first_owner_found = owner;
-        if (first_owner_found != owner)
-            return ALL_EMPIRES;
-    }
-    return first_owner_found;
-}
+bool System::HasStarlaneTo(int id) const
+{ return m_starlanes.contains(id); }
 
 bool System::Contains(int object_id) const {
     if (object_id == INVALID_OBJECT_ID)
         return false;
-    return m_objects.count(object_id);
+    return m_objects.contains(object_id);
 }
 
 std::shared_ptr<UniverseObject> System::Accept(const UniverseObjectVisitor& visitor) const
 { return visitor.Visit(std::const_pointer_cast<System>(std::static_pointer_cast<const System>(shared_from_this()))); }
 
-void System::Insert(std::shared_ptr<UniverseObject> obj, int orbit, int current_turn)
-{ Insert(obj.get(), orbit, current_turn); }
+std::size_t System::SizeInMemory() const {
+    std::size_t retval = UniverseObject::SizeInMemory();
+    retval += sizeof(System) - sizeof(UniverseObject);
 
-void System::Insert(UniverseObject* obj, int orbit, int current_turn) {
+    retval += sizeof(decltype(m_orbits)::value_type)*m_orbits.capacity();
+    retval += sizeof(decltype(m_objects)::value_type)*m_objects.capacity();
+    retval += sizeof(decltype(m_planets)::value_type)*m_planets.capacity();
+    retval += sizeof(decltype(m_buildings)::value_type)*m_buildings.capacity();
+    retval += sizeof(decltype(m_fleets)::value_type)*m_fleets.capacity();
+    retval += sizeof(decltype(m_ships)::value_type)*m_ships.capacity();
+    retval += sizeof(decltype(m_fields)::value_type)*m_fields.capacity();
+    retval += sizeof(decltype(m_starlanes)::value_type)*m_starlanes.capacity();
+    retval += sizeof(decltype(m_overlay_texture)::value_type)*m_overlay_texture.capacity();
+
+    return retval;
+}
+
+void System::Insert(std::shared_ptr<UniverseObject> obj, int orbit, int current_turn, const ObjectMap& objects)
+{ Insert(obj.get(), orbit, current_turn, objects); }
+
+void System::Insert(UniverseObject* obj, int orbit, int current_turn, const ObjectMap& objects) {
     if (!obj) {
         ErrorLogger() << "System::Insert() : Attempted to place a null object in a System";
         return;
@@ -352,12 +347,12 @@ void System::Insert(UniverseObject* obj, int orbit, int current_turn) {
     case UniverseObjectType::OBJ_SHIP: {
         m_ships.insert(obj->ID());
         if (auto ship = static_cast<Ship*>(obj))
-            ship->SetArrivedOnTurn(CurrentTurn());
+            ship->SetArrivedOnTurn(current_turn);
         break;
     }
     case UniverseObjectType::OBJ_FLEET: {
         m_fleets.insert(obj->ID());
-        FleetsInsertedSignal({static_cast<Fleet*>(obj)});
+        FleetsInsertedSignal({obj->ID()}, objects);
         break;
     }
     case UniverseObjectType::OBJ_PLANET:
@@ -405,10 +400,8 @@ void System::Remove(int id) {
     m_buildings.erase(id);
     m_objects.erase(id);
 
-    if (removed_fleet) {
-        if (auto fleet = Objects().getRaw<Fleet>(id))
-            FleetsRemovedSignal({fleet});
-    }
+    if (removed_fleet)
+        FleetsRemovedSignal(std::vector<int>{id});
     StateChangedSignal();
 }
 
@@ -420,44 +413,24 @@ void System::SetStarType(StarType type) {
 }
 
 void System::AddStarlane(int id) {
-    if (!HasStarlaneTo(id) && id != this->ID()) {
-        m_starlanes_wormholes[id] = false;
+    const auto added = m_starlanes.insert(id).second;
+    if (added) {
         StateChangedSignal();
         TraceLogger() << "Added starlane from system " << this->Name() << " (" << this->ID() << ") system " << id;
     }
 }
 
-void System::AddWormhole(int id) {
-    if (!HasWormholeTo(id) && id != this->ID()) {
-        m_starlanes_wormholes[id] = true;
-        StateChangedSignal();
-    }
-}
-
 bool System::RemoveStarlane(int id) {
-    bool retval = false;
-    if ((retval = HasStarlaneTo(id))) {
-        m_starlanes_wormholes.erase(id);
+    const auto erased_count = m_starlanes.erase(id);
+    if (erased_count > 0)
         StateChangedSignal();
-    }
-    return retval;
+    return erased_count > 0;
 }
 
-bool System::RemoveWormhole(int id) {
-    bool retval = false;
-    if ((retval = HasWormholeTo(id))) {
-        m_starlanes_wormholes.erase(id);
-        StateChangedSignal();
-    }
-    return retval;
-}
-
-void System::SetLastTurnBattleHere(int turn)
-{ m_last_turn_battle_here = turn; }
-
 void System::ResetTargetMaxUnpairedMeters() {
     UniverseObject::ResetTargetMaxUnpairedMeters();
 
+    static_assert(noexcept(GetMeter(MeterType::METER_STEALTH)->ResetCurrent()));
     // give systems base stealth slightly above zero, so that they can't be
     // seen from a distance without high detection ability
     if (Meter* stealth = GetMeter(MeterType::METER_STEALTH)) {
@@ -487,7 +460,7 @@ int System::OrbitOfPlanet(int object_id) const {
     return NO_ORBIT;
 }
 
-std::set<int> System::FreeOrbits() const {
+std::set<int> System::FreeOrbits() const { // TODO: return something better
     std::set<int> retval;
     for (int o = first_orbit; o < static_cast<int>(m_orbits.size()); ++o)
         if (m_orbits[o] == INVALID_OBJECT_ID)
@@ -495,12 +468,12 @@ std::set<int> System::FreeOrbits() const {
     return retval;
 }
 
-std::map<int, bool> System::VisibleStarlanesWormholes(int empire_id, const Universe& universe) const {
+System::IDSet System::VisibleStarlanes(int empire_id, const Universe& universe) const {
     if (empire_id == ALL_EMPIRES)
-        return m_starlanes_wormholes;
+        return m_starlanes;
 
     const ObjectMap& objects = universe.Objects();
-    Visibility this_system_vis = universe.GetObjectVisibilityByEmpire(this->ID(), empire_id);
+    const Visibility this_system_vis = universe.GetObjectVisibilityByEmpire(this->ID(), empire_id);
 
     //visible starlanes are:
     //  - those connected to systems with vis >= partial
@@ -509,23 +482,21 @@ std::map<int, bool> System::VisibleStarlanesWormholes(int empire_id, const Unive
 
     // return all starlanes if partially visible or better
     if (this_system_vis >= Visibility::VIS_PARTIAL_VISIBILITY)
-        return m_starlanes_wormholes;
+        return m_starlanes;
 
 
     // compile visible lanes connected to this only basically-visible system
-    std::map<int, bool> retval;
-
-
-    // check if any of the adjacent systems are partial or better visible
-    for (const auto& [lane_end_sys_id, is_wormhole] : m_starlanes_wormholes) {
-        if (universe.GetObjectVisibilityByEmpire(lane_end_sys_id, empire_id) >=
-            Visibility::VIS_PARTIAL_VISIBILITY)
-        { retval[lane_end_sys_id] = is_wormhole; }
-    }
-
+    // by checking if any of the adjacent systems are partial or better visible
+    System::IDSet retval;
+    retval.reserve(m_starlanes.size());
+    std::copy_if(m_starlanes.begin(), m_starlanes.end(), std::inserter(retval, retval.end()),
+                 [&universe, empire_id](auto lane_end_sys_id) {
+                     return universe.GetObjectVisibilityByEmpire(lane_end_sys_id, empire_id) >=
+                        Visibility::VIS_PARTIAL_VISIBILITY;
+                  });
 
     // early exit check... can't see any more lanes than exist, so don't need to check for more if all lanes are already visible
-    if (retval == m_starlanes_wormholes)
+    if (retval == m_starlanes)
         return retval;
 
 
@@ -559,12 +530,10 @@ std::map<int, bool> System::VisibleStarlanesWormholes(int empire_id, const Unive
             other_lane_end_sys_id = prev_sys_id;
 
         if (other_lane_end_sys_id != INVALID_OBJECT_ID) {
-            auto lane_it = m_starlanes_wormholes.find(other_lane_end_sys_id);
-            if (lane_it == m_starlanes_wormholes.end()) {
+            if (m_starlanes.contains(other_lane_end_sys_id))
+                retval.insert(other_lane_end_sys_id);
+            else
                 ErrorLogger() << "System::VisibleStarlanesWormholes found an owned fleet moving along a starlane connected to this system that isn't also connected to one of this system's starlane-connected systems...?";
-                continue;
-            }
-            retval[other_lane_end_sys_id] = lane_it->second;
         }
     }
 
