@@ -6,15 +6,14 @@
 #include "OptionValidators.h"
 
 #include <boost/any.hpp>
+#include <boost/container/flat_set.hpp>
 #include <boost/filesystem/fstream.hpp>
 #include <boost/signals2/signal.hpp>
 
 #include <functional>
 #include <map>
-#include <unordered_set>
 #include <unordered_map>
 
-
 class OptionsDB;
 class XMLDoc;
 class XMLElement;
@@ -110,13 +109,68 @@ template<typename T> constexpr bool is_unique_ptr_v = is_unique_ptr<T>::value;
 class FO_COMMON_API OptionsDB {
 public:
     /** emitted when an option has changed */
-    typedef boost::signals2::signal<void ()> OptionChangedSignalType;
+    using OptionChangedSignalType = boost::signals2::signal<void ()>;
 
-    /** indicates whether an option with name \a name has been added to this
-        OptionsDB. */
+    struct FO_COMMON_API Option {
+        Option(char short_name_, std::string name_, boost::any value_,
+               boost::any default_value_, std::string description_,
+               std::unique_ptr<ValidatorBase>&& validator_, bool storable_, bool flag_,
+               bool recognized_, std::string section = std::string());
+        Option(Option&& rhs) = default;
+        Option(const Option& rhs) = delete;
+        virtual ~Option();
+        Option& operator=(Option&& rhs) = default;
+        Option& operator=(const Option& rhs) = delete;
+
+        // non-defaulted comparisons to only consider names
+        bool operator<(const Option& rhs) const noexcept { return name < rhs.name; };
+        bool operator==(const Option& rhs) const noexcept { return name == rhs.name; };
+
+        // SetFromValue returns true if this->value is successfully changed
+        template <typename T>
+        bool SetFromValue(T&& value_);
+
+        // SetFromString returns true if this->value is successfully changed
+        bool SetFromString(std::string_view str);
+
+        // SetToDefault returns true if this->value is successfully changed
+        bool SetToDefault();
+
+        [[nodiscard]] std::string ValueToString() const;
+        [[nodiscard]] std::string DefaultValueToString() const;
+        [[nodiscard]] bool        ValueIsDefault() const;
+
+        using StringSet = boost::container::flat_set<std::string>;
+
+        std::string     name;               ///< the name of the option
+        boost::any      value;              ///< the value of the option
+        boost::any      default_value;      ///< the default value of the option
+        std::string     description;        ///< a desription of the option
+        StringSet       sections;           ///< sections this option should display under
+
+        // A validator for the option.  Flags have no validators; lexical_cast boolean conversions are done for them.
+        std::unique_ptr<ValidatorBase> validator;
+        std::unique_ptr<OptionChangedSignalType> option_changed_sig; // pointer so that references are valid even if this Option is moved
+
+        char            short_name{0};      ///< the one character abbreviation of the option
+        bool            storable = false;   ///< whether this option can be stored in an XML config file for use across multiple runs
+        bool            flag = false;
+        bool            recognized = false; ///< whether this option has been registered before being specified via an XML input, unrecognized options can't be parsed (don't know their type) but are stored in case they are later registered with Add()
+    };
+
+private:
+    using ContainerType = std::vector<Option>;
+
+    auto find_option(const std::string_view name) const
+    { return std::find_if(m_options.begin(), m_options.end(), [name](const auto& o) { return o.name == name; }); }
+    auto find_option(const std::string_view name)
+    { return std::find_if(m_options.begin(), m_options.end(), [name](const auto& o) { return o.name == name; }); }
+
+public:
+    /** Has an option with name \a name been added to this OptionsDB? */
     bool OptionExists(std::string_view name) const {
-        auto it = m_options.find(name);
-        return it != m_options.end() && it->second.recognized;
+        auto it = find_option(name);
+        return it != m_options.end() && it->recognized;
     }
 
     /** write the optionDB's non-default state to the XML config file. */
@@ -134,24 +188,40 @@ public:
     void Validate(std::string_view name, std::string_view value) const;
 
 
-
     /** returns the value of option \a name. Note that the exact type of item
       * stored in the option \a name must be known in advance.  This means that
       * Get() must be called as Get<int>("foo"), etc. */
     template <typename T>
     T Get(std::string_view name) const
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         if (!OptionExists(it))
             throw std::runtime_error(std::string{"OptionsDB::Get<>() : Attempted to get nonexistent option \""}.append(name).append("\"."));
         try {
-            return boost::any_cast<T>(it->second.value);
+            return boost::any_cast<T>(it->value);
         } catch (const boost::bad_any_cast&) {
-            ErrorLogger() << "bad any cast converting value option named: " << name << ". Returning default value instead";
+            if constexpr (std::is_enum_v<T>) {
+                try {
+                    return T{boost::any_cast<std::underlying_type_t<T>>(it->value)};
+                } catch (...) {}
+            }
+            ErrorLogger() << "bad any cast converting value option named: " << name
+                          << ". Returning default value instead";
+
             try {
-                return boost::any_cast<T>(it->second.default_value);
+                return boost::any_cast<T>(it->default_value);
             } catch (const boost::bad_any_cast&) {
-                ErrorLogger() << "bad any cast converting default value of option named: " << name << ". Returning data-type default value instead: " << T();
+                if constexpr (std::is_enum_v<T>) {
+                    try {
+                        return T{boost::any_cast<std::underlying_type_t<T>>(it->default_value)};
+                    } catch (...) {
+                        ErrorLogger() << "bad any cast converting default value of option named: " << name
+                                      << ". Returning data-type default value instead.";
+                    }
+                } else {
+                    ErrorLogger() << "bad any cast converting default value of option named: " << name
+                                  << ". Returning data-type default value instead: " << T();
+                }
                 return T();
             }
         }
@@ -163,19 +233,24 @@ public:
     template <typename T>
     T GetDefault(std::string_view name) const
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         if (!OptionExists(it))
             throw std::runtime_error(std::string{"OptionsDB::GetDefault<>() : Attempted to get nonexistent option: "}.append(name));
         try {
-            return boost::any_cast<T>(it->second.default_value);
+            return boost::any_cast<T>(it->default_value);
         } catch (const boost::bad_any_cast&) {
+            if constexpr (std::is_enum_v<T>) {
+                try {
+                    return T{boost::any_cast<std::underlying_type_t<T>>(it->default_value)};
+                } catch (...) {}
+            }
             ErrorLogger() << "bad any cast converting default value of option named: " << name << "  returning type default value instead";
             return T();
         }
     }
 
     bool IsDefaultValue(std::string_view name) const {
-        auto it = m_options.find(name);
+        const auto it = find_option(name);
         if (!OptionExists(it))
             throw std::runtime_error(std::string{"OptionsDB::IsDefaultValue<>() : Attempted to get nonexistent option: "}.append(name));
         return IsDefaultValue(it);
@@ -210,10 +285,7 @@ public:
 
     /** find all registered Options that begin with \a prefix and store them in
       * \a ret. If \p allow_unrecognized then include unrecognized options. */
-    void FindOptions(std::set<std::string>& ret, std::string_view prefix,
-                     bool allow_unrecognized = false) const;
-    std::vector<std::string_view> FindOptions(std::string_view prefix,
-                                              bool allow_unrecognized = false) const;
+    std::vector<std::string_view> FindOptions(std::string_view prefix, bool allow_unrecognized = false) const;
 
     /** the option changed signal object for the given option */
     OptionChangedSignalType& OptionChangedSignal(std::string_view option);
@@ -225,7 +297,7 @@ public:
              std::unique_ptr<ValidatorBase> validator = nullptr, bool storable = true,
              std::string section = "")
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         boost::any value = default_value;
         if (!validator)
             validator = std::make_unique<Validator<std::decay_t<T>>>();
@@ -233,21 +305,21 @@ public:
         // Check that this option hasn't already been registered and apply any
         // value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
-            if (it->second.recognized)
+            if (it->recognized)
                 throw std::runtime_error("OptionsDB::Add<>() : Option registered twice: " + name);
 
             // SetFrom[...]() sets "flag" to true for unrecognised options if they look like flags
             // (i.e. no parameter is found for the option)
-            if (it->second.flag) {
+            if (it->flag) {
                 ErrorLogger() << "OptionsDB::Add<>() : Option " << name << " was specified on the command line or in a config file with no value, using default value.";
             } else {
                 try {
                     // This option was previously specified externally but was not recognized at the time.
                     // Attempt to parse the value found there, overriding the default value passed in
-                    value = validator->Validate(it->second.ValueToString());
+                    value = validator->Validate(it->ValueToString());
                 } catch (const boost::bad_lexical_cast&) {
                     ErrorLogger() << "OptionsDB::Add<>() : Option " << name
-                                  << " was given the value \"" << it->second.ValueToString()
+                                  << " was given the value \"" << it->ValueToString()
                                   << "\" from the command line or a config file but that value couldn't be converted to the correct type, using default value instead.";
                 }
             }
@@ -256,13 +328,15 @@ public:
         Option option{static_cast<char>(0), name, std::move(value), std::forward<T>(default_value),
                       std::move(description), std::move(validator), storable, false, true,
                       std::move(section)};
-        it = m_options.insert_or_assign(std::move(name), std::move(option)).first;
+        if (it != m_options.end())
+            *it = std::move(option);
+        else
+            m_options.push_back(std::move(option));
+
         m_dirty = true;
     }
 
-    template <typename T, typename V,
-              typename std::enable_if_t<!is_unique_ptr_v<V>>* = nullptr,
-              typename std::enable_if_t<!std::is_null_pointer_v<V>>* = nullptr>
+    template <typename T, typename V> requires(!is_unique_ptr_v<V> && !std::is_null_pointer_v<V>)
     void Add(std::string name, std::string description, T&& default_value,
              V&& validator, // validator needs to be wrapped in unique_ptr (eg. by cloning itself)
              bool storable = true, std::string section = "")
@@ -271,12 +345,10 @@ public:
             std::forward<V>(validator).Clone(), storable, std::move(section));
     }
 
-    template <typename T, typename V,
-        typename std::enable_if_t<!is_unique_ptr_v<V>>* = nullptr,
-        typename std::enable_if_t<!std::is_null_pointer_v<V>>* = nullptr>
-        void Add(const char* name, const char* description, T&& default_value,
-                 V&& validator, // validator needs to be wrapped in unique_ptr (eg. by cloning itself)
-                 bool storable = true, const char* section = "")
+    template <typename T, typename V> requires(!is_unique_ptr_v<V> && !std::is_null_pointer_v<V>)
+    void Add(const char* name, const char* description, T&& default_value,
+             V&& validator, // validator needs to be wrapped in unique_ptr (eg. by cloning itself)
+             bool storable = true, const char* section = "")
     {
         Add(name, description, std::forward<T>(default_value),
             std::forward<V>(validator).Clone(), storable, section);
@@ -289,7 +361,7 @@ public:
              std::unique_ptr<ValidatorBase> validator = nullptr, bool storable = true,
              std::string section = "")
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         boost::any value{default_value};
         if (!validator)
             validator = std::make_unique<Validator<std::decay_t<T>>>();
@@ -297,19 +369,19 @@ public:
         // Check that this option hasn't already been registered and apply any
         // value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
-            if (it->second.recognized)
+            if (it->recognized)
                 throw std::runtime_error("OptionsDB::Add<>() : Option registered twice: " + name);
 
             // SetFrom[...]() sets "flag" to true for unrecognised options if they look like flags
             // (i.e. no parameter is found for the option)
-            if (it->second.flag) {
+            if (it->flag) {
                 ErrorLogger() << "OptionsDB::Add<>() : Option " << name
                               << " was specified on the command line or in a config file with no value, using default value.";
             } else {
                 try {
                     // This option was previously specified externally but was not recognized at the time,
                     // attempt to parse the value found there
-                    value = validator->Validate(it->second.ValueToString());
+                    value = validator->Validate(it->ValueToString());
                 } catch (const boost::bad_lexical_cast&) {
                     ErrorLogger() << "OptionsDB::Add<>() : Option " << name
                                   << " was given the value from the command line or a config file that cannot be converted to the correct type. Using default value instead.";
@@ -320,21 +392,21 @@ public:
         Option option{short_name, name, std::move(value), std::forward<T>(default_value),
                       std::move(description), std::move(validator), storable, false, true,
                       std::move(section)};
-        m_options.insert_or_assign(std::move(name), std::move(option));
+        if (it != m_options.end())
+            *it = std::move(option);
+        else
+            m_options.push_back(std::move(option));
+
         m_dirty = true;
     }
 
-    template <typename T, typename V,
-              typename std::enable_if_t<!is_unique_ptr_v<V>>* = nullptr,
-              typename std::enable_if_t<!std::is_null_pointer_v<V>>* = nullptr>
+    template <typename T, typename V> requires(!is_unique_ptr_v<V> && !std::is_null_pointer_v<V>)
     void Add(char short_name, std::string name, std::string description,
              T default_value, V&& validator, // validator should be wrapped in unique_ptr
              bool storable = true, std::string section = "")
     {
-        Add<T>(short_name, std::move(name), std::move(description),
-               std::move(default_value),
-               std::make_unique<V>(std::move(validator)),
-               storable, std::move(section));
+        Add<T>(short_name, std::move(name), std::move(description), std::move(default_value),
+               std::make_unique<V>(std::move(validator)), storable, std::move(section));
     }
 
     /** adds a flag Option, which is treated as a boolean value with a default
@@ -343,27 +415,30 @@ public:
     void AddFlag(std::string name, std::string description,
                  bool storable = true, std::string section = std::string())
     {
-        auto it = m_options.find(name);
+        const auto it = find_option(name);
         bool value = false;
-        auto validator = std::make_unique<Validator<bool>>();
 
         // Check that this option hasn't already been registered and apply any value
         // that was specified on the command line or from a config file.
         if (it != m_options.end()) {
-            if (it->second.recognized)
+            if (it->recognized)
                 throw std::runtime_error("OptionsDB::AddFlag<>() : Option registered twice: " + name);
 
             // SetFrom[...]() sets "flag" to false on unrecognised options if they don't look like flags
             // (flags have no parameter on the command line or have an empty tag in XML)
-            if (!it->second.flag)
+            if (!it->flag)
                 ErrorLogger() << "OptionsDB::AddFlag<>() : Option " << name << " was specified with the value \""
-                              << it->second.ValueToString() << "\", but flags should not have values assigned to them.";
+                              << it->ValueToString() << "\", but flags should not have values assigned to them.";
             value = true; // if the flag is present at all its value is true
         }
 
-        Option&& option{static_cast<char>(0), name, value, value, std::move(description),
-                        std::move(validator), storable, true, true, std::move(section)};
-        m_options.insert_or_assign(name, std::move(option));
+        Option option{static_cast<char>(0), name, value, value, std::move(description),
+                      std::make_unique<Validator<bool>>(), storable, true, true, std::move(section)};
+        if (it != m_options.end())
+            *it = std::move(option);
+        else
+            m_options.push_back(std::move(option));
+
         m_dirty = true;
     }
 
@@ -373,27 +448,31 @@ public:
     void AddFlag(char short_name, std::string name, std::string description,
                  bool storable = true, std::string section = std::string())
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         bool value = false;
         auto validator = std::make_unique<Validator<bool>>();
 
         // Check that this option hasn't already been registered and apply any value
         // that was specified on the command line or from a config file.
         if (it != m_options.end()) {
-            if (it->second.recognized)
+            if (it->recognized)
                 throw std::runtime_error("OptionsDB::AddFlag<>() : Option registered twice: " + name);
 
             // SetFrom[...]() sets "flag" to false on unrecognised options if they don't look like flags
             // (flags have no parameter on the command line or have an empty tag in XML)
-            if (!it->second.flag)
+            if (!it->flag)
                 ErrorLogger() << "OptionsDB::AddFlag<>() : Option " << name << " was specified with the value \""
-                              << it->second.ValueToString() << "\", but flags should not have values assigned to them.";
+                              << it->ValueToString() << "\", but flags should not have values assigned to them.";
             value = true; // if the flag is present at all its value is true
         }
 
-        Option&& option{short_name, name, value, value, std::move(description),
-                        std::move(validator), storable, true, true, std::move(section)};
-        m_options.insert_or_assign(name, std::move(option));
+        Option option{short_name, name, value, value, std::move(description),
+                      std::move(validator), storable, true, true, std::move(section)};
+        if (it != m_options.end())
+            *it = std::move(option);
+        else
+            m_options.push_back(std::move(option));
+
         m_dirty = true;
     }
 
@@ -408,21 +487,21 @@ public:
     template <typename T>
     void Set(std::string_view name, T&& value)
     {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::Set<>() : Attempted to set nonexistent option " + std::string{name});
-        m_dirty |= it->second.SetFromValue(std::forward<T>(value));
+        m_dirty |= it->SetFromValue(std::forward<T>(value));
     }
 
     /** Set the default value of option @p name to @p value */
     template <typename T>
     void SetDefault(std::string_view name, T&& value) {
-        auto it = m_options.find(name);
+        auto it = find_option(name);
         if (!OptionExists(it))
             throw std::runtime_error("Attempted to set default value of nonexistent option \"" + std::string{name});
-        if (it->second.default_value.type() != typeid(std::decay_t<T>))
+        if (it->default_value.type() != typeid(std::decay_t<T>))
             throw boost::bad_any_cast();
-        it->second.default_value = std::forward<T>(value);
+        it->default_value = std::forward<T>(value);
     }
 
     void SetToDefault(std::string_view name);
@@ -440,53 +519,6 @@ public:
       * XMLDoc \a doc */
     void SetFromXML(const XMLDoc& doc);
 
-    struct FO_COMMON_API Option {
-        Option() = default;
-        Option(char short_name_, std::string name_, boost::any value_,
-               boost::any default_value_, std::string description_,
-               std::unique_ptr<ValidatorBase>&& validator_, bool storable_, bool flag_,
-               bool recognized_, std::string section = std::string());
-        Option(Option&& rhs) = default;
-        virtual ~Option();
-        Option& operator=(Option&& rhs) = default;
-
-        // SetFromValue returns true if this->value is successfully changed
-        template <typename T>
-        bool SetFromValue(T&& value_);
-
-        // SetFromString returns true if this->value is successfully changed
-        bool SetFromString(std::string_view str);
-
-        // SetToDefault returns true if this->value is successfully changed
-        bool SetToDefault();
-
-        [[nodiscard]] std::string ValueToString() const;
-        [[nodiscard]] std::string DefaultValueToString() const;
-        [[nodiscard]] bool        ValueIsDefault() const;
-
-        std::string     name;               ///< the name of the option
-        char            short_name{0};      ///< the one character abbreviation of the option
-        bool            storable = false;   ///< whether this option can be stored in an XML config file for use across multiple runs
-        bool            flag = false;
-        bool            recognized = false; ///< whether this option has been registered before being specified via an XML input, unrecognized options can't be parsed (don't know their type) but are stored in case they are later registered with Add()
-        boost::any      value;              ///< the value of the option
-        boost::any      default_value;      ///< the default value of the option
-        std::string     description;        ///< a desription of the option
-        std::unordered_set<std::string> sections; ///< sections this option should display under
-
-        /** A validator for the option.  Flags have no validators; lexical_cast
-            boolean conversions are done for them. */
-        std::unique_ptr<ValidatorBase> validator;
-
-        OptionChangedSignalType option_changed_sig;
-    };
-
-    struct FO_COMMON_API OptionSection {
-        std::string name;
-        std::string description;
-        std::function<bool (const std::string&)> option_predicate;
-    };
-
     /** Defines an option section with a description and optionally a option predicate.
      *  @param name Name of section, typically in the form of a left side subset of an option name.
      *  @param description Stringtable key used for local description
@@ -496,34 +528,51 @@ public:
                     std::function<bool (const std::string&)> option_predicate = nullptr);
 
 private:
-    /** indicates whether the option referenced by \a it has been added to this
-        OptionsDB.  Overloaded for convenient use within other OptionsDB
-        functions */
-    bool OptionExists(std::map<std::string, Option>::const_iterator it) const
-    { return it != m_options.end() && it->second.recognized; }
+    /** is option at \a it in OptionsDB? */
+    bool OptionExists(auto it) const
+        requires requires (ContainerType::const_iterator cit) { cit == it; }
+    { return it != m_options.end() && it->recognized; }
 
-    /** indicates whether the current value of the option references by \a is
-        the default value for that option */
-    bool IsDefaultValue(std::map<std::string, Option>::const_iterator it) const
-    { return it != m_options.end() && it->second.ValueToString() == it->second.DefaultValueToString(); }
+    /** is current value of option at \a it the default value for that option? */
+    bool IsDefaultValue(auto it) const
+        requires requires (ContainerType::const_iterator cit) { cit == it; }
+    { return it != m_options.end() && it->ValueToString() == it->DefaultValueToString(); }
 
     void SetFromXMLRecursive(const XMLElement& elem, std::string_view section_name);
 
     /** Determine known option sections and which options each contains
      *  A special "root" section is added for determined top-level sections */
-    std::unordered_map<std::string_view, std::set<std::string_view>> OptionsBySection(
-        bool allow_unrecognized = false) const;
+    std::unordered_map<std::string_view, std::set<std::string_view>>
+        OptionsBySection(bool allow_unrecognized = false) const;
 
-    std::map<std::string, Option, std::less<>>          m_options;
-    std::unordered_map<std::string_view, OptionSection> m_sections;
-    bool                                                m_dirty = false; //< has OptionsDB changed since last Commit()
+    struct OptionSection {
+        OptionSection() = default;
+        OptionSection(OptionSection&&) = default;
+        OptionSection(auto&& name_, auto&& desc_, auto&& pred_) :
+            name(std::forward<decltype(name_)>(name_)),
+            description(std::forward<decltype(desc_)>(desc_)),
+            option_predicate(std::forward<decltype(pred_)>(pred_))
+        {}
+        std::string name;
+        std::string description;
+        std::function<bool (const std::string&)> option_predicate;
+    };
+
+    ContainerType              m_options = [](){ ContainerType retval; retval.reserve(1000); return retval; }();
+    std::vector<OptionSection> m_sections = []() { std::vector<OptionSection> retval; retval.reserve(20); return retval; }();
+    bool                       m_dirty = false; //< has OptionsDB changed since last Commit()
 
     friend FO_COMMON_API OptionsDB& GetOptionsDB();
 };
 
 template <typename T>
 bool OptionsDB::Option::SetFromValue(T&& value_) {
-    if (value.type() != typeid(std::decay_t<T>)) {
+    if constexpr (!std::is_same_v<std::decay_t<decltype(value_)>, std::string> &&
+                  std::is_convertible_v<decltype(value_), std::string>)
+    {
+        return SetFromValue(std::string(value_));
+
+    } else if (value.type() != typeid(std::decay_t<T>)) {
         DebugLogger() << "OptionsDB::Option::SetFromValue expected type " << value.type().name()
                       << " but got value of type " << typeid(T).name();
     }
@@ -547,7 +596,7 @@ bool OptionsDB::Option::SetFromValue(T&& value_) {
 
     if (changed) {
         value = std::forward<T>(value_);
-        option_changed_sig();
+        (*option_changed_sig)();
     }
     return changed;
 }
