@@ -13,6 +13,7 @@
 #include "../Empire/Empire.h"
 #include "../Empire/EmpireManager.h"
 #include "../universe/Building.h"
+#include "../universe/Condition.h"
 #include "../universe/Fleet.h"
 #include "../universe/Pathfinder.h"
 #include "../universe/Planet.h"
@@ -50,7 +51,12 @@ bool Order::Undo(ScriptingContext& context) const {
 }
 
 namespace {
+#if defined(__cpp_lib_constexpr_string) && ((!defined(__GNUC__) || (__GNUC__ > 12) || (__GNUC__ == 12 && __GNUC_MINOR__ >= 2))) && ((!defined(_MSC_VER) || (_MSC_VER >= 1934))) && ((!defined(__clang_major__) || (__clang_major__ >= 17)))
+    constexpr std::string EMPTY_STRING;
+#else
     const std::string EMPTY_STRING;
+#endif
+
     const std::string& ExecutedTag(const Order* order) {
         if (order && !order->Executed())
             return UserString("ORDER_UNEXECUTED");
@@ -68,7 +74,7 @@ namespace {
 
         while (true) {
             const uint8_t c = *it;
-            std::ptrdiff_t sequence_length =
+            const std::ptrdiff_t sequence_length =
                 (c <= 0x7F) ? 1 :
                 (c >= 0xC2 && c <= 0xDF) ? 2 :
                 (c >= 0xE0 && c <= 0xEF) ? 3 :
@@ -380,7 +386,8 @@ void NewFleetOrder::ExecuteImpl(ScriptingContext& context) const {
 
     Universe& u = context.ContextUniverse();
     ObjectMap& o = context.ContextObjects();
-    const auto& empire_ids = context.EmpireIDs();
+    const auto& ids_as_flatset{context.EmpireIDs()};
+    const std::vector<int> empire_ids{ids_as_flatset.begin(), ids_as_flatset.end()};
 
     u.InhibitUniverseObjectSignals(true);
 
@@ -413,7 +420,7 @@ void NewFleetOrder::ExecuteImpl(ScriptingContext& context) const {
     // an ID is provided to ensure consistancy between server and client universes
     u.SetEmpireObjectVisibility(EmpireID(), fleet->ID(), Visibility::VIS_FULL_VISIBILITY);
 
-    system->Insert(fleet, System::NO_ORBIT, context.current_turn);
+    system->Insert(fleet, System::NO_ORBIT, context.current_turn, o);
 
     // new fleet will get same m_arrival_starlane as fleet of the first ship in the list.
     auto first_ship{validated_ships[0]};
@@ -442,8 +449,7 @@ void NewFleetOrder::ExecuteImpl(ScriptingContext& context) const {
 
     u.InhibitUniverseObjectSignals(false);
 
-    std::vector<const Fleet*> created_fleets{fleet.get()};
-    system->FleetsInsertedSignal(created_fleets);
+    system->FleetsInsertedSignal(std::vector<int>{fleet->ID()}, o);
     system->StateChangedSignal();
 
     // Signal changed state of modified fleets and remove any empty fleets.
@@ -505,7 +511,7 @@ FleetMoveOrder::FleetMoveOrder(int empire_id, int fleet_id, int dest_system_id,
 }
 
 std::string FleetMoveOrder::Dump() const
-{ return UserString("ORDER_FLEET_MOVE"); }
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_FLEET_MOVE")) % m_fleet % m_dest_system) + ExecutedTag(this); }
 
 bool FleetMoveOrder::Check(int empire_id, int fleet_id, int dest_system_id,
                            bool append, const ScriptingContext& context)
@@ -580,14 +586,14 @@ void FleetMoveOrder::ExecuteImpl(ScriptingContext& context) const {
     }
 
     // check destination validity: disallow movement that's out of range
-    auto eta = fleet->ETA(fleet->MovePath(fleet_travel_route, false, context));
+    const auto eta = fleet->ETA(fleet->MovePath(fleet_travel_route, false, context));
     if (eta.first == Fleet::ETA_NEVER || eta.first == Fleet::ETA_OUT_OF_RANGE) {
         DebugLogger() << "FleetMoveOrder::ExecuteImpl rejected out of range move order";
         return;
     }
 
     try {
-        fleet->SetRoute(fleet_travel_route, context.ContextObjects());
+        fleet->SetRoute(std::move(fleet_travel_route), context.ContextObjects());
         fleet->SetMoveOrderedTurn(context.current_turn);
         // todo: set last turn ordered moved
     } catch (const std::exception& e) {
@@ -608,8 +614,16 @@ FleetTransferOrder::FleetTransferOrder(int empire, int dest_fleet, std::vector<i
         ErrorLogger() << "FleetTransferOrder constructor found problem...";
 }
 
-std::string FleetTransferOrder::Dump() const
-{ return UserString("ORDER_FLEET_TRANSFER"); }
+std::string FleetTransferOrder::Dump() const {
+    std::string ships;
+    if (!m_add_ships.empty()) {
+        ships.reserve(15*m_add_ships.size()); // guesstimate
+        for (auto s : m_add_ships)
+            ships.append(std::to_string(s)).append(" ");
+        ships.resize(ships.length() - 1);
+    }
+    return boost::io::str(FlexibleFormat(UserString("ORDER_FLEET_TRANSFER")) % ships % m_dest_fleet) + ExecutedTag(this);
+}
 
 bool FleetTransferOrder::Check(int empire_id, int dest_fleet_id, const std::vector<int>& ship_ids,
                                const ScriptingContext& context)
@@ -724,7 +738,8 @@ void FleetTransferOrder::ExecuteImpl(ScriptingContext& context) const {
 
     // signal change to fleet states
     modified_fleets.insert(target_fleet.get());
-    const auto& empire_ids = context.EmpireIDs();
+    const auto& ids_as_flatset{context.EmpireIDs()};
+    const std::vector<int> empire_ids{ids_as_flatset.begin(), ids_as_flatset.end()};
 
     for (auto* modified_fleet : modified_fleets) {
         if (!modified_fleet) {
@@ -740,6 +755,88 @@ void FleetTransferOrder::ExecuteImpl(ScriptingContext& context) const {
     }
 }
 
+////////////////////////////////////////////////
+// AnnexOrder
+////////////////////////////////////////////////
+AnnexOrder::AnnexOrder(int empire, int planet, const ScriptingContext& context) :
+    Order(empire),
+    m_planet(planet)
+{ Check(empire, m_planet, context); }
+
+std::string AnnexOrder::Dump() const
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_ANNEX")) % m_planet) + ExecutedTag(this); }
+
+bool AnnexOrder::Check(int empire_id, int planet_id, const ScriptingContext& context) {
+    const ObjectMap& o = context.ContextObjects();
+
+    const auto* planet = o.getRaw<const Planet>(planet_id);
+    if (!planet) {
+        ErrorLogger() << "AnnexOrder couldn't get planet with id " << planet_id;
+        return false;
+    }
+
+    if (empire_id == ALL_EMPIRES) {
+        ErrorLogger() << "AnnexOrder given non-empire empire id: " << empire_id;
+        return false;
+    }
+
+    const auto& planet_species_name = planet->SpeciesName();
+    if (planet_species_name.empty()) {
+        ErrorLogger() << "AnnexOrder given planet without a species: " << planet_id;
+        return false;
+    }
+    const auto* planet_species = context.species.GetSpecies(planet_species_name);
+    if (!planet_species) {
+        ErrorLogger() << "AnnexOrder given planet with an unknown species: " << planet_species_name;
+        return false;
+    }
+    const auto* annexation_condition = planet_species->AnnexationCondition();
+    if (!annexation_condition) {
+        ErrorLogger() << "AnnexOrder given planet with species with no annexation condition: " << planet_species_name;
+        return false;
+    }
+
+    if (!context.source)
+        ErrorLogger() << "AnnexOrder given context with no source... Context source should be an object owned by the order issuing empire";
+
+    if (!context.source->OwnedBy(empire_id))
+        ErrorLogger() << "AnnexOrder given context with source not owned by passed in empire id";
+
+    if (!annexation_condition->EvalOne(context, planet)) {
+        ErrorLogger() << "AnnexOrder given planet that does not meet its species annexation condition: " << planet_species_name;
+        return false;
+    }
+
+    // TODO: check IP costs, like adopting policies
+
+    return true;
+}
+
+void AnnexOrder::ExecuteImpl(ScriptingContext& context) const {
+    GetValidatedEmpire(context);
+
+    if (!Check(EmpireID(), m_planet, context))
+        return;
+
+    ObjectMap& objects{context.ContextObjects()};
+    if (auto* planet = objects.getRaw<Planet>(m_planet))
+        planet->SetIsOrderAnnexedByEmpire(EmpireID());
+}
+
+bool AnnexOrder::UndoImpl(ScriptingContext& context) const {
+    ObjectMap& objects{context.ContextObjects()};
+
+    auto* planet = objects.getRaw<Planet>(m_planet);
+    if (!planet) {
+        ErrorLogger() << "AnnexOrder::UndoImpl couldn't get planet with id " << m_planet;
+        return false;
+    }
+
+    planet->ResetBeingAnnxed();
+
+    return true;
+}
+
 ////////////////////////////////////////////////
 // ColonizeOrder
 ////////////////////////////////////////////////
@@ -750,15 +847,18 @@ ColonizeOrder::ColonizeOrder(int empire, int ship, int planet, const ScriptingCo
 { Check(empire, m_ship, m_planet, context); }
 
 std::string ColonizeOrder::Dump() const
-{ return UserString("ORDER_COLONIZE"); }
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_COLONIZE")) % m_planet % m_ship) + ExecutedTag(this); }
 
-bool ColonizeOrder::Check(int empire_id, int ship_id, int planet_id,
-                          const ScriptingContext& context)
-{
+bool ColonizeOrder::Check(int empire_id, int ship_id, int planet_id, const ScriptingContext& context) {
     const Universe& u = context.ContextUniverse();
     const ObjectMap& o = context.ContextObjects();
     const SpeciesManager& sm = context.species;
 
+    if (empire_id == ALL_EMPIRES) {
+        ErrorLogger() << "ColonizeOrder::Check() : empire " << empire_id << " is not an empire";
+        return false;
+    }
+
     auto ship = o.get<Ship>(ship_id);
     if (!ship) {
         ErrorLogger() << "ColonizeOrder::Check() : empire " << empire_id
@@ -809,7 +909,7 @@ bool ColonizeOrder::Check(int empire_id, int ship_id, int planet_id,
         return false;
     }
     if (colonist_capacity > 0.0f &&
-        planet->EnvironmentForSpecies(context, ship->SpeciesName()) < PlanetEnvironment::PE_HOSTILE)
+        planet->EnvironmentForSpecies(context.species, ship->SpeciesName()) < PlanetEnvironment::PE_HOSTILE)
     {
         ErrorLogger() << "ColonizeOrder::Check() : nonzero colonist capacity, " << colonist_capacity
                       << ", and planet " << planet->Name() << " of type, " << planet->Type() << ", that ship's species, "
@@ -839,7 +939,11 @@ void ColonizeOrder::ExecuteImpl(ScriptingContext& context) const {
 
     ObjectMap& objects{context.ContextObjects()};
     auto ship = objects.get<Ship>(m_ship);
+    if (!ship)
+        return;
     auto planet = objects.get<Planet>(m_planet);
+    if (!planet)
+        return;
 
     planet->SetIsAboutToBeColonized(true);
     ship->SetColonizePlanet(m_planet);
@@ -890,76 +994,81 @@ InvadeOrder::InvadeOrder(int empire, int ship, int planet, const ScriptingContex
 { Check(empire, m_ship, m_planet, context); }
 
 std::string InvadeOrder::Dump() const
-{ return UserString("ORDER_INVADE"); }
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_INVADE")) % m_planet % m_ship) + ExecutedTag(this); }
 
 bool InvadeOrder::Check(int empire_id, int ship_id, int planet_id, const ScriptingContext& context) {
     const Universe& u = context.ContextUniverse();
     const ObjectMap& o = context.ContextObjects();
 
+    if (empire_id == ALL_EMPIRES) {
+        ErrorLogger() << "InvadeOrder::Check() : empire " << empire_id << " is not an empire";
+        return false;
+    }
+
     // make sure ship_id is a ship...
     auto ship = o.get<Ship>(ship_id);
     if (!ship) {
-        ErrorLogger() << "IssueInvadeOrder : passed an invalid ship_id";
+        ErrorLogger() << "IssueInvadeOrder: passed an invalid ship_id";
         return false;
     }
 
     if (!ship->OwnedBy(empire_id)) {
-        ErrorLogger() << "IssueInvadeOrder : empire does not own passed ship";
+        ErrorLogger() << "IssueInvadeOrder: empire does not own passed ship";
         return false;
     }
     if (!ship->HasTroops(u)) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl got ship that can't invade";
+        ErrorLogger() << "InvadeOrder got ship that can't invade";
         return false;
     }
 
     // get fleet of ship
     auto fleet = o.get<Fleet>(ship->FleetID());
     if (!fleet) {
-        ErrorLogger() << "IssueInvadeOrder : ship with passed ship_id has invalid fleet_id";
+        ErrorLogger() << "IssueInvadeOrder: ship with passed ship_id has invalid fleet_id";
         return false;
     }
 
     // make sure player owns ship and its fleet
     if (!fleet->OwnedBy(empire_id)) {
-        ErrorLogger() << "IssueInvadeOrder : empire does not own fleet of passed ship";
+        ErrorLogger() << "IssueInvadeOrder: empire does not own fleet of passed ship";
         return false;
     }
 
     auto planet = o.get<Planet>(planet_id);
     if (!planet) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl couldn't get planet with id " << planet_id;
+        ErrorLogger() << "InvadeOrder couldn't get planet with id " << planet_id;
         return false;
     }
 
     if (ship->SystemID() != planet->SystemID()) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given ids of ship and planet not in the same system";
+        ErrorLogger() << "InvadeOrder given ids of ship and planet not in the same system";
         return false;
     }
 
     if (u.GetObjectVisibilityByEmpire(planet_id, empire_id) < Visibility::VIS_BASIC_VISIBILITY) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given planet that empire reportedly has insufficient visibility of, but will be allowed to proceed pending investigation";
+        ErrorLogger() << "InvadeOrder given planet that empire reportedly has insufficient visibility of";
         return false;
     }
 
     if (planet->OwnedBy(empire_id)) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given planet that is already owned by the order-issuing empire";
+        ErrorLogger() << "InvadeOrder given planet that is already owned by the order-issuing empire";
         return false;
     }
 
     if (planet->Unowned() && planet->GetMeter(MeterType::METER_POPULATION)->Initial() == 0.0f) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given unpopulated planet";
+        ErrorLogger() << "InvadeOrder given unpopulated planet";
         return false;
     }
 
     if (planet->GetMeter(MeterType::METER_SHIELD)->Initial() > 0.0f) {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given planet with shield > 0";
+        ErrorLogger() << "InvadeOrder given planet with shield > 0";
         return false;
     }
 
     if (!planet->Unowned() && context.ContextDiploStatus(planet->Owner(), empire_id) !=
                               DiplomaticStatus::DIPLO_WAR)
     {
-        ErrorLogger() << "InvadeOrder::ExecuteImpl given planet owned by an empire not at war with order-issuing empire";
+        ErrorLogger() << "InvadeOrder given planet owned by an empire not at war with order-issuing empire";
         return false;
     }
 
@@ -1024,7 +1133,7 @@ BombardOrder::BombardOrder(int empire, int ship, int planet, const ScriptingCont
 { Check(empire, m_ship, m_planet, context); }
 
 std::string BombardOrder::Dump() const
-{ return UserString("ORDER_BOMBARD"); }
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_BOMBARD")) % m_planet % m_ship) + ExecutedTag(this); }
 
 bool BombardOrder::Check(int empire_id, int ship_id, int planet_id,
                          const ScriptingContext& context)
@@ -1032,6 +1141,11 @@ bool BombardOrder::Check(int empire_id, int ship_id, int planet_id,
     const Universe& universe = context.ContextUniverse();
     const ObjectMap& objects = context.ContextObjects();
 
+    if (empire_id == ALL_EMPIRES) {
+        ErrorLogger() << "BombardOrder::Check() : empire " << empire_id << " is not an empire";
+        return false;
+    }
+
     auto ship = objects.get<Ship>(ship_id);
     if (!ship) {
         ErrorLogger() << "BombardOrder::ExecuteImpl couldn't get ship with id " << ship_id;
@@ -1129,15 +1243,14 @@ bool BombardOrder::UndoImpl(ScriptingContext& context) const {
 ////////////////////////////////////////////////
 // ChangeFocusOrder
 ////////////////////////////////////////////////
-ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, std::string focus,
-                                   const ScriptingContext& context) :
+ChangeFocusOrder::ChangeFocusOrder(int empire, int planet, std::string focus, const ScriptingContext& context) :
     Order(empire),
     m_planet(planet),
     m_focus(std::move(focus))
 { Check(empire, m_planet, m_focus, context); }
 
 std::string ChangeFocusOrder::Dump() const
-{ return UserString("ORDER_FOCUS_CHANGE"); }
+{ return boost::io::str(FlexibleFormat(UserString("ORDER_FOCUS_CHANGE")) % m_planet % m_focus) + ExecutedTag(this); }
 
 bool ChangeFocusOrder::Check(int empire_id, int planet_id, const std::string& focus,
                              const ScriptingContext& context)
@@ -1188,8 +1301,11 @@ PolicyOrder::PolicyOrder(int empire, std::string name, std::string category, boo
     m_adopt(adopt)
 {}
 
-std::string PolicyOrder::Dump() const
-{ return m_adopt ? UserString("ORDER_POLICY_ADOPT") : UserString("ORDER_POLICY_ABANDON"); }
+std::string PolicyOrder::Dump() const {
+    const auto& template_str = m_adopt ? UserString("ORDER_POLICY_ADOPT") : UserString("ORDER_POLICY_ABANDON");
+    return boost::io::str(FlexibleFormat(template_str)
+                          % m_policy_name % m_category % m_slot) + ExecutedTag(this);
+}
 
 void PolicyOrder::ExecuteImpl(ScriptingContext& context) const {
     auto empire = GetValidatedEmpire(context);
@@ -1209,26 +1325,39 @@ void PolicyOrder::ExecuteImpl(ScriptingContext& context) const {
 ////////////////////////////////////////////////
 // ResearchQueueOrder
 ////////////////////////////////////////////////
-ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name) :
+ResearchQueueOrder::ResearchQueueOrder(int empire, std::string tech_name) :
     Order(empire),
-    m_tech_name(tech_name),
+    m_tech_name(std::move(tech_name)),
     m_remove(true)
 {}
 
-ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name, int position) :
+ResearchQueueOrder::ResearchQueueOrder(int empire, std::string tech_name, int position) :
     Order(empire),
-    m_tech_name(tech_name),
+    m_tech_name(std::move(tech_name)),
     m_position(position)
 {}
 
-ResearchQueueOrder::ResearchQueueOrder(int empire, const std::string& tech_name, bool pause, float dummy) :
+ResearchQueueOrder::ResearchQueueOrder(int empire, std::string tech_name, bool pause, float dummy) :
     Order(empire),
-    m_tech_name(tech_name),
+    m_tech_name(std::move(tech_name)),
     m_pause(pause ? PAUSE : RESUME)
 {}
 
-std::string ResearchQueueOrder::Dump() const
-{ return UserString("ORDER_RESEARCH"); }
+std::string ResearchQueueOrder::Dump() const {
+    const auto& template_str = [this]() -> const auto& {
+        if (m_remove)
+            return UserString("ORDER_RESEARCH_REMOVE");
+        else if (m_pause == PAUSE)
+            return UserString("ORDER_RESEARCH_PAUSE");
+        else if (m_pause == RESUME)
+            return UserString("ORDER_RESEARCH_RESUME");
+        else
+            return UserString("ORDER_RESEARCH_ENQUEUE_AT");
+    }();
+
+    const auto& tech_name = UserStringExists(m_tech_name) ? UserString(m_tech_name) : m_tech_name;
+    return boost::io::str(FlexibleFormat(template_str) % tech_name % m_position) + ExecutedTag(this);
+}
 
 void ResearchQueueOrder::ExecuteImpl(ScriptingContext& context) const {
     auto empire = GetValidatedEmpire(context);
@@ -1254,10 +1383,10 @@ void ResearchQueueOrder::ExecuteImpl(ScriptingContext& context) const {
 // ProductionQueueOrder
 ////////////////////////////////////////////////
 ProductionQueueOrder::ProductionQueueOrder(ProdQueueOrderAction action, int empire,
-                                           const ProductionQueue::ProductionItem& item, // TODO: pass by value and move?
+                                           ProductionQueue::ProductionItem item,
                                            int number, int location, int pos) :
     Order(empire),
-    m_item(item),
+    m_item(std::move(item)),
     m_location(location),
     m_new_quantity(number),
     m_new_index(pos),
@@ -1276,6 +1405,7 @@ ProductionQueueOrder::ProductionQueueOrder(ProdQueueOrderAction action, int empi
 {
     switch(m_action) {
     case ProdQueueOrderAction::REMOVE_FROM_QUEUE:
+    case ProdQueueOrderAction::UNREMOVE_FROM_QUEUE:
         break;
     case ProdQueueOrderAction::SPLIT_INCOMPLETE:
     case ProdQueueOrderAction::DUPLICATE_ITEM:
@@ -1326,12 +1456,22 @@ void ProductionQueueOrder::ExecuteImpl(ScriptingContext& context) const {
             break;
         }
         case ProdQueueOrderAction::REMOVE_FROM_QUEUE: {
-            auto idx = empire->GetProductionQueue().IndexOfUUID(m_uuid);
+            const auto idx = empire->GetProductionQueue().IndexOfUUID(m_uuid);
             if (idx == -1) {
                 ErrorLogger() << "ProductionQueueOrder asked to remove invalid UUID: " << boost::uuids::to_string(m_uuid);
             } else {
                 DebugLogger() << "ProductionQueueOrder removing item at index: " << idx;
-                empire->RemoveProductionFromQueue(idx);
+                empire->MarkToBeRemoved(idx);
+            }
+            break;
+        }
+        case ProdQueueOrderAction::UNREMOVE_FROM_QUEUE: {
+            const auto idx = empire->GetProductionQueue().IndexOfUUID(m_uuid);
+            if (idx == -1) {
+                ErrorLogger() << "ProductionQueueOrder asked to unremove invalid UUID: " << boost::uuids::to_string(m_uuid);
+            } else {
+                DebugLogger() << "ProductionQueueOrder unremoving item at index: " << idx;
+                empire->MarkNotToBeRemoved(idx);
             }
             break;
         }
@@ -1505,34 +1645,36 @@ void ShipDesignOrder::ExecuteImpl(ScriptingContext& context) const {
         if (!CheckNew(EmpireID(), m_name, m_description, m_hull, m_parts, context))
             return;
 
-        // TODO: put into unique_ptr and pass as such...
-        ShipDesign* new_ship_design = nullptr;
         try {
-            new_ship_design = new ShipDesign(std::invalid_argument(""), m_name, m_description,
-                                             m_designed_on_turn, EmpireID(), m_hull, m_parts,
-                                             m_icon, m_3D_model, m_name_desc_in_stringtable,
-                                             m_is_monster, m_uuid);
+            ShipDesign new_ship_design(std::invalid_argument(""), m_name, m_description,
+                                       m_designed_on_turn, EmpireID(), m_hull, m_parts,
+                                       m_icon, m_3D_model, m_name_desc_in_stringtable,
+                                       m_is_monster, m_uuid);
+
+            if (m_design_id == INVALID_DESIGN_ID) {
+                // On the client create a new design id
+                m_design_id = universe.InsertShipDesign(std::move(new_ship_design));
+                DebugLogger() << "ShipDesignOrder::ExecuteImpl inserted new ship design ID " << m_design_id;
+
+            } else {
+                // On the server use the design id passed from the client
+                const auto success = universe.InsertShipDesignID(std::move(new_ship_design), EmpireID(),
+                                                                 m_design_id);
+                if (!success) {
+                    ErrorLogger() << "Couldn't insert ship design by ID " << m_design_id;
+                    return;
+                }
+            }
+
+            universe.SetEmpireKnowledgeOfShipDesign(m_design_id, EmpireID());
+            empire->AddShipDesign(m_design_id, universe);
+
+
         } catch (const std::exception& e) {
             ErrorLogger() << "Couldn't create ship design: " << e.what();
             return;
         }
 
-        if (m_design_id == INVALID_DESIGN_ID) {
-            // On the client create a new design id
-            universe.InsertShipDesign(new_ship_design);
-            m_design_id = new_ship_design->ID();
-            DebugLogger() << "ShipDesignOrder::ExecuteImpl inserted new ship design ID " << m_design_id;
-        } else {
-            // On the server use the design id passed from the client
-            if (!universe.InsertShipDesignID(new_ship_design, EmpireID(), m_design_id)) {
-                ErrorLogger() << "Couldn't insert ship design by ID " << m_design_id;
-                return;
-            }
-        }
-
-        universe.SetEmpireKnowledgeOfShipDesign(m_design_id, EmpireID());
-        empire->AddShipDesign(m_design_id, universe);
-
     } else if (m_update_name_or_description) {
         if (!CheckRename(EmpireID(), m_design_id, m_name, m_description, context))
             return;
@@ -1571,7 +1713,7 @@ bool ShipDesignOrder::CheckRemember(int empire_id, int existing_design_id_to_rem
 
     // check if the empire can see any objects that have this design (thus enabling it to be copied)
     auto& empire_known_design_ids = context.ContextUniverse().EmpireKnownShipDesignIDs(empire_id);
-    if (empire_known_design_ids.count(existing_design_id_to_remember) < 1) {
+    if (!empire_known_design_ids.contains(existing_design_id_to_remember)) {
         ErrorLogger() << "Empire " << empire_id
                       << " tried to remember a ShipDesign id = " << existing_design_id_to_remember
                       << " that this empire hasn't seen";
@@ -1625,11 +1767,12 @@ bool ShipDesignOrder::CheckRename(int empire_id, int existing_design_id, const s
     }
 
     const auto& universe = context.ContextUniverse();
-    // check if a design with this ID already exists
-    if (const auto existing = universe.GetShipDesign(existing_design_id)) {
+    // check if a design with this ID exists
+    auto existing = universe.GetShipDesign(existing_design_id);
+    if (!existing) {
         ErrorLogger() << "Empire " << empire_id
-                      << " tried to create a new ShipDesign with an id, " << existing_design_id
-                      << " of an already-existing ShipDesign " << existing->Name();
+                      << " tried to rename a ShipDesign with an id, " << existing_design_id
+                      << " that does not exist";
         return false;
     }
 
