@@ -9,11 +9,16 @@
 #include "../universe/Fleet.h"
 #include "../universe/Universe.h"
 
-#include <boost/lexical_cast.hpp>
+namespace {
+#if defined(__cpp_lib_constexpr_string) && ((!defined(__GNUC__) || (__GNUC__ > 12) || (__GNUC__ == 12 && __GNUC_MINOR__ >= 2))) && ((!defined(_MSC_VER) || (_MSC_VER >= 1934))) && ((!defined(__clang_major__) || (__clang_major__ >= 17)))
+    constexpr std::string EMPTY_STRING;
+#else
+    const std::string EMPTY_STRING;
+#endif
+}
+
 
 SitRepEntry::SitRepEntry() :
-    VarText(),
-    m_turn(INVALID_GAME_TURN),
     m_icon("/icons/sitrep/generic.png")
 {}
 
@@ -32,20 +37,8 @@ SitRepEntry::SitRepEntry(std::string&& template_string, int turn,
     m_label(std::move(label))
 {}
 
-int SitRepEntry::GetDataIDNumber(const std::string& tag) const {
-    auto elem = m_variables.find(tag);
-    try {
-        if (elem != m_variables.end())
-            return boost::lexical_cast<int>(elem->second);
-    } catch (...) {
-        return -1;
-    }
-    return -1;
-}
-
 const std::string& SitRepEntry::GetDataString(const std::string& tag) const {
-    static const std::string EMPTY_STRING;
-    auto elem = m_variables.find(tag);
+    const auto elem = m_variables.find(tag);
     if (elem == m_variables.end())
         return EMPTY_STRING;
     return elem->second;
@@ -61,13 +54,13 @@ std::string SitRepEntry::Dump() const {
     return retval;
 }
 
-SitRepEntry CreateTechResearchedSitRep(const std::string& tech_name, int current_turn) {
+SitRepEntry CreateTechResearchedSitRep(std::string tech_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_TECH_RESEARCHED"),
         current_turn,
         "icons/sitrep/tech_researched.png",
         UserStringNop("SITREP_TECH_RESEARCHED_LABEL"), true);
-    sitrep.AddVariable(VarText::TECH_TAG, tech_name);
+    sitrep.AddVariable(VarText::TECH_TAG, std::move(tech_name));
     return sitrep;
 }
 
@@ -106,53 +99,53 @@ SitRepEntry CreateBuildingBuiltSitRep(int building_id, int planet_id, int curren
     return sitrep;
 }
 
-SitRepEntry CreateTechUnlockedSitRep(const std::string& tech_name, int current_turn) {
+SitRepEntry CreateTechUnlockedSitRep(std::string tech_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_TECH_UNLOCKED"),
         current_turn,
         "icons/sitrep/tech_unlocked.png",
         UserStringNop("SITREP_TECH_UNLOCKED_LABEL"), true);
-    sitrep.AddVariable(VarText::TECH_TAG, tech_name);
+    sitrep.AddVariable(VarText::TECH_TAG, std::move(tech_name));
     return sitrep;
 }
 
-SitRepEntry CreatePolicyUnlockedSitRep(const std::string& policy_name, int current_turn) {
+SitRepEntry CreatePolicyUnlockedSitRep(std::string policy_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_POLICY_UNLOCKED"),
         current_turn + 1,
         "icons/sitrep/policy_unlocked.png",
         UserStringNop("SITREP_POLICY_UNLOCKED_LABEL"), true);
-    sitrep.AddVariable(VarText::POLICY_TAG, policy_name);
+    sitrep.AddVariable(VarText::POLICY_TAG, std::move(policy_name));
     return sitrep;
 }
 
-SitRepEntry CreateBuildingTypeUnlockedSitRep(const std::string& building_type_name, int current_turn) {
+SitRepEntry CreateBuildingTypeUnlockedSitRep(std::string building_type_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_BUILDING_TYPE_UNLOCKED"),
         current_turn,
         "icons/sitrep/building_type_unlocked.png",
         UserStringNop("SITREP_BUILDING_TYPE_UNLOCKED_LABEL"), true);
-    sitrep.AddVariable(VarText::BUILDING_TYPE_TAG, building_type_name);
+    sitrep.AddVariable(VarText::BUILDING_TYPE_TAG, std::move(building_type_name));
     return sitrep;
 }
 
-SitRepEntry CreateShipHullUnlockedSitRep(const std::string& ship_hull_name, int current_turn) {
+SitRepEntry CreateShipHullUnlockedSitRep(std::string ship_hull_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_SHIP_HULL_UNLOCKED"),
         current_turn,
         "icons/sitrep/ship_hull_unlocked.png",
         UserStringNop("SITREP_SHIP_HULL_UNLOCKED_LABEL"), true);
-    sitrep.AddVariable(VarText::SHIP_HULL_TAG, ship_hull_name);
+    sitrep.AddVariable(VarText::SHIP_HULL_TAG, std::move(ship_hull_name));
     return sitrep;
 }
 
-SitRepEntry CreateShipPartUnlockedSitRep(const std::string& ship_part_name, int current_turn) {
+SitRepEntry CreateShipPartUnlockedSitRep(std::string ship_part_name, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_SHIP_PART_UNLOCKED"),
         current_turn,
         "icons/sitrep/ship_part_unlocked.png",
         UserStringNop("SITREP_SHIP_PART_UNLOCKED_LABEL"), true);
-    sitrep.AddVariable(VarText::SHIP_PART_TAG, ship_part_name);
+    sitrep.AddVariable(VarText::SHIP_PART_TAG, std::move(ship_part_name));
     return sitrep;
 }
 
@@ -172,7 +165,7 @@ SitRepEntry CreateCombatSitRep(int system_id, int log_id, int enemy_id, int curr
     return sitrep;
 }
 
-SitRepEntry CreateGroundCombatSitRep(int planet_id, int enemy_id) {
+SitRepEntry CreateGroundCombatSitRep(int planet_id, int enemy_id, int current_turn) {
     std::string template_string = (enemy_id == ALL_EMPIRES)
         ? UserStringNop("SITREP_GROUND_BATTLE")
         : UserStringNop("SITREP_GROUND_BATTLE_ENEMY");
@@ -180,32 +173,32 @@ SitRepEntry CreateGroundCombatSitRep(int planet_id, int enemy_id) {
         ? UserStringNop("SITREP_GROUND_BATTLE_LABEL")
         : UserStringNop("SITREP_GROUND_BATTLE_ENEMY_LABEL");
     SitRepEntry sitrep(
-        std::move(template_string), CurrentTurn() + 1,
+        std::move(template_string), current_turn + 1,
         "icons/sitrep/ground_combat.png", std::move(label_string), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(enemy_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG, std::to_string(planet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(enemy_id));
     return sitrep;
 }
 
-SitRepEntry CreatePlanetCapturedSitRep(int planet_id, int empire_id) {
+SitRepEntry CreatePlanetCapturedSitRep(int planet_id, int empire_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_CAPTURED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/planet_captured.png",
         UserStringNop("SITREP_PLANET_CAPTURED_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(empire_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG, std::to_string(planet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
-SitRepEntry CreatePlanetRebelledSitRep(int planet_id, int empire_id) {
+SitRepEntry CreatePlanetRebelledSitRep(int planet_id, int empire_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_CAPTURED_NEUTRALS"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/planet_captured.png",
         UserStringNop("SITREP_PLANET_CAPTURED_NEUTRALS_LABEL"), true);
-    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
-    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(empire_id));
+    sitrep.AddVariable(VarText::PLANET_ID_TAG, std::to_string(planet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
@@ -236,7 +229,7 @@ SitRepEntry CreateCombatDamagedObjectSitRep(const UniverseObject* obj, int comba
 {
     if (!obj)
         return GenericCombatDamagedObjectSitrep(combat_system_id, current_turn);
-    int object_id = obj->ID();
+    const int object_id = obj->ID();
 
     SitRepEntry sitrep;
 
@@ -288,11 +281,12 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
         DebugLogger() << "CreateCombatDestroyedObjectSitRep: passed null object";
         return GenericCombatDestroyedObjectSitrep(combat_system_id, current_turn);
     }
-    int object_id = obj->ID();
+    const int object_id = obj->ID();
 
     SitRepEntry sitrep;
 
-    if (obj->ObjectType() == UniverseObjectType::OBJ_SHIP) {
+    switch (obj->ObjectType()) {
+    case UniverseObjectType::OBJ_SHIP: {
         auto ship = static_cast<const Ship*>(obj);
         if (ship->Unowned())
             sitrep = SitRepEntry(
@@ -314,8 +308,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
                 UserStringNop("SITREP_SHIP_DESTROYED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::SHIP_ID_TAG,   std::to_string(object_id));
         sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
-
-    } else if (obj->ObjectType() == UniverseObjectType::OBJ_FLEET) {
+        break;
+    }
+    case UniverseObjectType::OBJ_FLEET: {
         auto fleet = static_cast<const Fleet*>(obj);
         if (fleet->Unowned())
             sitrep = SitRepEntry(
@@ -330,8 +325,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_FLEET_DESTROYED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::FLEET_ID_TAG, std::to_string(object_id));
-
-    } else if (obj->ObjectType() == UniverseObjectType::OBJ_PLANET) {
+        break;
+    }
+    case UniverseObjectType::OBJ_PLANET: {
         auto planet = static_cast<const Planet*>(obj);
         if (planet->Unowned())
             sitrep = SitRepEntry(
@@ -346,8 +342,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
                 "icons/sitrep/combat_destroyed.png",
                 UserStringNop("SITREP_PLANET_DESTROYED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::PLANET_ID_TAG, std::to_string(object_id));
-
-    } else if (obj->ObjectType() == UniverseObjectType::OBJ_BUILDING) {
+        break;
+    }
+    case UniverseObjectType::OBJ_BUILDING: {
         auto building = static_cast<const Building*>(obj);
         if (building->Unowned())
             sitrep = SitRepEntry(
@@ -363,8 +360,9 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
                 UserStringNop("SITREP_BUILDING_DESTROYED_ON_PLANET_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::BUILDING_ID_TAG, std::to_string(object_id));
         sitrep.AddVariable(VarText::PLANET_ID_TAG,   std::to_string(building->PlanetID()));
-
-    } else {
+        break;
+    }
+    default:
         sitrep = GenericCombatDestroyedObjectSitrep(combat_system_id, current_turn);
     }
 
@@ -374,41 +372,57 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(const UniverseObject* obj, int com
     return sitrep;
 }
 
-SitRepEntry CreatePlanetDepopulatedSitRep(int planet_id) {
+SitRepEntry CreatePlanetDepopulatedSitRep(int planet_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_DEPOPULATED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/colony_destroyed.png",
         UserStringNop("SITREP_PLANET_DEPOPULATED_LABEL"), true);
     sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
     return sitrep;
 }
 
-SitRepEntry CreatePlanetColonizedSitRep(int planet_id, const std::string& species) {
-    SitRepEntry sitrep(
-        UserStringNop("SITREP_PLANET_COLONIZED"),
-        CurrentTurn() + 1,
-        "icons/sitrep/planet_colonized.png",
-        UserStringNop("SITREP_PLANET_COLONIZED_LABEL"), true);
+SitRepEntry CreatePlanetAnnexedSitRep(int planet_id, int original_owner_id, int annexer_empire_id, int current_turn) {
+    static constexpr std::string_view neutral_annex_txt = UserStringNop("SITREP_PLANET_ANNEXED");
+    static constexpr std::string_view other_empire_annex_txt = UserStringNop("SITREP_PLANET_ANNEXED_FROM_OTHER_EMPIRE");
+    static constexpr std::string_view neutral_annex_label = UserStringNop("SITREP_PLANET_ANNEXED_LABEL");
+    static constexpr std::string_view other_empire_annex_label = UserStringNop("SITREP_PLANET_ANNEXED_FROM_OTHER_EMPIRE_LABEL");
+
+    const auto msg = (original_owner_id == ALL_EMPIRES) ? neutral_annex_txt : other_empire_annex_txt;
+    const auto label = (original_owner_id == ALL_EMPIRES) ? neutral_annex_label : other_empire_annex_label;
+    SitRepEntry sitrep{msg.data(), current_turn + 1, "icons/sitrep/annexed.png", label.data(), true};
     sitrep.AddVariable(VarText::PLANET_ID_TAG,  std::to_string(planet_id));
-    sitrep.AddVariable(VarText::SPECIES_TAG,    species);
+    sitrep.AddVariable("annexer",               std::to_string(annexer_empire_id));
+    if (original_owner_id != ALL_EMPIRES)
+        sitrep.AddVariable("original",          std::to_string(original_owner_id));
     return sitrep;
 }
 
-SitRepEntry CreatePlanetOutpostedSitRep(int planet_id) {
+SitRepEntry CreatePlanetColonizedSitRep(int planet_id, std::string species, int current_turn) {
+    SitRepEntry sitrep(
+        UserStringNop("SITREP_PLANET_COLONIZED"),
+        current_turn + 1,
+        "icons/sitrep/planet_colonized.png",
+        UserStringNop("SITREP_PLANET_COLONIZED_LABEL"), true);
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,  std::to_string(planet_id));
+    sitrep.AddVariable(VarText::SPECIES_TAG,    std::move(species));
+    return sitrep;
+}
+
+SitRepEntry CreatePlanetOutpostedSitRep(int planet_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_OUTPOSTED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/planet_colonized.png",
         UserStringNop("SITREP_PLANET_OUTPOSTED_LABEL"), true);
     sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
     return sitrep;
 }
 
-SitRepEntry CreatePlanetEstablishFailedSitRep(int planet_id, int ship_id) {
+SitRepEntry CreatePlanetEstablishFailedSitRep(int planet_id, int ship_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_ESTABLISH_FAILED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/planet_colonized.png",
         UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_LABEL"), true);
     sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
@@ -416,10 +430,50 @@ SitRepEntry CreatePlanetEstablishFailedSitRep(int planet_id, int ship_id) {
     return sitrep;
 }
 
-SitRepEntry CreatePlanetGiftedSitRep(int planet_id, int empire_id) {
+SitRepEntry CreatePlanetEstablishFailedVisibleOtherSitRep(int planet_id, int ship_id, int other_empire_id,
+                                                          int current_turn)
+{
+    SitRepEntry sitrep(
+        UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_VISIBLE_OTHER"),
+        current_turn + 1,
+        "icons/sitrep/planet_colonized.png",
+        UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_VISIBLE_OTHER_LABEL"), true);
+    sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+    sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(other_empire_id));
+    return sitrep;
+}
+
+SitRepEntry CreatePlanetEstablishFailedArmedSitRep(int planet_id, int ship_id, int other_empire_id,
+                                                   int current_turn)
+{
+    if (other_empire_id == ALL_EMPIRES) {
+        SitRepEntry sitrep(
+            UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_NEUTRAL_ARMED"),
+            current_turn + 1,
+            "icons/sitrep/planet_colonized.png",
+            UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_ARMED_NEUTRAL_LABEL"), true);
+        sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+        sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+        return sitrep;
+
+    } else {
+        SitRepEntry sitrep(
+            UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_ARMED"),
+            current_turn + 1,
+            "icons/sitrep/planet_colonized.png",
+            UserStringNop("SITREP_PLANET_ESTABLISH_FAILED_ARMED_LABEL"), true);
+        sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(planet_id));
+        sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+        sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(other_empire_id));
+        return sitrep;
+    }
+}
+
+SitRepEntry CreatePlanetGiftedSitRep(int planet_id, int empire_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_PLANET_GIFTED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/gift.png",
         UserStringNop("SITREP_PLANET_GIFTED_LABEL"), true);
     sitrep.AddVariable(VarText::PLANET_ID_TAG,  std::to_string(planet_id));
@@ -427,10 +481,10 @@ SitRepEntry CreatePlanetGiftedSitRep(int planet_id, int empire_id) {
     return sitrep;
 }
 
-SitRepEntry CreateFleetGiftedSitRep(int fleet_id, int empire_id) {
+SitRepEntry CreateFleetGiftedSitRep(int fleet_id, int empire_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_FLEET_GIFTED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/gift.png",
         UserStringNop("SITREP_FLEET_GIFTED_LABEL"), true);
     sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
@@ -438,14 +492,13 @@ SitRepEntry CreateFleetGiftedSitRep(int fleet_id, int empire_id) {
     return sitrep;
 }
 
-SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
-                                                  int recipient_empire_id,
+SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, int recipient_empire_id,
                                                   const ScriptingContext& context)
 {
     const ObjectMap& o = context.ContextObjects();
     const Universe& u = context.ContextUniverse();
 
-    auto fleet = o.get<Fleet>(fleet_id);
+    const auto fleet = o.get<Fleet>(fleet_id);
 
     //bool system_contains_recipient_empire_planets = false;
     //if (const System* system = o.get<System>(system_id)) {
@@ -472,6 +525,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
         sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
         sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
         return sitrep;
+
     } else if (fleet->Unowned() && fleet->HasMonsters(u)) {
         if (fleet->NumShips() == 1) {
             SitRepEntry sitrep(
@@ -497,6 +551,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
             sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
             return sitrep;
         }
+
     } else if (fleet->Unowned()) {
         SitRepEntry sitrep(
             UserStringNop("SITREP_FLEET_ARRIVED_AT_DESTINATION"),
@@ -507,6 +562,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
         sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
         sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
         return sitrep;
+
     } else if (fleet->OwnedBy(recipient_empire_id)) {
         if (fleet->NumShips() == 1) {
             SitRepEntry sitrep(
@@ -517,7 +573,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
             sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
             sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
             sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
-            int ship_id = *fleet->ShipIDs().begin();
+            const int ship_id = *fleet->ShipIDs().begin();
             sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
             if (auto ship = o.get<Ship>(ship_id))
                 sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
@@ -534,59 +590,93 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id,
             sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
             return sitrep;
         }
+
+    } else if (fleet->NumShips() == 1) {
+        SitRepEntry sitrep(
+            UserStringNop("SITREP_FOREIGN_SHIP_ARRIVED_AT_DESTINATION"),
+            context.current_turn + 1,
+            "icons/sitrep/fleet_arrived.png",
+            UserStringNop("SITREP_FOREIGN_SHIP_ARRIVED_AT_DESTINATION_LABEL"), true);
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
+        sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
+        sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
+        const int ship_id = *fleet->ShipIDs().begin();
+        sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
+        if (auto ship = o.getRaw<Ship>(ship_id))
+            sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
+        return sitrep;
+
     } else {
-        if (fleet->NumShips() == 1) {
-            SitRepEntry sitrep(
-                UserStringNop("SITREP_FOREIGN_SHIP_ARRIVED_AT_DESTINATION"),
-                context.current_turn + 1,
-                "icons/sitrep/fleet_arrived.png",
-                UserStringNop("SITREP_FOREIGN_SHIP_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,     std::to_string(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
-            int ship_id = *fleet->ShipIDs().begin();
-            sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
-            if (auto ship = o.get<Ship>(ship_id))
-                sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
-            return sitrep;
-        } else {
-            SitRepEntry sitrep(
-                UserStringNop("SITREP_FOREIGN_FLEET_ARRIVED_AT_DESTINATION"),
-                context.current_turn + 1,
-                "icons/sitrep/fleet_arrived.png",
-                UserStringNop("SITREP_FOREIGN_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
-            sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
-            sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
-            sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  std::to_string(fleet->Owner()));
-            sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
-            return sitrep;
-        }
+        SitRepEntry sitrep(
+            UserStringNop("SITREP_FOREIGN_FLEET_ARRIVED_AT_DESTINATION"),
+            context.current_turn + 1,
+            "icons/sitrep/fleet_arrived.png",
+            UserStringNop("SITREP_FOREIGN_FLEET_ARRIVED_AT_DESTINATION_LABEL"), true);
+        sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+        sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(fleet_id));
+        sitrep.AddVariable(VarText::EMPIRE_ID_TAG,  std::to_string(fleet->Owner()));
+        sitrep.AddVariable(VarText::RAW_TEXT_TAG,   std::to_string(fleet->NumShips()));
+        return sitrep;
     }
 }
 
-SitRepEntry CreateEmpireEliminatedSitRep(int empire_id) {
+SitRepEntry CreateFleetBlockadedSitRep(int system_id, int blockaded_fleet_id, int blockaded_empire_id,
+                                       int blockading_empire_id, const ScriptingContext& context)
+{
+    const auto template_label = (blockaded_empire_id != ALL_EMPIRES) ?
+        std::pair{UserStringNop("SITREP_FLEET_BLOCKADED"), UserStringNop("SITREP_FLEET_BLOCKADED_LABEL")} :
+        std::pair{UserStringNop("SITREP_FLEET_BLOCKADED_NEUTRAL"), UserStringNop("SITREP_FLEET_BLOCKADED_LABEL")};
+
+    SitRepEntry sitrep(
+        template_label.first,
+        context.current_turn + 1,
+        "icons/sitrep/blockade.png",
+        template_label.second,
+        true);
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG,  std::to_string(system_id));
+    sitrep.AddVariable(VarText::FLEET_ID_TAG,   std::to_string(blockaded_fleet_id));
+    sitrep.AddVariable("blockaded",             std::to_string(blockaded_empire_id));
+    //if (blockading_empire_id != ALL_EMPIRES)
+        sitrep.AddVariable("blockader",        std::to_string(blockading_empire_id));
+    return sitrep;
+}
+
+SitRepEntry CreateFleetBlockadedSitRep(int system_id, int blockaded_fleet_id,
+                                       int blockaded_empire_id, const ScriptingContext& context)
+{
+    SitRepEntry sitrep(
+        UserStringNop("SITREP_FLEET_BLOCKADED_NO_EMPIRE"),
+        context.current_turn + 1,
+        "icons/sitrep/blockade.png",
+        UserStringNop("SITREP_FLEET_BLOCKADED_NO_EMPIRE_LABEL"), true);
+    sitrep.AddVariable(VarText::SYSTEM_ID_TAG, std::to_string(system_id));
+    sitrep.AddVariable(VarText::FLEET_ID_TAG,  std::to_string(blockaded_fleet_id));
+    sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(blockaded_empire_id));
+    return sitrep;
+}
+
+SitRepEntry CreateEmpireEliminatedSitRep(int empire_id, int current_turn) {
     SitRepEntry sitrep(
         UserStringNop("SITREP_EMPIRE_ELIMINATED"),
-        CurrentTurn() + 1,
+        current_turn + 1,
         "icons/sitrep/empire_eliminated.png",
         UserStringNop("SITREP_EMPIRE_ELIMINATED_LABEL"), true);
     sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
-SitRepEntry CreateVictorySitRep(const std::string& reason_string, int empire_id) {
-    SitRepEntry sitrep(reason_string.c_str(), CurrentTurn() + 1,
+SitRepEntry CreateVictorySitRep(std::string reason_string, int empire_id, int current_turn) {
+    SitRepEntry sitrep(std::move(reason_string), current_turn + 1,
                        "icons/sitrep/victory.png", UserStringNop("SITREP_VICTORY_LABEL"), true);
     sitrep.AddVariable(VarText::EMPIRE_ID_TAG, std::to_string(empire_id));
     return sitrep;
 }
 
-SitRepEntry CreateSitRep(const std::string& template_string, int turn, const std::string& icon,
+SitRepEntry CreateSitRep(std::string template_string, int turn, std::string icon,
                          std::vector<std::pair<std::string, std::string>> parameters,
-                         const std::string& label, bool stringtable_lookup)
+                         std::string label, bool stringtable_lookup)
 {
-    SitRepEntry sitrep(template_string.c_str(), turn, icon.c_str(),
-                       label.c_str(), stringtable_lookup);
+    SitRepEntry sitrep(std::move(template_string), turn, std::move(icon), std::move(label), stringtable_lookup);
     sitrep.AddVariables(std::move(parameters));
     return sitrep;
 }
